unit K_Params;

{$mode delphi}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  StdCtrls, Buttons, ComCtrls, Spin, IniFiles, Math
  {$ifdef mswindows}
  , Registry
  {$endif}
  ;

{TKparams}

type
  TKparams = class(TForm)
    Panel1: TPanel;
    Next: TButton;
    Back: TButton;
    Execute: TButton;
    cbKF: TComboBox;
    Image1: TImage;
    Label1: TLabel;
    Label2: TLabel;
    Image2: TImage;
    Edit1: TEdit;
    Edit2: TEdit;
    Edit3: TEdit;
    Label4: TLabel;
    BitBtn1: TBitBtn;
    StatusBar1: TStatusBar;
    Label5: TLabel;
    Edit4: TEdit;
    Panel2: TPanel;
    Label6: TLabel;
    CheckBox1: TCheckBox;
    Label7: TLabel;
    CheckBox2: TCheckBox;
    CheckBox3: TCheckBox;
    seLayerNumber: TSpinEdit;
    procedure ExecuteClick(Sender: TObject);
    procedure BackClick(Sender: TObject);
    procedure NextClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cbKFClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure BitBtn1Click(Sender: TObject);
    procedure ed_NumLayerChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure seLayerNumberChange(Sender: TObject);
  private
    procedure ShowEquation;
    procedure GetParameters;
    procedure Language;
  public
    ExeDir, Slash, MyDocDir: String;
    t_unit: Integer;
    FitMode, ShowMode, dialogbox, KsatFix, akFix, bkFix, UseOutFlowForK: Shortint;
    {
    Ksat, ak, bk, ck, mp1, Wsat, Wr, Tref : array[1..5] of Double;
    Modified, Mode, NumLayer, Jpn: Shortint;
    IPET_type, Independent: Shortint;
    SoilName: array[1..5] of String;
    KF, RF: array[1..5] of Shortint;
    }
    GoAhead: Boolean;
    procedure SetProperty_of_HyConCurve(layer: Integer; FileName: String; ExecXYGrapher: Boolean);
   // function HydraulicConductivity(W, T: Double; j: Integer): Double;
  end;

var
  Kparams: TKparams;
  j: Shortint;
  AllowModified: Boolean;
  Msg: Array[1..20] of String;

implementation

uses FigFileMaker, RC_params;

{TKparams}
//==========================================================================

procedure TKparams.FormCreate(Sender: TObject);
{$ifdef MSWINDOWS}
var
  Reg: TRegistry;
{$endif}
begin
  AllowModified := False;
  ExeDir := ExtractFileDir(Application.Exename);
  ShowMode := 0; FitMode := 1;
  t_unit := 3600;
  KsatFix := 1; akFix := 1; bkFix := 0; UseOutFlowForK := 1;// Independent := 1;
  {
  Mode := 1; NumLayer := 1;
  IPET_type := 1;
  SoilName[1] := '';
  KF[1] := 1;
  Wsat[1] := 0.4; Wr[1] := 0.1; Tref[1] := 298.2;
  mp1[1] := 0.3333;
  Ksat[1] := 0.1; ak[1] := 1; bk[1] := 1; ck[1] := 1;
  Independent := 1;
  Jpn := 1;
  GoAhead:= True;
  }
  {$ifdef Linux}
  MyDocDir := GetEnvironmentVariable('HOME');
  Slash := '/';
  {$else}
  Reg := TRegistry.Create;
  with Reg do
  try
    RootKey := HKEY_CURRENT_USER;
    OpenKey('Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders', False);
    MyDocDir := ReadString('Personal') + '\';// lpszPath; // Modified 02/01/05
    CloseKey;
  finally
    Free;
  end;
  Slash := '\';
  {$endif}
end;
//------------------------------------------------------

procedure TKparams.ShowEquation;
var S: String;
begin
  Panel2.Top := 210;
  Edit4.Visible := False;
  cbKF.itemindex := Soil.KF[j] - 1;
  CheckBox3.Enabled := False;
  case Soil.KF[j] of
  1:begin
      S := 'modified Campbell0.png';
      Edit4.Visible := True;
      Panel2.Top := 230;
      CheckBox3.Enabled := True;
    end;
  2:S := 'Fujimaki.png';
  3:begin
      S := 'Mualem.png';
     // if Soil.bk[j] = 0 then
     //   Soil.ak[j]:= 0.5;
      CheckBox3.Checked := False;
    end;
  4:begin
      S := 'Shiozawa.png';
      Edit4.Visible := True;
      Panel2.Top := 230;
    end;
  end;
  S := ExeDir + Slash + 'Equations' + Slash + S;
  if FileExists(S) then
    Image1.picture.loadfromfile(S);
  S := ExeDir + Slash + 'Equations' + Slash + 'mC params0.png';
  if FileExists(S) then
    image2.picture.loadfromfile(S);
  Edit1.Text := Copy(FloatToStr(Soil.Ksat[j] * t_unit), 1, 8);
  Edit2.Text := Copy(FloatToStr(Soil.ak[j]), 1, 8);
  Edit3.Text := Copy(FloatToStr(Soil.bk[j]), 1, 8);
  Edit4.Text := Copy(FloatToStr(Soil.ck[j]), 1, 8);
  if (Soil.Mode = 2) and (Soil.RF[j] = 2) and (Soil.KF[j] = 3) then
  begin
  //  if Edit3.Text = '0' then
    Edit3.Text := FloatToStr(Soil.mp1[j]);
    Edit3.Enabled := False;
    Edit4.Enabled := False;
  end
  else
  begin
    Edit3.Enabled := True;
    Edit4.Enabled := True;
  end;
end;
//--------------------------------------------------------------------
procedure TKparams.cbKFClick(Sender: TObject);
begin
  Soil.KF[j] := cbKF.itemindex + 1;
  ShowEquation;
end;
//----------------------------------------------------------
procedure TKparams.BitBtn1Click(Sender: TObject);
var i: Integer;
begin
  GetParameters;
  if Soil.Mode = 2 then
    i := 1
  else
    i := j;
  SetProperty_of_HyConCurve(i, Soil.CaseName + '_K_Curve', True);
end;
//----------------------------------------------------------
procedure TKparams.GetParameters;
begin
  begin
    Soil.Ksat[j] := StrToFloat(Edit1.Text)/ t_unit;
    Soil.ak[j] := StrToFloat(Edit2.Text);
    Soil.bk[j] := StrToFloat(Edit3.Text);
    Soil.ck[j] := StrToFloat(Edit4.Text);
  end;
end;
//----------------------------------------------------------
procedure TKparams.ExecuteClick(Sender: TObject);
begin
  dialogbox := 0;
  Close;
end;
//------------------------------------------------------
procedure TKparams.BackClick(Sender: TObject);
begin
  if FitMode = 2 then
    BitBtn1Click(Sender)
  else
  begin
    GoAhead := False;
    dialogbox := 3;
    Close;
  end;
end;
//------------------------------------------------------
procedure TKparams.NextClick(Sender: TObject);
begin
  if FitMode = 2 then
  begin
    {$ifdef Linux}
    //Printer.ExecuteSetup;
    {$else}
   // if PrintDialog.Execute then
   //   Print;
    {$endif}
  end
  else
  begin
    GoAhead:=True;
    dialogbox:=5;
    Close;
  end;
end;
//--------------------------------------------------------------------
procedure TKparams.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  
  begin
    if Soil.Mode = 2 then
    begin
      if CheckBox1.Checked = True then
        KsatFix  := 1
      else
        KsatFix  := 0;
      if CheckBox2.Checked = True then
        akFix := 1
      else
        akFix := 0;
      if (CheckBox3.Enabled) and (CheckBox3.Checked = True) then
        bkFix := 1
      else
        bkFix := 0;
    end;
    GetParameters;
    //--- Parameter Check ---//
    Case Soil.KF[j] of
      1:;
      2:;
      3: if Soil.bk[j] <= 0 then begin ShowMessage('"bk" value must be larger than 0.'); dialogbox := 4; end;
      4:;
    end;
  end;
end;
//----------------------------------------------------------
procedure TKparams.ed_NumLayerChange(Sender: TObject);
begin
end;
//===============================================================================================================================
procedure TKparams.Language;
var
  IniFile: TIniFile;
  S, ResourceFileName: String;
  i: Integer;
begin
  Msg[1] := '&Execute';
  Msg[2] := '&Next >>';
  Msg[3] := '<< &Back';
  Msg[4] := 'Results of parameter estimation';
  Msg[5] := '&Figure';
  Msg[6] := '&O K';
  Msg[7] := '&Print';
  Msg[8] := '&Cancel';
  Msg[9] := 'Unsaturated hydraulic conductivity of ';
  Msg[10]:= '  V.W.C.     unsat.H.C.(cm/s) ';
  Msg[11]:= '[r][b]Fig.2[ ]  Unsaturated hydraulic conductivity of ';
  Msg[12]:= '\n';
  Msg[13]:= 'Volumetric water content [s]q[a] (cm[^]3[-]/cm[^]3[-])\n';
  Msg[14]:= 'Unsaturated hydraulic conductivity [r][i]K[ ][a](cm/s)\n';
  S := Self.Name;
  ResourceFileName := ChangeFileExt(Application.ExeName, '.lng');
  Font.Assign(Soil.Font);
  if FileExists(ResourceFileName) then
  begin
    IniFile := TIniFile.Create(ResourceFileName);
    with IniFile do
      try
        Caption                  := ReadString(S, 'Caption', '');
        Label2.Caption           := ReadString(S, 'Label2.Caption', '');
        Label4.Caption           := ReadString(S, 'Label4.Caption', '');
        Label5.Caption           := ReadString(S, 'Label5.Caption', '');
        Label6.Caption           := ReadString(S, 'Label6.Caption', '');
        Label7.Caption           := ReadString(S, 'Label7.Caption', '');
        Panel1.Caption           := ReadString(S, 'Panel1.Caption', '');
        CheckBox2.Caption        := ReadString(S, 'CheckBox2.Caption', '');
        CheckBox1.Hint           := ReadString(S, 'CheckBox1.Hint', '');
        i := cbKF.ItemIndex;
        cbKF.Items.Text    := StringReplace(ReadString(S, 'cbKF.Items.Text', ''), '<br>', #13#10, [rfReplaceAll, rfIgnoreCase]);
        cbKF.ItemIndex := i;
        for i := 1 to 14 do
          Msg[i] := StringReplace(ReadString(S, 'Msg' + IntToStr(i), ''), '<br>', #13#10, [rfReplaceAll, rfIgnoreCase]);
      finally
        Free;
      end;
  end;
  if Soil.LANG = 128 then
    Label2.Top := 123;
  Case ShowMode of
  0:begin
      Execute.Caption  := Msg[1];
      Next.Caption     := Msg[2];
      Back.Caption     := Msg[3];
    end;
  1:begin
      StatusBar1.SimpleText := Msg[4];
      Back.Caption     := Msg[5];
      Execute.Caption  := Msg[6];
      Next.Caption     := Msg[7];
    end;
  2:begin
      Back.Caption     := Msg[5];
      Execute.Caption  := Msg[6];
      Next.Caption     := Msg[8];
    end
  end;
  Case t_unit of
      1: S := 's';
     60: S := 'min';
   3600: S := 'h';
  86400: S := 'd';
  end;
  Label2.Caption := Label2.Caption + S + ')';
  Label4.Caption := 'cm/' + S;
end;

procedure TKparams.FormShow(Sender: TObject);
begin
  dialogbox := -1;
  StatusBar1.Visible := False;
  seLayerNumber.Value := 1;
  j := 1;
  seLayerNumber.MaxValue := Soil.NumLayer;
  if Soil.NumLayer = 1 then
  begin
    Label6.Visible := False;
    seLayerNumber.Visible := False;
    Label5.Top := 30;
    cbKF.Top := 46;
  end
  else
  begin
    Label5.Top := 48;
    cbKF.Top := 64;
    end;
  if (Soil.SoilName[1] = '') and (Soil.Mode = 2) then
  begin
    ShowMessage('Please input initial value of parameters to be estimated');
    StatusBar1.Visible := True;
    StatusBar1.SimpleText := 'Initial values of parameters to be estimated';
  end;
  if FitMode = 2 then
  begin
    cbKF.Enabled  := False;
    Edit1.Enabled := False;
    Edit2.Enabled := False;
    Edit3.Enabled := False;
    Edit4.Enabled := False;
    Execute.TabOrder := 0;
    StatusBar1.Visible := True;
    StatusBar1.SimpleText := 'Result of Inverse Parameter Estimation';
  end;
  if Soil.Mode = 2 then
  begin
    Label7.Visible    := True;
    CheckBox1.Visible := True;
    if KsatFix        = 1 then CheckBox1.Checked := True;
    if (Soil.IPET_type = 1) and (UseOutFlowForK = 0) then
      CheckBox1.Enabled := False;
    if akFix = 1 then
      CheckBox2.Checked := True;
    if bkFix = 1 then
      CheckBox3.Checked := True;
  end
  else
  begin
    Label7.Visible    := False;
    CheckBox1.Visible := False;
    CheckBox2.Visible := False;
    CheckBox3.Visible := False;
  end;
  if Soil.Mode = 2 then
    if (Soil.IPET_type <> 1) and (Soil.IPET_type <> 5) and (Soil.Independent <> 1) and (Soil.IPET_type < 7) then
    begin
      Soil.KF[1] := 3;
      cbKF.Enabled := False;
      Soil.bk[1]  := Soil.mp1[1];
      //if FitMode <> 2 then
      //  Soil.ak[1] := 0.5;
      Edit3.Enabled := False;
      //ShowMessage('Only van Genuchten - Mualem soil hydraulic relationship is allowed for simultaneous estimation of hydraulic properties.');
      if Soil.IPET_type = 4 then
        Kparams.cbKF.ItemIndex := 2;
    end;
  ShowEquation;
  Language;
  AllowModified := True;
end;

procedure TKparams.SetProperty_of_HyConCurve(layer: Integer; FileName: String; ExecXYGrapher: Boolean);
var
  i : Integer;
  f : TextFile;
  S: String;
  GX, GY: array[0..50] of Double;
  IniFile: TIniFile;
  HyConCurveFig : TXYGrapher;
begin
  IniFile := TIniFile.Create(MyDocDir + Slash + '.XYGrapher.ini');
  try
    IniFile.WriteInteger('Options', 'Height', 550);
    IniFile.WriteInteger('Options', 'Width' , 500);
  finally
    IniFile.Free;
  end;
  //--- Saving of Fitted Curve ---//
    j := layer;
    with Soil do
    begin
      for i := 1 to 50 do
      begin
        if KF[1] <> 2 then
          GX[i] := (Wsat[1] - Wr[1]) * i / 50 + Wr[1]
        else
          if bk[1] < 0 then
            GX[i] := (Wsat[1] + bk[1] * Wsat[1]) * i / 50 - bk[1] * Wsat[1]
          else
            GX[i] := Wsat[1] * i / 50;
        GY[i] := HydraulicConductivity(GX[i], Tref[layer], layer) / t_unit;
      end;
    S := BaseDir + Slash + SoilName[layer];
//    S := ExeDir + Slash + 'SoilProperties' + Slash + SoilName[layer];
    if DirectoryExists(S) = False then
      CreateDir(S);
    AssignFile(f, S + Slash + SoilName[layer] + '_uhc.txt');
    Rewrite(f);
    if LANG = 128 then
      Writeln(f, SoilName[j] + Msg[9])
    else
      Writeln(f, Msg[9] + SoilName[j]);
    Writeln(f, Kparams.cbKF.Items[KF[j]-1]);
    if (KF[j] = 2) or (KF[j] = 3) then
    begin
      Writeln(f, '  Ksat       ak       bk');
      Writeln(f, ' '+FloatToStr(Ksat[j])+'    '+FloatToStr(ak[j])+'    '+FloatToStr(bk[j]));
    end
    else
    begin
      Writeln(f, '  Ksat       ak       bk      ck');
      Writeln(f, ' '+FloatToStr(Ksat[j])+'    '+FloatToStr(ak[j])+'    '+FloatToStr(bk[j])+'    '+FloatToStr(ck[j]));
    end;
    Writeln(f, Msg[10]);
    for i := 1 to 50 do
      if GY[i] > 0.0000000000000001 then
        Writeln(f, '  ' + FormatFloat('0.0000', GX[i]) + '      ' + FormatFloat('0.0000000000000000', GY[i]));
    CloseFile(f);
    end;
    if ExecXYGrapher then
    begin
      HyConCurveFig := TXYGrapher.Create;
      //---------------------------------
      with HyConCurveFig do
      begin
        NumDataSeries := 1;
        //----------
        DataSeries[1] := S + Slash + Soil.SoilName[layer] + '_uhc.txt';
        Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 6; ReadEnd[1] := 0;
        LineStyle[1] := 1; LineWidth[1] := 0.25; LineColor[1] := 12615680; MarkStyle[1] := 0; MarkSize[1] := 0;
        Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
        //----------
        Height := 65; Width := 65; Lmargin := 30; Tmargin := 20; FLineWidth := 0.4; NumFontSize := 9;
        NumFontName := 'Arial';
        i := Trunc(Soil.Wsat[j] * 10) + 1;
        Max[1] := 0.1 * i; Min[1] := 0 ; Division[1] := i; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
        Max[2] := 0.1 * i; Min[2] := 0 ; Division[2] := i; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
        Max[3] := 0.01; Min[3] := 1E-12; Division[3] := 5; Minordivision[3] := 0; logarithm[3] := 1; NumeralsXshift[3] := -2; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
        Max[4] := 0.01; Min[4] := 1E-12; Division[4] := 5; Minordivision[4] := 0; logarithm[4] := 1; NumeralsXshift[4] := -2; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
        NumLegend := 5;
        Legend[1] := Msg[11] + Soil.SoilName[j] + Msg[12];
        Legend[2] := Msg[13];
        Legend[3] := '';
        Legend[4] := Msg[14];
        Legend[5] := '';
        LegendLspacing[1] :=4.5; LegendXshift[1] := 0; LegendYshift[1] := 0;
        LegendLspacing[2] :=  0; LegendXshift[2] := 0; LegendYshift[2] := 0;
        LegendLspacing[3] :=  0; LegendXshift[3] := 0; LegendYshift[3] := 0;
        LegendLspacing[4] :=  0; LegendXshift[4] := 0; LegendYshift[4] := 0;
        LegendLspacing[5] :=  0; LegendXshift[5] := 0; LegendYshift[5] := 0;
        SaveFigFile(FileName);
        ExecuteXYGrapher(FileName);
      end;
    HyConCurveFig.Free;
  end;
end;

procedure TKparams.seLayerNumberChange(Sender: TObject);
begin
  if AllowModified then
    GetParameters;
  j := seLayerNumber.Value;
  ShowEquation;
end;

initialization
  {$I K_Params.lrs}


end.
