
unit Output;

interface

{$mode delphi}{$H+}
uses 
  LResources,Buttons,
{$ifdef Linux}
  unix,
{$else}
  Registry,
{$endif}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Math, IniFiles;

type
  TOutputResults = class(TComponent)
  private

  public
    Timer1: TTimer;
    constructor Create(AOwner: TComponent); override;
    function NextValue: Double;
    procedure UpdateClock(Sender: TObject);
    procedure ConvertToRate;
    procedure PrepareAnime;
    procedure Play_or_StopClick;
    procedure RewindClick;
    procedure FastForwardClick;
    procedure tbSaveProfileClick;
    procedure SliceRings;
    procedure DisplayIt(Sender: TObject);
    procedure StepForward(Sender: TObject);
    procedure StepBackward(Sender: TObject);
    procedure SetProperty_of_TSM_ObservationFigure(FileName: String);
    procedure SetProperty_of_MSO_ObservationFigure(FileName: String);
    procedure SetProperty_of_MSI_ObservationFigure(FileName: String);
    procedure SetProperty_of_UIM_ObservationFigure(FileName: String);
    procedure SetProperty_of_EM_ObservationFigure(FileName: String);
    procedure SetProperty_of_TherConCurve(layer: Integer; FileName: String);
    procedure SetProperty_of_AlbedoCurve(layer: Integer; FileName: String);
    procedure SetProperty_of_EnCurve(layer: Integer; FileName: String);
    procedure SetProperty_of_ProfileFigure(FileName: String);
    procedure SetProperty_of_ReductionFunction(FileName: String);
    procedure SetProperty_of_RootDensityDistribution(FileName: String);
    procedure SetProperty_of_RootDepth_and_LAI(FileName: String);
    procedure SetProperty_of_CropCoefficient(FileName: String);

end;

var
  Msg: array[1..40] of String;

implementation

uses
 WASH1dMain, Simulation, RC_Params, K_Params, FigFileMaker;

constructor TOutputResults.Create(AOwner: TComponent);
var
  IniFile: TIniFile;
  ResourceFileName: String;
  i: Integer;
begin
  inherited Create(AOwner);
  DemonStration := False;
  Timer1 := TTimer.Create(self);
  Timer1.Enabled := False;
  Timer1.OnTimer := UpdateClock;
  Timer1.Interval := 500;
  Msg[1] := '[r][b]Fig.1[ ] Comparison of observed and optimized\n         [r]outflow and pressure head\n';
  Msg[2] := 'Time(h)\n';
  Msg[3] := '';
  Msg[4] := 'Cumulative outflow  [r][i]Q[ ][a](cm)\n';
  Msg[5] := 'Pressure head [s]y[a](cm)\n';
  Msg[6] := '%1 [1]Observed\n%2 [1]Optimized\n%3 [1][s]y[a] at the bottom\n';
  Msg[7] := '[r][b]Fig.1[ ] Comparison of observed and optimized inflow\n';
  Msg[8] := 'Time(h)\n';
  Msg[9] := '';
  Msg[10]:= 'Cumulative inflow  [r][i]Q[ ][a](cm)\n';
  Msg[11]:= 'Pressure head [s]y[a](cm)\n';
  Msg[12]:= '%1 [1]Observed\n%2 [1]Optimized\n%3 [1][s]y[a] at the bottom\n';
  Msg[13]:= '[r][b]Fig.1[ ] Comparison of observed and optimized inflow amount\n';
  Msg[14]:= 'Time(h)\n';
  Msg[15]:= '';
  Msg[16]:= 'Cumulative inflow  [r][i]Q[ ][a](cm)\n';
  Msg[17]:= 'Depth of infiltrating front (cm)\n';
  Msg[18]:= '%1 [1]Observed\n%2 [1]Optimized\n%3 Depth of front\n';
  Msg[19]:= 'Thermal conductivity of ';
  Msg[20]:= '  V.W.C.     Thermal Cond.(W/cm/K) ';
  Msg[21]:= '[r][b]Fig._[ ]  Thermal conductivity of ';
  Msg[22]:= 'Degree of saturation [s]q/q[a][_]s[-][a]\n';
  Msg[23]:= '';
  Msg[24]:= 'Thermal conductivity [r][i]K[_]h[-][ ][a](W/cm/K)\n';
  Msg[25]:= '';
  Msg[26]:= 'Dependence of albedo on water content of ';
  Msg[27]:= '  V.W.C.     Albedo';
  Msg[28]:= '[r][b]Fig._[ ]  Thermal conductivity of ';
  Msg[29]:= 'Volumetric water content [s]q[a] (cm[^]3[-]/cm[^]3[-])\n';
  Msg[30]:= '';
  Msg[31]:= 'Albedo  [s]a\n';
  Msg[32]:= '';
  Msg[33]:= '[r][b]Fig._[ ]  Dependence of enhancement factor for thermal vapor diffusion\n[r]          of ' + SoilName[j] + '\n';
  Msg[34]:= 'Degree of saturation [s]q/q[a][_][i]sat\n';
  Msg[35]:= '';
  Msg[36]:= 'Enhancement factor  [s]h\n';
  Msg[37]:= '';
  ResourceFileName := ChangeFileExt(Application.ExeName, '.lng');
  if FileExists(ResourceFileName) then
  begin
    IniFile := TIniFile.Create(ResourceFileName);
    with IniFile do
      try
        for i := 1 to 37 do
          Msg[i] := StringReplace(ReadString('OutputResults', 'Msg' + IntToStr(i), ''), '<br>', #13#10, [rfReplaceAll, rfIgnoreCase]);
      finally
        Free;
      end;
  end;
end;
//----------------------------------------------------------
procedure TOutputResults.ConvertToRate;
var
  i, i2: Integer;
  SL, SL1 : TStringList;
  ql_top: array[0..500] of Single;
begin
  SL1 := TStringList.Create;
  i2 := 0;
  try
    SL1.LoadFromFile(CaseName + '_flux.txt');
    for i := 3 to SL1.Count do
    begin
      S := Main.ConvertDecimalSeparator(SL1[i-1]);
      try
        time[i2] := NextValue;
        CumE[i2] := NextValue;
        CumT[i2] := NextValue;
        CumD[i2] := NextValue;
        CumI[i2] := NextValue;
        Front[i] := NextValue;
        Cumqs[i2]:= NextValue;
        Vr[i2]   := NextValue;
        i2 := i2 + 1;
      except
      end;
    end;
  finally
    SL1.Free;
  end;
  i2 := Min(i2, 500);
  for i := 1 to i2 do
  begin
    Evap[i]   := (CumE[i] - CumE[i-1]) / (time[i] - time[i-1]) * 864000 / t_unit;
    ql_bot[i] := - (CumD[i] - CumD[i-1]) / (time[i] - time[i-1]) * 864000 / t_unit;
    ql_top[i] := - (CumI[i] - CumI[i-1]) / (time[i] - time[i-1]) * 864000 / t_unit;
  end;
  if JustLoading <> 1 then
  begin
    dumint := i2;
    SL := TStringList.Create;
    with SL do
      try
        Add('>>>>>>>>>> Time Change of Fluxes <<<<<<<<<<<') ;
        Add('time, Evaporation rate, Outflow rate');
        for i := 1 to dumint-1 do
          Add(Simulator.SetLengthRJ(FormatFloat('######0.000', (time[i] + time[i-1]) / 2), 11) +Separator+
              Simulator.SetLengthRJ(FormatFloat('#0.00', Evap[i]) , 6) + Separator +
              Simulator.SetLengthRJ(FormatFloat('###0.00', ql_bot[i]), 8) + Separator +
              Simulator.SetLengthRJ(FormatFloat('####0.00', ql_top[i]), 8));
        SaveToFile(CaseName + '_rates.txt' );
      finally
        SL.Free;
      end;
  end;
end;
//====================  Animation Module =====================================================
procedure TOutputResults.PrepareAnime;
var
  i, i2: Integer;
  SL1: TStringList;
begin
  {$ifdef Linux}
  with ZipMaster do
  begin
    DateSeparator := '-';
    ZipFileName := CaseName + '_history.zip';
    DateSeparator := '/';
    FSpecArgs.Clear;
    FSpecArgs.Add('*.*');
    ExtrBaseDir := BaseDir + Slash + 'History';
    if FileExists(ZipFileName) then
      Extract;
    DeleteFile(ZipFileName);
  end;
  {$endif}
  if FileExists(BaseDir + Slash + 'History' + Slash + ExtractFileName(CaseName) + '_001.txt') = True then
  begin
    JustLoading := 1;
    if counter = 0 then
    begin
      // loading of data at the 1st observation point //
      SL1 := TStringList.Create;
      i2 := 0;
      try
        SL1.LoadFromFile(CaseName + '_observation1.txt');
        for i := 4 to SL1.Count do
        begin
          S := Main.ConvertDecimalSeparator(SL1[i-1]);
          try
            time[i2]  := NextValue;
            Wtime[1,i2] := NextValue;
            Ptime[1,i2] := NextValue;
            Ctime[1,i2] := NextValue;
            Ttime[1,i2] := NextValue;
            AvgWaterContentAbove[1,i2] := NextValue;
            i2 := i2 + 1;
          except
          end;
        end;
      finally
        SL1.Free;
      end;
      ConvertToRate;
      //---------------------------------------------//
      Simulator.DetermineSpaceIncrements;
    end
    else
      counter := 0;
    if ((Mode = 2) and (IPET_Type <> 4) and (IPET_Type <> 5)) or (l_bc_type = 4) then
    begin
      SL1 := TStringList.Create;
      i2 := 0;
      try
        SL1.LoadFromFile(fl_p_bot);
        for i := 1 to SL1.Count do
        begin
          S := Main.ConvertDecimalSeparator(SL1[i-1]);
          try
            t_p_bot[i2] := NextValue;
            Pbot[i2]    := NextValue;
            i2 := i2 + 1;
          except
          end;
        end;
      finally
        SL1.Free;
      end;
      NumPbotData := i2 - 1;
    end;
    Main.Play_or_Stop.Enabled := True;
    Main.Rewind.Enabled       := True;
    Main.FastForward.Enabled  := True;
    Main.tbSaveProfile.Enabled := True;
    Main.AnimeCounter.Color := clWindow;
    Main.PlayorStop2.Enabled := True;
    Main.Slow2.Enabled         := True;
    Main.Rewind2.Enabled       := True;
    Main.FastForward2.Enabled  := True;
    Main.StatusBar1.SimpleText := '  ' + comment;
  end;
end;
//----------------------------------------------------------
procedure TOutputResults.Play_or_StopClick;
begin
  GoAhead := True; Fast := 0;
  Timer1.Interval := 250;
  if Main.Play_or_Stop.Caption = '|>' then
  begin
    playing := True;
    Main.Play_or_Stop.Caption := '[]';
    Main.tbSaveProfile.Enabled  := False;
    Main.tbStepForward.Enabled  := False;
    Main.tbStepBackward.Enabled := False;
    Main.Slow.Enabled := True;
    Timer1.Enabled := True;
    ShortDateFormat := 'yyyy/m/d';
    DateSeparator := '/';
  end
  else
  begin
    playing := False;
    Main.Play_or_Stop.Caption := '|>';
    Main.tbSaveProfile.Enabled  := True;
    Main.tbStepForward.Enabled  := True;
    Main.tbStepBackward.Enabled := True;
    Main.Slow.Enabled := False;
    Timer1.Enabled := False;
  end;
end;
//----------------------------------------------------------
procedure TOutputResults.RewindClick;
begin
  Timer1.Enabled := True;
  if playing then
  begin
    if GoAhead = False then
      Timer1.Interval := Timer1.Interval div 2;
  end
  else
  begin
    Play_or_StopClick;
    Fast := 1;
    Timer1.Interval := 100;
  end;
  GoAhead := False;
end;
//----------------------------------------------------------
procedure TOutputResults.FastForwardClick;
begin
  Timer1.Enabled := True;
  if playing then
  begin
    if GoAhead = True then
      Timer1.Interval := Timer1.Interval div 2;
  end
  else
  begin
    Play_or_StopClick;
    Fast := 1;
    Timer1.Interval := 100;
  end;
  GoAhead := True;
end;
//----------------------------------------------------------
procedure TOutputResults.tbSaveProfileClick;
var
  SL: TStringList;
  S: String;
begin
  SL := TStringList.Create;
  try
    S := ExtractFileName(CaseName) + '_' + FormatFloat('000', counter) + '.txt';
    SL.LoadFromFile(BaseDir + Slash + 'History' + Slash + S);
    SL.SaveToFile(BaseDir + Slash + S);
    ShowMessage('"' + S + '" has been copied to ' + #13#10 + BaseDir);
  finally
    SL.Free;
  end;
end;
//----------------------------------------------------------
procedure TOutputResults.UpdateClock(Sender: TObject);
begin
  Application.ProcessMessages;
  if Fast = 1 then
    dumint := 4
  else
    dumint := 1;
  if GoAhead = True then
  begin
    if (counter + 1 > t_end / int_rec) and (t_end > 0) then
      if DemonStration = True then
        counter := 0
      else
      begin
        Main.Play_or_StopClick(Sender);
        Exit;
      end;
    counter := counter + dumint;
  end
  else
  begin
    if counter = 0 then
      Play_or_StopClick;
    counter := counter - dumint;
    if counter <= 0 then
      counter := 0;
  end;
  if (Playing) or (counter = 0) then
    DisplayIt(Sender);
end;

procedure TOutputResults.StepForward(Sender: TObject);
begin
  if (counter < t_end / int_rec) then
  begin
    Inc(counter);
    DisplayIt(Sender);
  end;
end;

procedure TOutputResults.StepBackward(Sender: TObject);
begin
  if counter > 0 then
  begin
    Dec(counter);
    DisplayIt(Sender);
  end;
end;

procedure TOutputResults.DisplayIt(Sender: TObject);
var
  i, j, i2: Integer;
  SL1: TStringList;
begin
  time_ := counter * int_rec + tin_a_day_ini * 3600 / t_unit;
  Main.AnimeCounter.Text := FloatToStr(time_);
  for i := 1 to n do
    dumarray[i] := (z[i] + z[i-1]) / 2;
  dumstr :=  BaseDir + Slash + 'History' + Slash + ExtractFileName(CaseName) + '_' + FormatFloat('000', counter) + '.txt';
  if FileExists(dumstr) = False then
  begin
    Main.Play_or_StopClick(Sender);
    t_end := counter * int_rec ;
    Exit;
  end;
  {$I-}
  SL1 := TStringList.Create;
  i2 := 1;
  try
    SL1.LoadFromFile(dumstr);
    S := Main.ConvertDecimalSeparator(SL1[0]);
    dum := NextValue;
    avdt  := NextValue;
    navdt := Trunc(NextValue);
    WCumRError := NextValue;
    SCumRError := NextValue;
    ECumRError := NextValue;
    Hpond      := NextValue;
    CumEvap    := NextValue;
    CumTr      := NextValue;
    CumInfilt  := NextValue;
    dum := NextValue;
    if dum > 0 then
      CumTrp     := CumTr / dum;
    for i := 3 to SL1.Count do
    begin
      S := Main.ConvertDecimalSeparator(SL1[i-1]);
      try
        dum := NextValue;
        W[i2] := NextValue;
        P[i2] := NextValue;
        T[i2] := NextValue;
        C[i2] := NextValue;
        dum := NextValue;
        ql[i2]:= NextValue;
        qv[i2]:= NextValue;
        Swp[i2]:= - NextValue * t_unit / 864000;
        Swr[i2]:= - NextValue * t_unit / 864000;
        i2 := i2 + 1;
      except
      end;
    end;
  finally
    SL1.Free;
  end;
  Vreservoir := Vr[counter];
  for i := 1 to i2 do
  begin
    T[i] := T[i] + 273.2;
    ql[i] := - ql[i];
    qv[i] := - qv[i];
  end;
  {$I+}
  Main.Profile;
  Main.DrawProfile(W,dumarray,n,0,0.6,clBlue);
  Main.DrawProfile(C,dumarray,n,0,6,$009090);
  //----------------------------------------------------------
  if Heat then
  begin
    Main.DrawProfile(T, dumarray, n , 263.2, 323.2, $000080FF);
    Main.DrawProfile(Swp , dumarray, n, 0, 10 / t_unit, clTeal);
    Main.DrawProfile(Swr , dumarray, n, 0, 10 / t_unit, clGreen);
  end
  else
  begin
    if P[n] < -150 then
      dum := -900
    else
      dum := -150;
    Main.DrawProfile(P,dumarray, n, dum, 0,clGray);
  end;
  Main.FluxProfile;
  Main.DrawFluxProfile(ql,dumarray, clNavy);
  Main.DrawFluxProfile(qv,dumarray, clAqua);
  j := Mode; Mode := 1;
  Main.History;
  DayCounter := Trunc(time_ * t_unit / 86400);
  if DayCounter < 9 then
    HistoryDrawStartingPoint := 1
  else
    HistoryDrawStartingPoint := Trunc(86400 * (DayCounter - 9) / (int_rec * t_unit) ) + 1;
  time_in_a_day := time_ - DayCounter * 86400 / t_unit;
  for i := 1 to counter do
    dumarray[i] := Wtime[1, i];
  Main.HistoryDraw(time, dumarray, HistoryDrawStartingPoint, counter, -0.6, 0.4, clBlue, psSolid);
  if solute then
  begin
    for i := 1 to counter do
      dumarray[i] := Ctime[1, i];
    Main.HistoryDraw(time, dumarray, HistoryDrawStartingPoint, counter, -12 ,   8, $009090, psSolid);
  end;
  if l_bc_type = 4 then
    Main.HistoryDraw(t_P_bot, Pbot, HistoryDrawStartingPoint, NumPbotData, -600, 400, clTeal, psDot);
  if heat then
  begin
    for i := 1 to counter do
      dumarray[i] := Ttime[1, i];
    Main.HistoryDraw(time, dumarray, HistoryDrawStartingPoint, counter, -60 ,  40, $000080FF, psSolid);
  end;
  for i := 1 to counter do
    dumarray[i] := Ptime[1, i];
  Main.HistoryDraw(time, dumarray, HistoryDrawStartingPoint, counter, -600, 400, clTeal, psSolid);
  Main.HistoryDraw(time, ql_bot, HistoryDrawStartingPoint, counter,  -60,  40, clNavy, psSolid);
  Main.HistoryDraw(time, Evap  , HistoryDrawStartingPoint, counter,  -60,  40, clAqua, psSolid);
  Main.Report;
  Mode := j;
end;

//===============================================================================================================================
procedure TOutputResults.SetProperty_of_TSM_ObservationFigure(FileName: String);
var
  TSM_ObservationFigure: TXYGrapher;
begin
  TSM_ObservationFigure := TXYGrapher.Create;
  with TSM_ObservationFigure do
  begin
    SetFormSize(600, 750);
    NumDataSeries := 5;
    //----------
    DataSeries[1] := fl_outflow;
    Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 0; ReadEnd[1] := 0;
    LineStyle[1] := 0; LineWidth[1] := 0.25; LineColor[1] := 0; MarkStyle[1] := 1; MarkSize[1] := 1.5; MarkColor[1] := 16776960;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    //----------
    DataSeries[2] := fl_clmtc_con;
    Xb[2] := 1; Xt[2] := 0; Yl[2] := 2; Yr[2] := 0; ReadStart[2] := 0; ReadEnd[2] := 0;
    LineStyle[2] := 0; LineWidth[2] := 0.25; LineColor[2] := 0; MarkStyle[2] := 5; MarkSize[2] := 1.5; MarkColor[2] := 16711680;
    Xmult[2] := 1; Xoffset[2] := 0; Ymult[2] := 1; Yoffset[2] := 0;
    //----------
    DataSeries[3] := fl_P_bot;
    Xb[3] := 1; Xt[3] := 0; Yl[3] := 0; Yr[3] := 2; ReadStart[3] := 0; ReadEnd[3] := 0;
    LineStyle[3] := 2; LineWidth[3] := 0.2; LineColor[3] := 16744576; MarkStyle[3] := 0; MarkSize[3] := 0; MarkColor[3] := 0;
    Xmult[3] := 1; Xoffset[3] := 0; Ymult[3] := 1; Yoffset[3] := 0;
    //----------
    DataSeries[4] := CaseName + '_flux.txt';
    Xb[4] := 1; Xt[4] := 0; Yl[4] := 3; Yr[4] := 0; ReadStart[4] := 0; ReadEnd[4] := 0;
    LineStyle[4] := 1; LineWidth[4] := 0.25; LineColor[4] := 8421440; MarkStyle[4] := 0; MarkSize[4] := 0; MarkColor[4] := 0;
    Xmult[4] := 1; Xoffset[4] := 0; Ymult[4] := 1; Yoffset[4] := 0;
    //----------
    DataSeries[5] := CaseName + '_flux.txt';
    Xb[5] := 1; Xt[5] := 0; Yl[5] := 2; Yr[5] := 0; ReadStart[5] := 0; ReadEnd[5] := 0;
    LineStyle[5] := 1; LineWidth[5] := 0.25; LineColor[5] := 8421440; MarkStyle[5] := 0; MarkSize[5] := 0; MarkColor[5] := 0;
    Xmult[5] := 1; Xoffset[5] := 0; Ymult[5] := 1; Yoffset[5] := 0;
    //----------
    Height := 60; Width := 120; Lmargin := 30; Tmargin := 20; FLineWidth := 0.3; NumFontSize := 10;
    NumFontName := 'Arial';
    Max[1] := 6 * Ceil(t_end * t_unit / (3600 * 6)) ; Min[1] := 0 ; Division[1] := 6; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 6 * Ceil(t_end * t_unit / (3600 * 6)) ; Min[2] := 0 ; Division[2] := 6; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] := 1.5; Min[3] := 0 ; Division[3] := 6; Minordivision[3] := 2; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] := 0; Min[4] := -200; Division[4] := 4; Minordivision[4] := 2; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 7;
    Legend[1] := '[r][b][4]Fig.1[ ] Comparison of observed and optimized amounts of outflow and evaporation for ' + SoilName[1] +'. \n';
    Legend[2] := 'Time(h)\n';
    Legend[3] := '';
    Legend[4] := 'Cumulative outflow or evaporation (cm)\n';
    Legend[5] := 'Pressure head [s]y[a](cm)\n';
    Legend[6] := '[b]Measured[ ]\n%1 Outflow \n%2 Evaporation\n%3 [i][r]h[a][ ] at the bottom\n';
    Legend[7] := '%4 Optimized\n';
    LegendLspacing[1] :=5.0; LegendXshift[1] :=2.4; LegendYshift[1] :=-1.4;
    LegendLspacing[2] :=  0; LegendXshift[2] :=1.1; LegendYshift[2] := 0.3;
    LegendLspacing[3] :=  0; LegendXshift[3] :=  0; LegendYshift[3] := 0;
    LegendLspacing[4] :=  0; LegendXshift[4] :=  0; LegendYshift[4] := 0;
    LegendLspacing[5] :=  0; LegendXshift[5] :=  0; LegendYshift[5] := 0;
    LegendLspacing[6] :=  5; LegendXshift[6] :=-5.9; LegendYshift[6] :=-26.9;
    LegendLspacing[7] :=  5; LegendXshift[7] :=28.1; LegendYshift[7] :=-26.9;
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  TSM_ObservationFigure.Free;
end;
//----------------------------------------------------------
procedure TOutputResults.SetProperty_of_EM_ObservationFigure(FileName: String);
var
  EM_ObservationFigure: TXYGrapher;
begin
  EM_ObservationFigure := TXYGrapher.Create;
  with EM_ObservationFigure do
  begin
    SetFormSize(600, 750);
    NumDataSeries := 5;
    //----------
    DataSeries[1] := fl_pressure;
    Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 0; ReadEnd[1] := 0;
    LineStyle[1] := 0; LineWidth[1] := 0.25; LineColor[1] := 0; MarkStyle[1] := 11; MarkSize[1] := 1; MarkColor[1] := 32896;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    //----------
    DataSeries[2] := fl_pressure;
    Xb[2] := 1; Xt[2] := 0; Yl[2] := 3; Yr[2] := 0; ReadStart[2] := 0; ReadEnd[2] := 0;
    LineStyle[2] := 0; LineWidth[2] := 0.25; LineColor[2] := 0; MarkStyle[2] := 13; MarkSize[2] := 1; MarkColor[2] := 32896;
    Xmult[2] := 1; Xoffset[2] := 0; Ymult[2] := 1; Yoffset[2] := 0;
    //----------
    DataSeries[3] := CaseName + '_observation1.txt';
    Xb[3] := 1; Xt[3] := 0; Yl[3] := 3; Yr[3] := 0; ReadStart[3] := 0; ReadEnd[3] := 0;
    LineStyle[3] := 1; LineWidth[3] := 0.2; LineColor[3] := 8421440; MarkStyle[3] := 0; MarkSize[3] := 0; MarkColor[3] := 0;
    Xmult[3] := 1; Xoffset[3] := 0; Ymult[3] := 1; Yoffset[3] := 0;
    //----------
    DataSeries[4] := CaseName + '_observation2.txt';
    Xb[4] := 1; Xt[4] := 0; Yl[4] := 3; Yr[4] := 0; ReadStart[4] := 0; ReadEnd[4] := 0;
    LineStyle[4] := 2; LineWidth[4] := 0.2; LineColor[4] := 8421440; MarkStyle[4] := 0; MarkSize[4] := 0; MarkColor[4] := 0;
    Xmult[4] := 1; Xoffset[4] := 0; Ymult[4] := 1; Yoffset[4] := 0;
    //----------
    DataSeries[5] := fl_clmtc_con;
    Xb[5] := 1; Xt[5] := 0; Yl[5] := 0; Yr[5] := 2; ReadStart[5] := 0; ReadEnd[5] := 0;
    LineStyle[5] := 1; LineWidth[5] := 0.2; LineColor[5] := 16776960; MarkStyle[5] := 0; MarkSize[5] := 0; MarkColor[5] := 0;
    Xmult[5] := 1; Xoffset[5] := 0; Ymult[5] := 1; Yoffset[5] := 0;
    //----------
    Height := 70; Width := 100; Lmargin := 30; Tmargin := 20; FLineWidth := 0.3; NumFontSize := 10;
    NumFontName := 'Arial';
    Max[1] := 6 * Ceil(t_end * t_unit / (3600 * 6)) ; Min[1] := 0 ; Division[1] := 6; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 6 * Ceil(t_end * t_unit / (3600 * 6)) ; Min[2] := 0 ; Division[2] := 6; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] := 0; Min[3] := -800 ; Division[3] := 4; Minordivision[3] := 2; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] := 2; Min[4] := 0    ; Division[4] := 4; Minordivision[4] := 2; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 7;
    Legend[1] := '[r] Fig. Comparison of measured and optimized pressure head during evaporation experiment.\n';
    Legend[2] := 'Time(h)\n';
    Legend[3] := '';
    Legend[4] := 'Pressure head [s]y[a](cm)\n';
    Legend[5] := 'Cumulative evaporation  [s]S[r]E[a](cm)\n';
    Legend[6] := 'Measured\n%1 [r][i]z[/i] = ' + FloatToStr(z_obs[1]) + 'cm\n%2 [i]z[/i] = ' + FloatToStr(z_obs[2]) + 'cm\nOptimized\n%3 [r][i]z[/i] = ' + FloatToStr(z_obs[1]) + 'cm\n%4 [i]z[/i] = ' + FloatToStr(z_obs[2]) + 'cm\n';
    Legend[7] := '%5 Evaporation\n';
    LegendLspacing[1] :=5.0; LegendXshift[1] :=2.4; LegendYshift[1] :=-1.4;
    LegendLspacing[2] :=  0; LegendXshift[2] :=1.1; LegendYshift[2] := 0.3;
    LegendLspacing[3] :=  0; LegendXshift[3] :=  0; LegendYshift[3] := 0;
    LegendLspacing[4] :=  0; LegendXshift[4] :=  0; LegendYshift[4] := 0;
    LegendLspacing[5] :=  0; LegendXshift[5] :=  0; LegendYshift[5] := 0;
    LegendLspacing[6] :=  5; LegendXshift[6] :=-45; LegendYshift[6] :=-22;
    LegendLspacing[7] :=  5; LegendXshift[7] :=-19; LegendYshift[7] :=22;
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  EM_ObservationFigure.Free;
end;
//----------------------------------------------------------
procedure TOutputResults.SetProperty_of_MSO_ObservationFigure;
var
  MSO_ObservationFigure : TXYGrapher;
  i: Integer;
begin
  MSO_ObservationFigure := TXYGrapher.Create;
  with MSO_ObservationFigure do
  begin
    SetFormSize(600, 750);
    if tensio <> 0 then NumDataSeries := 5 else NumDataSeries := 4;
    //----------
    DataSeries[1] := fl_outflow;
    Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 0; ReadEnd[1] := 0;
    LineStyle[1] := 1; LineWidth[1] := 0.25; LineColor[1] := 8421440; MarkStyle[1] := 0; MarkSize[1] := 0;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    //----------
    DataSeries[2] := CaseName + '_flux.txt';
    Xb[2] := 1; Xt[2] := 0; Yl[2] := 3; Yr[2] := 0; ReadStart[2] := 0; ReadEnd[2] := 0;
    LineStyle[2] := 1; LineWidth[2] := 0.25; LineColor[2] := 16711680; MarkStyle[2] := 0; MarkSize[2] := 0;
    Xmult[2] := 1; Xoffset[2] := 0; Ymult[2] := 1; Yoffset[2] := 0;
    //----------
    DataSeries[3] := fl_P_bot;
    Xb[3] := 1; Xt[3] := 0; Yl[3] := 0; Yr[3] := 2; ReadStart[3] := 0; ReadEnd[3] := 0;
    LineStyle[3] := 2; LineWidth[3] := 0.2; LineColor[3] := 12615680; MarkStyle[3] := 0; MarkSize[3] := 0;
    Xmult[3] := 1; Xoffset[3] := 0; Ymult[3] := 1; Yoffset[3] := 0;
    //----------
    DataSeries[4] := CaseName + '_observation1.txt';
    Xb[4] := 1; Xt[4] := 0; Yl[4] := 0; Yr[4] := 3; ReadStart[4] := 0; ReadEnd[4] := 0;
    LineStyle[4] := 1; LineWidth[4] := 0.25; LineColor[4] := 16711680; MarkStyle[4] := 0; MarkSize[4] := 0;
    Xmult[4] := 1; Xoffset[4] := 0; Ymult[4] := 1; Yoffset[4] := 0;
    //----------
    DataSeries[5] := fl_pressure;
    Xb[5] := 1; Xt[5] := 0; Yl[5] := 0; Yr[5] := 2; ReadStart[5] := 0; ReadEnd[5] := 0;
    LineStyle[5] := 1; LineWidth[5] := 0.25; LineColor[5] := 8421440; MarkStyle[5] := 0; MarkSize[5] := 0;
    Xmult[5] := 1; Xoffset[5] := 0; Ymult[5] := 1; Yoffset[5] := 0;
    //----------
    Height := 60; Width := 120; Lmargin := 30; Tmargin := 20; FLineWidth := 0.4; NumFontSize := 9;
    NumFontName := 'Arial';
    Max[1] := 6 * Ceil(t_end * t_unit / (3600 * 6)) ; Min[1] := 0 ; Division[1] := 6; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 6 * Ceil(t_end * t_unit / (3600 * 6)) ; Min[2] := 0 ; Division[2] := 6; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] := 1.5; Min[3] := 0 ; Division[3] := 5; Minordivision[3] := 2; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] := 0; Min[4] := -800; Division[4] := 4; Minordivision[4] := 2; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 6;
    for i := 1 to 6 do
      Legend[i] := Msg[i];
    LegendLspacing[1] :=4.5; LegendXshift[1] :=  0; LegendYshift[1] := 0;
    LegendLspacing[2] :=  0; LegendXshift[2] :=1.1; LegendYshift[2] := 0.3;
    LegendLspacing[3] :=  0; LegendXshift[3] :=  0; LegendYshift[3] := 0;
    LegendLspacing[4] :=  0; LegendXshift[4] :=  0; LegendYshift[4] := 0;
    LegendLspacing[5] :=  0; LegendXshift[5] :=  0; LegendYshift[5] := 0;
    LegendLspacing[6] :=  5; LegendXshift[6] :=-20; LegendYshift[6] := 10;
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  MSO_ObservationFigure.Free;
end;
//----------------------------------------------------------
procedure TOutputResults.SetProperty_of_MSI_ObservationFigure;
var
  MSI_ObservationFigure : TXYGrapher;
  i: Integer;
begin
  MSI_ObservationFigure := TXYGrapher.Create;
  with MSI_ObservationFigure do
  begin
    SetFormSize(600, 750);
    NumDataSeries := 3;
    //----------
    DataSeries[1] := fl_outflow;
    Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 0; ReadEnd[1] := 0;
    LineStyle[1] := 1; LineWidth[1] := 0.25; LineColor[1] := 8421440; MarkStyle[1] := 0; MarkSize[1] := 0;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    //----------
    DataSeries[2] := CaseName + '_flux.txt';
    Xb[2] := 1; Xt[2] := 0; Yl[2] := 3; Yr[2] := 0; ReadStart[2] := 0; ReadEnd[2] := 0;
    LineStyle[2] := 1; LineWidth[2] := 0.25; LineColor[2] := 16711680; MarkStyle[2] := 0; MarkSize[2] := 0;
    Xmult[2] := 1; Xoffset[2] := 0; Ymult[2] := -1; Yoffset[2] := 0;
    //----------
    DataSeries[3] := fl_P_bot;
    Xb[3] := 1; Xt[3] := 0; Yl[3] := 0; Yr[3] := 2; ReadStart[3] := 0; ReadEnd[3] := 0;
    LineStyle[3] := 2; LineWidth[3] := 0.2; LineColor[3] := 12615680; MarkStyle[3] := 0; MarkSize[3] := 0;
    Xmult[3] := 1; Xoffset[3] := 0; Ymult[3] := 1; Yoffset[3] := 0;
    //----------
    Height := 60; Width := 120; Lmargin := 30; Tmargin := 20; FLineWidth := 0.4; NumFontSize := 9;
    NumFontName := 'Arial';
    Max[1] := 6 * Ceil(t_end * t_unit / (3600 * 6)) ; Min[1] := 0 ; Division[1] := 6; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 6 * Ceil(t_end * t_unit / (3600 * 6)) ; Min[2] := 0 ; Division[2] := 6; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] := 1.25; Min[3] :=    0; Division[3] := 5; Minordivision[3] := 2; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] :=    0; Min[4] := -250; Division[4] := 5; Minordivision[4] := 2; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 6;
    for i := 1 to 6 do
      Legend[i] := Msg[i+6];
    LegendLspacing[1] :=4.5; LegendXshift[1] :=  0; LegendYshift[1] := 0;
    LegendLspacing[2] :=  0; LegendXshift[2] :=1.1; LegendYshift[2] := 0.3;
    LegendLspacing[3] :=  0; LegendXshift[3] :=  0; LegendYshift[3] := 0;
    LegendLspacing[4] :=  0; LegendXshift[4] :=  0; LegendYshift[4] := 0;
    LegendLspacing[5] :=  0; LegendXshift[5] :=  0; LegendYshift[5] := 0;
    LegendLspacing[6] :=  5; LegendXshift[6] :=-20; LegendYshift[6] := 10;
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  MSI_ObservationFigure.Free;
end;
//----------------------------------------------------------------------
procedure TOutputResults.SetProperty_of_UIM_ObservationFigure;
var
  UIM_ObservationFigure : TXYGrapher;
  i: Integer;
begin
  UIM_ObservationFigure := TXYGrapher.Create;
  with UIM_ObservationFigure do
  begin
    SetFormSize(600, 750);
    if fl_front <> '' then
      NumDataSeries := 4
    else
      NumDataSeries := 3;
    //----------
    DataSeries[1] := fl_outflow;
    Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 0; ReadEnd[1] := 0;
    LineStyle[1] := 1; LineWidth[1] := 0.25; LineColor[1] := 8421440; MarkStyle[1] := 0; MarkSize[1] := 0;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    //----------
    DataSeries[2] := CaseName + '_flux.txt';
    Xb[2] := 1; Xt[2] := 0; Yl[2] := 3; Yr[2] := 0; ReadStart[2] := 0; ReadEnd[2] := 0;
    LineStyle[2] := 2; LineWidth[2] := 0.25; LineColor[2] := 8421440; MarkStyle[2] := 0; MarkSize[2] := 0;
    Xmult[2] := 1; Xoffset[2] := 0; Ymult[2] := -1; Yoffset[2] := 0;
    //----------
    DataSeries[3] := CaseName + '_flux.txt';
    Xb[3] := 1; Xt[3] := 0; Yl[3] := 0; Yr[3] := 5; ReadStart[3] := 0; ReadEnd[3] := 0;
    LineStyle[3] := 2; LineWidth[3] := 0.2; LineColor[3] := 12615680; MarkStyle[3] := 0; MarkSize[3] := 0;
    Xmult[3] := 1; Xoffset[3] := 0; Ymult[3] := 1; Yoffset[3] := 0;
    //----------
    if fl_front <> '' then
    begin
      DataSeries[4] := fl_front;
      Xb[4] := 1; Xt[4] := 0; Yl[4] := 0; Yr[4] := 2; ReadStart[4] := 0; ReadEnd[4] := 0;
      LineStyle[4] := 1; LineWidth[4] := 0.2; LineColor[4] := 12615680; MarkStyle[4] := 0; MarkSize[4] := 0;
      Xmult[4] := 1; Xoffset[4] := 0; Ymult[4] := 1; Yoffset[4] := 0;
    end;
    //----------
    Height := 60; Width := 120; Lmargin := 30; Tmargin := 20; FLineWidth := 0.4; NumFontSize := 9;
    NumFontName := 'Arial';
    Max[1] := 6 * Ceil(t_end * t_unit / (3600 * 6)) ; Min[1] := 0 ; Division[1] := 6; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 6 * Ceil(t_end * t_unit / (3600 * 6)) ; Min[2] := 0 ; Division[2] := 6; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] := 24; Min[3] :=   0; Division[3] := 6; Minordivision[3] := 2; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] :=  0; Min[4] :=  30; Division[4] := 6; Minordivision[4] := 2; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 6;
    for i := 1 to 6 do
      Legend[i] := Msg[i+12];
    LegendLspacing[1] :=4.5; LegendXshift[1] :=  0; LegendYshift[1] := 0;
    LegendLspacing[2] :=  0; LegendXshift[2] :=1.1; LegendYshift[2] := 0.3;
    LegendLspacing[3] :=  0; LegendXshift[3] :=  0; LegendYshift[3] := 0;
    LegendLspacing[4] :=  0; LegendXshift[4] :=  0; LegendYshift[4] := 0;
    LegendLspacing[5] :=  0; LegendXshift[5] :=  0; LegendYshift[5] := 0;
    LegendLspacing[6] :=  5; LegendXshift[6] :=-20; LegendYshift[6] := 10;
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  UIM_ObservationFigure.Free;
end;
//----------------------------------------------------------
procedure TOutputResults.SetProperty_of_ProfileFigure(FileName: String);
var
  f : TextFile;
  i: Integer;
  ProfileFigure : TXYGrapher;
begin
  ProfileFigure := TXYGrapher.Create;
  with ProfileFigure do
  begin
    SetFormSize(500, 500);
    NumDataSeries := 1;
    //----------
    DataSeries[1] := CaseName + '_profile.txt';
    Xt[1] := 0; Yl[1] := 1; Yr[1] := 0; ReadStart[1] := 7; ReadEnd[1] := 0;
    if Water then
    begin
      Xb[1] := 2;
      LineColor[1] := 16776960;
    end
    else
    begin
      Xb[1] := 4;
      LineColor[1] := 43775;
    end;
    LineStyle[1] := 1; LineWidth[1] := 0.2; MarkStyle[1] := 0; MarkSize[1] := 1.5; MarkColor[1] := LineColor[1];
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    //----------
    if (IPET_type = 1) or (IPET_type = 4) or (IPET_type = 5) or (IPET_type = 11) then
    begin
      NumDataSeries := 2;
      dumstr := CaseName + '_MeasuredProfile.txt';
      DataSeries[2] := dumstr;
      Xb[2] := 2; Xt[2] := 0; Yl[2] := 1; Yr[2] := 0; ReadStart[2] := 0; ReadEnd[2] := 0;
      LineStyle[2] := 0; LineWidth[2] := 0.25; LineColor[2] := 0; MarkStyle[2] := 11; MarkSize[2] := 1.5;
      Xmult[2] := 1; Xoffset[2] := 0; Ymult[2] := 1; Yoffset[2] := 0;
      //---------
      AssignFile(f, dumstr);
      Rewrite(f);
      if IPET_type <> 11 then
      begin
        Writeln(f, 'Measured water content profile');
        Writeln(f, ' z(cm)   VWC');
        MarkColor[2] := 16776960;
      end
      else
      begin
        Writeln(f, 'Measured relative concentration profile');
        Writeln(f, ' z(cm)   C/Ci');
        MarkColor[2] := 43775;
      end;
      for i := 1 to NumSampling do
        if IPET_type <> 11 then
          Writeln(f, FormatFloat('#0.##', zm[i]) + Separator + FormatFloat('0.####', Wm[i]))
        else
          Writeln(f, FormatFloat('#0.##', zms[i]) + Separator + FormatFloat('0.####', Cm[i]));
      CloseFile(f);
    end;
    //----------
    Height := 65; Width := 65; Lmargin := 30; Tmargin := 20; FLineWidth := 0.3; NumFontSize := 10;
    NumFontName := 'Arial';
    if IPET_type <> 11 then
      dumint := Trunc(W[n] * 10) + 1
    else
      dumint := 5;
    Max[1] := 0.2 * dumint; Min[1] := 0; Division[1] := dumint; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 0.2 * dumint; Min[2] := 0; Division[2] := dumint; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    if z_lb < 25 then
      dumint := 5
    else
      dumint := 25;
    dumint := Ceil((z_lb - 0.001) / dumint) * dumint;
    if IPET_type = 1 then
      dumint := 5;
    Max[3] := 0; Min[3] := dumint; Division[3] := 4; Minordivision[3] := 0; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] := 0; Min[4] := dumint; Division[4] := 4; Minordivision[4] := 0; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    if IPET_type = 1 then
      NumLegend := 6
    else
      NumLegend := 5;
    case t_unit of
        1: dumstr := 's';
       60: dumstr := 'min';
     3600: dumstr := 'h';
    86400: dumstr := 'd';
    end;
    if IPET_type <> 11 then
    begin
      Legend[1] := '[b]Fig.1[ ] Water content profile at [i]t[ ] = ' + FloatToStr(t_end) + dumstr + '. \n';
      Legend[2] := 'Volumetric water content [s]q[a] (cm[^]3[-]/cm[^]3[-\n';
    end
    else
    begin
      Legend[1] := '[b]Fig.1[ ] Concentration profile at [i]t[ ] = ' + FloatToStr(t_end) + dumstr + '. \n';
      Legend[2] := 'Relative concentration [r][i]C / C[_]i[-][/i][a]\n';
    end;
    Legend[3] := '';
    Legend[4] := 'Depth  [i]z[ ](cm)\n';
    Legend[5] := '';
    Legend[6] := '%2 Measured\n%1 Optimized\n';
    LegendLspacing[1] :=5.0; LegendXshift[1] :=2.4; LegendYshift[1] :=-1.4;
    LegendLspacing[2] :=  0; LegendXshift[2] :=1.1; LegendYshift[2] := 0.3;
    LegendLspacing[3] :=  0; LegendXshift[3] :=  0; LegendYshift[3] := 0;
    LegendLspacing[4] :=  0; LegendXshift[4] := 5.0; LegendYshift[4] := 0;
    LegendLspacing[5] :=  0; LegendXshift[5] :=  0; LegendYshift[5] := 0;
    LegendLspacing[6] :=  5; LegendXshift[6] :=-22; LegendYshift[6] :=17.4;
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  ProfileFigure.Free;
end;
//----------------------------------------------------------
procedure TOutputResults.SetProperty_of_TherConCurve(layer: Integer; FileName: String);
var
  i : Integer;
  TherConCurve : TXYGrapher;
begin
  //--- Saving of Curve ---//
  j := layer;
  TherConCurve := TXYGrapher.Create;
  with TherConCurve do
  begin
    SetFormSize(600, 750);
    NumDataSeries := 1;
    //----------
    DataSeries[1] := 'a  + b * x - (a - d) * exp(-c * x^e)  <> ?';
    Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 6; ReadEnd[1] := 0;
    Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 6; ReadEnd[1] := 0;
    LineStyle[1] := 1; LineWidth[1] := 0.25; LineColor[1] := 33023; MarkStyle[1] := 0; MarkSize[1] := 0;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    p[j,1] := Soil.Akh[j]; p[j,2] := Soil.Bkh[j]; p[j,3] := Soil.Dkh[j]; p[j,4] := Soil.Ckh[j]; p[j,5] := Soil.Ekh[j];
    //----------
    Height := 65; Width := 65; Lmargin := 30; Tmargin := 20; FLineWidth := 0.4; NumFontSize := 9;
    NumFontName := 'Arial';
    Max[1] := 1 ; Min[1] := 0 ; Division[1] := 5; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 1 ; Min[2] := 0 ; Division[2] := 5; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] := 0.01; Min[3] := 0; Division[3] := 5; Minordivision[3] := 2; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] := 0.01; Min[4] := 0; Division[4] := 5; Minordivision[4] := 2; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 5;
    for i := 2 to 5 do
      Legend[i] := Msg[i+20];
    Legend[1] := Msg[21] + SoilName[j] + '\n';
    LegendLspacing[1] := 4; LegendXshift[1] := 0; LegendYshift[1] := 0;
    LegendLspacing[2] := 0; LegendXshift[2] := 0; LegendYshift[2] := 0;
    LegendLspacing[3] := 0; LegendXshift[3] := 0; LegendYshift[3] := 0;
    LegendLspacing[4] := 0; LegendXshift[4] := 0; LegendYshift[4] := -2;
    LegendLspacing[5] := 0; LegendXshift[5] := 0; LegendYshift[5] := 0;
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  TherConCurve.Free;
end;
//----------------------------------------------------------
procedure TOutputResults.SetProperty_of_AlbedoCurve(layer: Integer; FileName: String);
var
  i: Integer;
  AlbedoCurve : TXYGrapher;
begin
  j := layer;
  AlbedoCurve := TXYGrapher.Create;
  with AlbedoCurve do
  begin
    SetFormSize(550, 550);
    NumDataSeries := 1;
    //----------
    DataSeries[1] := 'aMin + (aMax - aMin) / (1 + (Aal * x) ^ Bal) <> ?';
    Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 6; ReadEnd[1] := 0;
    LineStyle[1] := 1; LineWidth[1] := 0.25; LineColor[1] := 32896; MarkStyle[1] := 0; MarkSize[1] := 0;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    p[j,1] := Soil.AlbedoMin[j]; p[j,2] := Soil.AlbedoMax[j]; p[j,3] := Soil.Aal[j]; p[j,4] := Soil.Bal[j];
    //----------
    Height := 65; Width := 65; Lmargin := 30; Tmargin := 20; FLineWidth := 0.4; NumFontSize := 9;
    NumFontName := 'Arial';
    dumint := Trunc(Soil.Wsat[j] * 10) + 1;
    Max[1] := 0.1 * dumint; Min[1] := 0 ; Division[1] := dumint; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 0.1 * dumint; Min[2] := 0 ; Division[2] := dumint; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] := 0.5; Min[3] := 0; Division[3] := 5; Minordivision[3] := 0; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] := 0.5; Min[4] := 0; Division[4] := 5; Minordivision[4] := 0; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 5;
    for i := 2 to 5 do
      Legend[i] := Msg[i+27];
    Legend[1] := Msg[26] + SoilName[j] + '\n';
    LegendLspacing[1] := 4; LegendXshift[1] := 0; LegendYshift[1] := 0;
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  AlbedoCurve.Free;
end;
//----------------------------------------------------------
procedure TOutputResults.SetProperty_of_EnCurve(layer: Integer; FileName: String);
var
  i : Integer;
  EnCurve : TXYGrapher;
begin
  EnCurve := TXYGrapher.Create;
  j := layer;
  with EnCurve do
  begin
    SetFormSize(550, 500);
    //----------
    NumDataSeries := 1;
    DataSeries[1] := 'a + b * x - (a - d) * exp(-c * x^e)  <> ?';
    Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 0; ReadEnd[1] := 0;
    LineStyle[1] := 1; LineWidth[1] := 0.25; LineColor[1] := 8421440; MarkStyle[1] := 0; MarkSize[1] := 0;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    p[j,1] := Soil.Aen[j]; p[j,2] := Soil.Ben[j]; p[j,4] := Soil.Cen[j]; p[j,3] := Soil.Den[j]; p[j,5] := Soil.Een[j] ;
    //---------------------------------
    Height := 65; Width := 65; Lmargin := 30; Tmargin := 20; FLineWidth := 0.4; NumFontSize := 9;
    NumFontName := 'Arial';
    Max[1] := 1 ; Min[1] := 0 ; Division[1] := 5; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 1 ; Min[2] := 0 ; Division[2] := 5; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] := 25; Min[3] := 0; Division[3] := 5; Minordivision[3] := 0; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] := 25; Min[4] := 0; Division[4] := 5; Minordivision[4] := 0; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 5;
    for i := 2 to 5 do
      Legend[i] := Msg[i+32];
    Legend[1] := Msg[33] + SoilName[j] + '\n';
    LegendLspacing[1] := 4; LegendXshift[1] := 0; LegendYshift[1] := 0;
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  EnCurve.Free;
end;

procedure TOutputResults.SetProperty_of_ReductionFunction(FileName: String);
var
  XYGrapher : TXYGrapher;
begin
  XYGrapher := TXYGrapher.Create;
  with XYGrapher do
  begin
    SetFormSize(550, 500);
    NumDataSeries := 1;
    //----------
    DataSeries[1] := '1 / [1 + (x / pm50)^pm]  <> ?';
    Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 6; ReadEnd[1] := 0;
    LineStyle[1] := 1; LineWidth[1] := 0.25; LineColor[1] := 33023; MarkStyle[1] := 0; MarkSize[1] := 0;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    p[1,1] := - Pm50; p[1,2] := pm_;
    //----------
    Height := 65; Width := 65; Lmargin := 30; Tmargin := 20; FLineWidth := 0.4; NumFontSize := 9;
    NumFontName := 'Arial';
    Max[1] := 10000000; Min[1] := 1; Division[1] := 7; Minordivision[1] := 2; logarithm[1] := 1; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 1       ; Min[2] := 0; Division[2] := 5; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] := 1       ; Min[3] := 0; Division[3] := 5; Minordivision[3] := 0; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] := 1       ; Min[4] := 0; Division[4] := 5; Minordivision[4] := 0; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 5;
    Legend[1] := 'Water (drought) stress response function\n';
    Legend[2] := 'Suction  [s]-y[a](cm)\n';
    Legend[4] := 'Reduction coefficient  [s]a[r][_][i]m[/i][-]\n';
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  XYGrapher.Free;
end;

procedure TOutputResults.SetProperty_of_RootDensityDistribution(FileName: String);
var
  XYGrapher : TXYGrapher;
begin
  XYGrapher := TXYGrapher.Create;
  with XYGrapher do
  begin
    SetFormSize(550, 500);
    NumDataSeries := 1;
    //----------
    DataSeries[1] := 'x = (b + 1) * dr^(-b-1) * (dr - y)^b  <> ?';
    Xb[1] := 2; Xt[1] := 0; Yl[1] := 1; Yr[1] := 0; ReadStart[1] := 0; ReadEnd[1] := 0;
    LineStyle[1] := 1; LineWidth[1] := 0.25; LineColor[1] := 8421440; MarkStyle[1] := 0; MarkSize[1] := 0;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    Simulator.EvaluateDailyScaleParameters;
    p[1,1] := brt; p[1,2] := drt;
    CXmax[1] := '0'; CXmin[1] := 'dr';
    //----------
    Height := 65; Width := 65; Lmargin := 30; Tmargin := 20; FLineWidth := 0.4; NumFontSize := 9;
    NumFontName := 'Arial';
    Max[1] := 0.15;Min[1] := 0; Division[1] := 3; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 0.15;Min[2] := 0; Division[2] := 3; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] := 0  ; Min[3] :=50; Division[3] := 5; Minordivision[3] := 0; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] := 0  ; Min[4] :=50; Division[4] := 5; Minordivision[4] := 0; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 5;
    Legend[1] := 'Normalized root density profile\n';
    Legend[2] := 'Normalized root density  [s]b\n';
    Legend[4] := 'Depth  [i][r]z[a][/i] (cm)\n';
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  XYGrapher.Free;
end;

procedure TOutputResults.SetProperty_of_RootDepth_and_LAI(FileName: String);
var
  XYGrapher : TXYGrapher;
begin
  XYGrapher := TXYGrapher.Create;
  with XYGrapher do
  begin
    SetFormSize(550, 500);
    NumDataSeries := 2;
    //----------
    DataSeries[1] := 'a * [1 - exp(b * x)] + c<> ?';
    Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 0; ReadEnd[1] := 0;
    LineStyle[1] := 1; LineWidth[1] := 0.25; LineColor[1] := 8421440; MarkStyle[1] := 0; MarkSize[1] := 0;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    p[1,1] := adrt; p[1,2] := bdrt; p[1,3] := cdrt;
    DataSeries[2] := 'a * [1 - exp(b * x)]<> ?';
    Xb[2] := 1; Xt[2] := 0; Yl[2] := 0; Yr[2] := 2; ReadStart[2] := 0; ReadEnd[2] := 0;
    LineStyle[2] := 1; LineWidth[2] := 0.25; LineColor[2] := 16776960; MarkStyle[2] := 0; MarkSize[2] := 0;
    Xmult[2] := 1; Xoffset[2] := 0; Ymult[2] := 1; Yoffset[2] := 0;
    p[2,1] := aLAI; p[2,2] := bLAI;
    //----------
    Height := 60; Width := 80; Lmargin := 30; Tmargin := 20; FLineWidth := 0.4; NumFontSize := 9;
    NumFontName := 'Arial';
    Max[1] := 30; Min[1] := 0; Division[1] := 3; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 30; Min[2] := 0; Division[2] := 3; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] :=  0; Min[3] :=50; Division[3] := 5; Minordivision[3] := 0; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] :=  5; Min[4] := 0; Division[4] := 5; Minordivision[4] := 0; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 5;
    Legend[1] := 'Rooting depth and leaf area index as functions of cumulative transpiration amount\n';
    Legend[2] := 'Cumulative transpiration amount [s][/i]S[i][r]T[_]r[-] (cm)\n';
    Legend[4] := '[T]Rooting depth  [i][r]d[_]rt[-][a][/i] (cm)\n';
    Legend[5] := '[A]Leaf Area Index\n';
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  XYGrapher.Free;
end;

procedure TOutputResults.SetProperty_of_CropCoefficient(FileName: String);
var
  XYGrapher : TXYGrapher;
begin
  XYGrapher := TXYGrapher.Create;
  with XYGrapher do
  begin
    SetFormSize(550, 500);
    NumDataSeries := 1;
    //----------
    DataSeries[1] := 'a * [1 - exp(b * x)] + c - d * x ^ e<> ?';
    Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 0; ReadEnd[1] := 0;
    LineStyle[1] := 1; LineWidth[1] := 0.25; LineColor[1] := 8421440; MarkStyle[1] := 0; MarkSize[1] := 0;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    p[1,1] := aKc; p[1,2] := bKc; p[1,3] := cKc; p[1,4] := dKc; p[1,5] := eKc;
    //----------
    Height := 60; Width := 80; Lmargin := 30; Tmargin := 20; FLineWidth := 0.4; NumFontSize := 9;
    NumFontName := 'Arial';
    Max[1] := 50; Min[1] := 0; Division[1] := 5; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 50; Min[2] := 0; Division[2] := 3; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] :=1.5; Min[3] := 0; Division[3] := 6; Minordivision[3] := 0; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] :=1.5; Min[4] := 0; Division[4] := 6; Minordivision[4] := 0; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 5;
    Legend[1] := 'Crop coefficient as a function of cumulative transpiration amount\n';
    Legend[2] := 'Cumulative transpiration amount [s][/i]S[i][r]T[_]r[-][/i] (cm)\n';
    Legend[4] := '[T]Crop coefficient  [i][r]K[_]c[-][a][/i]\n';
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  XYGrapher.Free;
end;

procedure TOutputResults.SliceRings();
var
  f: TextFile;
  i, i2, j, m: Integer;
  Border: array[0..100] of Double;
  dz_: Double;
  function GetWaterContent(z_: Double): Double;
  var i: Integer;
  begin
    if z_ < z[1] then
      i := 2
    else
      if z_ > z[n] then
        i := n
      else
      begin
        while z_ > z[j] do
          j := j + 1;
        i := j;
      end;
    Result := (W[i] - W[i-1]) / (z[i] - z[i-1]) * (z_ - z[i-1]) + W[i-1];
  end;
begin
  AssignFile(f, Main.GetAbsoluteDirectory(BaseDir, 'BorderDepths.txt'));
  Reset(f);
  i := 0;
  j := 1;
  Border[0] := 0;
  repeat
    i := i + 1;
    Read(f, Border[i]);
  until EOF(f);
  CloseFile(f);
  i2 := i - 1;
  AssignFile(f, Main.GetAbsoluteDirectory(BaseDir, CaseName + '_numerically_sliced_water_content_profile.txt'));
  Rewrite(f);
  for i := 1 to i2 do
  begin
    dum := 0;
    dz_ := (Border[i] - Border[i-1]) / 10;
    for m := 1 to 9 do
      dum := dum + GetWaterContent(Border[i-1] + dz_ * m);
    dum := dz_ / 2 * (GetWaterContent(Border[i-1]) + 2 * dum + GetWaterContent(Border[i]));
    dum := dum / (Border[i] - Border[i-1]);
    Writeln(f, FormatFloat('#0.00', Border[i]) + '  ' + FormatFloat('0.000', dum));
  end;
  CloseFile(f);
end;

function TOutputResults.NextValue: Double;
var
  i: Integer;
begin
  S := TrimLeft(S);
  i := Min(Pos(' ', S), Pos(#9, S));
  while i = 1 do
  begin
    S := Copy(S, 2, 1000);
    i := Min(Pos(' ', S), Pos(#9, S));
  end;
  if i = 0 then
    i := Pos(' ', S);
  if i = 0 then
    i := Pos(#9, S);
  //if (i = 0) and (DecimalSeparator = '.') then
  //  i := Pos(',', S);
  if i = 0 then
    i := 1000;
  Result := StrToFloat(Copy(S, 1, i - 1));
  S := Copy(S, i, 100000);
end;


end.
