unit Pos_Information;

interface

{$mode delphi}{$H+}
uses 
  LResources,Buttons,
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Menus, Spin, IniFiles, Math;

type

  { TPositionInfo }

  TPositionInfo = class(TForm)
    GroupBox2: TGroupBox;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    ed_z_lb: TEdit;
    ed_dz_top: TEdit;
    ed_dz_bot: TEdit;
    Panel1: TPanel;
    Execute: TButton;
    Back: TButton;
    Next: TButton;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label6: TLabel;
    ed_z_border: TEdit;
    GroupBox3: TGroupBox;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    ed_z_obs: TEdit;
    Label5: TLabel;
    seNumLayer: TSpinEdit;
    seBorderNumber: TSpinEdit;
    seNumObs: TSpinEdit;
    seObsptNo: TSpinEdit;
    procedure ed_z_borderExit(Sender: TObject);
    procedure ExecuteClick(Sender: TObject);
    procedure BackClick(Sender: TObject);
    procedure NextClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure ed_z_obsChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure seNumLayerChange(Sender: TObject);
    procedure seBorderNumberChange(Sender: TObject);
    procedure seNumObsChange(Sender: TObject);
    procedure seObsptNoChange(Sender: TObject);
  private
  public
    function Suffix(i: Integer): String ;
  end;

var
  PositionInfo: TPositionInfo;

implementation

uses
  WASH1dMain, RC_Params;

procedure TPositionInfo.FormCreate(Sender: TObject);
begin
  seObsPtNo.MaxValue := MaxNumObs;
end;

procedure TPositionInfo.FormShow(Sender: TObject);
begin
  DialogBox := -1;
  if z_border[1] = 0 then
    z_border[1] := 10;
  if z_lb = 0 then
    if Mode = 2 then
    begin
      z[1]   := 0.2;
      dz_bot := 0.2;
      z_lb := 5;
    end
    else
      z_lb := 40;
  //ed_NumLayer.Text := IntToStr(NumLayer);
  ed_z_lb.Text   := FloatToStr(z_lb);
  ed_dz_top.Text := FloatToStr(z[1]);
  ed_dz_bot.Text := FloatToStr(dz_bot);
  if z_obs[1] = 0 then z_obs[1] := 1;
  ed_z_obs.Text  := FloatToStr(z_obs[1]);
  seObsPtNo.Value := 1;
  if Mode = 2 then
    seNumLayer.Enabled := False;
  seBorderNumberChange(Sender);
  if (Mode = 2) and (IPET_type = 4) then
  begin
    seNumObs.Value := Tensio;
    SeNumObs.Enabled := False;
  end;
  seNumLayerChange(Sender);
  seObsPtNo.MaxValue := NumObs;
  seObsPtNoChange(Sender);
end;
//----------------------------------------------------------
procedure TPositionInfo.ed_z_borderExit(Sender: TObject);
begin
  dumint := seBorderNumber.Value;
  z_border[dumint] := StrToFloat(ed_z_border.Text);
end;
//----------------------------------------------------------
function TPositionInfo.Suffix(i: Integer): String ;
begin
  Case i of
  0..1: Result := 'st';
  2: Result := 'nd';
  3: Result := 'rd';
  else
    Result := 'th';
  end;
end;
//----------------------------------------------------------
procedure TPositionInfo.ed_z_obsChange(Sender: TObject);
begin
  dumint := seObsPtNo.Value;
  if ed_z_obs.Text = '' then
    Exit;
  z_obs[dumint] := StrToFloat(ed_z_obs.Text);
end;
//--------------------------------------------------------------
procedure TPositionInfo.ExecuteClick(Sender: TObject);
begin
  DialogBox := 0;
  Close;
end;
//----------------------------------------------------------
procedure TPositionInfo.BackClick(Sender: TObject);
begin
  GoAhead := False;
  if Mode = 2 then
    DialogBox := 11
  else
    DialogBox := 1;
  Close;
end;
//----------------------------------------------------------
procedure TPositionInfo.NextClick(Sender: TObject);
begin
  GoAhead := True;
  DialogBox := 3;
  Close;
end;
//----------------------------------------------------------
procedure TPositionInfo.FormClose(Sender: TObject; var Action: TCloseAction);
var i: Integer;
begin
  ed_z_borderExit(Sender);
  /// Space Discretization ///
  z_lb := StrToFloat(ed_z_lb.Text);
  z[1] := StrToFloat(ed_dz_top.Text);
  dz_bot := StrToFloat(ed_dz_bot.Text);
  if NumLayer > 1 then
    for i := 1 to NumLayer - 1 do
      if z_border[i] = 0 then
      begin
        ShowMessage('Please input depth of ' + IntToStr(i) + Suffix(i) + ' border.');
        DialogBox := 2;
        Exit
      end;
  for i := 1 to NumObs do
    if z_obs[i] = 0 then
    begin
      ShowMessage('Please input depth of ' + IntToStr(i) + Suffix(i) + ' observation point.');
      DialogBox := 2;
    end;
  if NumLayer = 1 then
    z_border[1] := z_lb;
end;
//===============================================================================================================================

procedure TPositionInfo.seNumLayerChange(Sender: TObject);
begin
  try
    NumLayer := seNumLayer.Value;
  except
    Exit;
  end;
  if NumLayer >= 2 then
    seBorderNumber.MaxValue := NumLayer - 1;
  if NumLayer = 1 then
  begin
    seBorderNumber.Enabled := False;
    ed_z_border.Enabled  := False;
    z_border[1] := z_lb;
  end
  else
  begin
    seBorderNumber.Enabled := True;
    ed_z_border.Enabled  := True;
  end;
end;

procedure TPositionInfo.seBorderNumberChange(Sender: TObject);
begin
  try
    dumint := seBorderNumber.Value;
  except
    Exit;
  end;
  if dumint > NumLayer - 1 then dumint := NumLayer - 1;
  Label5.Caption := Suffix(dumint) + ' border =';
  ed_z_border.Text := FloatToStr(z_border[dumint]);
end;

procedure TPositionInfo.seNumObsChange(Sender: TObject);
begin
  try
    NumObs := seNumObs.Value;
  except
    Exit;
  end;
  seObsPtNo.MaxValue := NumObs;
  if NumObs = 1 then
    seObsPtNo.Enabled := False
  else
    seObsPtNo.Enabled := True;
end;

procedure TPositionInfo.seObsptNoChange(Sender: TObject);
begin
  {
  dumint := 1;
  try
    dumint := seObsPtNo.Value;
  except
    Exit;
  end;
  dumint := Max(NumObs, dumint);
  }
  Label10.Caption := Suffix(seObsPtNo.Value) + ' obs. point =';
  if z_obs[seObsPtNo.Value] > 0 then
    ed_z_obs.Text := FloatToStr(z_obs[seObsPtNo.Value])
  else
    ed_z_obs.Text := '1';
end;

Initialization
{$I Pos_Information.lrs}
end.
