unit RC_Params; 

{$mode delphi}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  StdCtrls, Buttons, ComCtrls, Spin, IniFiles, Math
  {$ifdef mswindows}
  , Registry
  {$endif}
  , HyperText;

type

  { TSoil }
  TSegments = array[-5..5,0..140] of Double;

  TSoil = class(TForm)
    HyperText: THyperText;
    Image3: TImage;
    Panel1: TPanel;
    Execute: TButton;
    Back: TButton;
    Next: TButton;
    Image1: TImage;
    Image2: TImage;
    Edit1: TEdit;
    Edit2: TEdit;
    Edit3: TEdit;
    Edit4: TEdit;
    Edit5: TEdit;
    Edit6: TEdit;
    Edit7: TEdit;
    Label1: TLabel;
    Edit8: TEdit;
    Label2: TLabel;
    Edit9: TEdit;
    Label3: TLabel;
    Edit10: TEdit;
    BitBtn1: TBitBtn;
    StatusBar1: TStatusBar;
    BitBtn2: TBitBtn;
    OpenDialog1: TOpenDialog;
    Label6: TLabel;
    cbRF: TComboBox;
    Label7: TLabel;
    CheckBox1: TCheckBox;
    CheckBox2: TCheckBox;
    CheckBox3: TCheckBox;
    Label8: TLabel;
    Panel2: TPanel;
    CheckBox4: TCheckBox;
    Edit11: TEdit;
    Edit12: TEdit;
    Edit13: TEdit;
    Edit14: TEdit;
    Edit15: TEdit;
    Label10: TLabel;
    Label11: TLabel;
    Panel3: TPanel;
    seLayerNumber: TSpinEdit;
    CheckBox5: TCheckBox;
    procedure ExecuteClick(Sender: TObject);
    procedure BackClick(Sender: TObject);
    procedure NextClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cbRFClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure BitBtn1Click(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
    procedure ed_NumLayerChange(Sender: TObject);
    procedure cbRFChange(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure CheckBox2Click(Sender: TObject);
    procedure Panel2Click(Sender: TObject);
    procedure CheckBox4Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure seLayerNumberChange(Sender: TObject);
  private
    procedure GetParameters;
    procedure ShowEquation;
    procedure Language;
    procedure SetProperty_of_RetCurveFig(layer: Integer; FileName: String);
    function  SetLengthRJ(S: String; i: Shortint): String;
  public
    Modified, Mode, NumLayer, ShowMode, num_add_ret_data :Shortint;
    WsatFix, LlambdaFix, zetafix, spFix, WrFix, IPET_type, Independent, Linked_m: Shortint;
    num_ret_data                                                : Integer;
    DialogBox: Shortint;
    CaseName, ExeDir, BaseDir: String;
    SoilName, SoilComment: array[1..5] of String;
    Prepared: array[1..5] of Boolean;
    RF, KF : array[1..5] of Shortint;
    Tref, MyuRef, Llambda, LogOfLlambda                              : array[1..5] of Double;
    Wsat, zeta, Wr                                                          : array[1..5] of Double;
    alpha, np, mp, sp, alpha1, np1, alpha2, np2, mp1, mp2          : array[-5..5] of Double;  //-5
    Ksat, ak, bk, ck                                                  : array[1..5] of Double;
    a_tau, b_tau, c_tau, d_tau, dispersivity                                                : array[1..5] of Double;
    Akh, Bkh, Ckh, Dkh, Ekh, AlbedoMax, AlbedoMin, Aal, Bal, ThickAlb, Aen, Ben, Cen, Den, Een : array[1..5] of Double;
    Water, Heat, GoAhead, Segments : Boolean;
    t_unit, LANG: Integer;
    fl_retention, VirtualFile, Slash, MyDocDir: String;
    Cwsat: Double;
    Wseg, Slope: TSegments;
    lnPseg: array[1..140] of Double;
    function Water_Content(P, T, Wsat_, zeta, Wr_: Double; j: Integer; Drying: Boolean): Double;
    function Pressure_Head(W, T, Wsat, zeta, Wr: Double; j: Integer; Drying: Boolean): Double;
    function Water_Capacity(P, T, Wsat_, zeta, Wr_: Double; j: Integer; Drying: Boolean): Double;
    function HydraulicConductivity(W, T: Double; j: Integer): Double;
    function Water_content_at_given_K(K: Double; j: Integer): Double;
    function ConvertDecimalSeparator(S: String): String;
    procedure LoadProperties(j: Shortint);
    procedure SaveProperties(j: Shortint; WriteOnFile: Boolean);
    procedure Initialize;
  end;

var
  Soil: TSoil;
  j : Shortint;
  dumstr: String;
  dumint: Integer;
  AllowModified : Boolean;
  Msg: Array[1..20] of String;
  MLB: Boolean = False;//True; // for fingering study

implementation

uses FigFileMaker;

//------------------------------------------------------
procedure TSoil.FormCreate(Sender: TObject);
begin
  Initialize;
end;

procedure TSoil.Initialize;
var i: Integer;
{$ifdef MSWINDOWS}
  Reg: TRegistry;
{$endif}
begin
  AllowModified := False;
  Mode := 1; water := True;  heat := True; NumLayer := 1; t_unit := 3600;
  ShowMode := 0;
  num_ret_data := 0; num_add_ret_data := 0;
  WsatFix := 1; LlambdaFix := 1; spFix := 1; WrFix := 1;
  ExeDir := ExtractFileDir(Application.ExeName);
  {$ifdef Linux}
  BaseDir := GetEnvironmentVariable('HOME') + '/.' + ExtractFileName(Application.ExeName);
  {$endif}
  SoilName[1] := ''; SoilComment[1] := '';
  RF[1] := 2;
  Wsat[1] := 0.4; zeta[1] := 0.1; Wr[1] := 0;
  Tref[1] := 298.2; Llambda[1] := 10000000;
  alpha[1] := 0.01 ; np[1] := 1.5 ; mp[1] := 0.3333;
  alpha[-1] := 0.02; np[-1] := 1.5; mp[-1] := 0.3333;
  sp[1] := 1 ; alpha1[1]  := 0.01 ; np1[1]  := 1.5 ; alpha2[1]  := 0.001 ; np2[1]  := 1.5 ; mp1[1]  := 0.3333; mp2[1]  := 0.3333;
  sp[-1]:= 1 ; alpha1[-1] := 0.02 ; np1[-1] := 1.5 ; alpha2[-1] := 0.001 ; np2[-1] := 1.5 ; mp1[-1] := 0.3333; mp2[-1] := 0.3333;
  Ksat[1] := 0.00028;
  ck[1] := 12;
  fl_retention := '';
  Independent := 1;
  Linked_m := 0;
  LANG := 1;
  Cwsat := 0.000000001; // 0.000001;
  DialogBox := -1;
  Segments := True;
  for i := 1 to 5 do
  begin
    RF[i] := 1;
    KF[i] := 1;
    Prepared[i] := False;
    MyuRef[i] := 0;
  end;
  {$ifdef Linux}
  MyDocDir := GetEnvironmentVariable('HOME');
  Slash := '/';
  {$else}
  Reg := TRegistry.Create;
  with Reg do
  try
    RootKey := HKEY_CURRENT_USER;
    OpenKey('Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders', False);
    MyDocDir := ReadString('Personal') + '\';// lpszPath; // Modified 02/01/05
    CloseKey;
  finally
    Free;
  end;
  Slash := '\';
  {$endif}
  Msg[1] := '&Execute';
  Msg[2] := '&Next >>';
  Msg[3] := '<< &Back';
  Msg[4] := 'Results of parameter estimation';
  Msg[5] := '&Figure';
  Msg[6] := '&O K';
  Msg[7] := '&Print';
  Msg[8] := '&Cancel';
  Msg[9] := 'Please select file which holds properties of the soil';
  Msg[10]:= 'Soil water retension curve of ';
  Msg[11]:= '  V.W.C.  presure head(cm)     Kdry(cm/s)     Kwet(cm/s)';
  Msg[12]:= '[r][b]Fig.2[ ]  Soil water retention curve of ';
  Msg[13]:= '\n';
  Msg[14]:= 'Volumetric water content [s]q[a] (cm[^]3[-]/cm[^]3[-])\n';
  Msg[15]:= 'Pressure head [s][~]y[a][.](cm)\n';
  Msg[16]:= 'Additional \n';
end;

procedure TSoil.FormShow(Sender: TObject);
var i: Integer;
begin
  StatusBar1.Visible := False;
  cbRF.Enabled := True;
  if Mode = 1 then
    dumstr := 'Are properties of the soil already held in a file?'
  else
    dumstr := 'Do you use parameters of other soil for initial value?';
  Application.ProcessMessages;
  if (CaseName = BaseDir + Slash + 'Untitled') and (SoilName[1] = '') then
  begin
    if water then
      if Application.MessageBox(PChar(dumstr), 'If datafile of the soil is already prepared in "Soils"folder, press [(Y)] button.',
                                4) //Yes+No
        = 6 then //Yes
      begin
        OpenDialog1.InitialDir := ExeDir + Slash + 'Soils';
        if OpenDialog1.Execute = true then
          for i := 1 to NumLayer do
          begin
            SoilName[i] := ExtractFileName(ChangeFileExt(OpenDialog1.FileName, ''));
            LoadProperties(i);
          end;
      end;
   end;
  if NumLayer = 1 then
  begin
    Label6.Visible := False;
    seLayerNumber.Visible := False;
    Label7.Top := 30;
    cbRF.Top := 46;
  end
  else
  begin
    Label7.Top := 48;
    cbRF.Top := 64;
  //  for i := 2 to NumLayer do RF[i] := 1;
  end;
  if (SoilName[1] = '') and (Mode = 2) then
  begin
    ShowMessage('Please input initial values of parameters to be estimated');
    StatusBar1.Visible := True;
    StatusBar1.SimpleText := 'Initial values of parameters to be estimated';
  end;
  if j = 0 then
    j := 1;
  if ShowMode = 1 then
  begin
    cbRF.Enabled  := False;
    Edit1.Enabled := False;
    Edit2.Enabled := False;
    Edit3.Enabled := False;
    Edit4.Enabled := False;
    Edit5.Enabled := False;
    Edit6.Enabled := False;
    Edit7.Enabled := False;
    Edit8.Enabled := False;
    Edit9.Enabled := False;
    Edit10.Enabled:= False;
    Edit11.Enabled:= False;
    Edit12.Enabled:= False;
    Edit13.Enabled:= False;
    Edit14.Enabled:= False;
    Edit15.Enabled:= False;
    StatusBar1.Visible := True;
    Execute.TabOrder := 0;
  end;
  if (Mode = 2) and (IPET_type <> 5) then
  begin
    Label8.Visible    := True;
    CheckBox1.Visible := True;
    CheckBox2.Visible := True;
    if RF[j] = 1 then
      CheckBox5.Visible := True
    else
      CheckBox5.Visible := False;
    if WsatFix = 1 then
      CheckBox1.Checked := True;
    if (LlambdaFix = 1) or (spFix = 1) then
      CheckBox2.Checked := True;
    if WrFix   = 1 then
      CheckBox3.Checked := True;
    if ZetaFix = 1 then
      CheckBox5.Checked := True
    else
      CheckBox5.Checked := False;
  end
  else
  begin
    Label8.Visible    := False;
    CheckBox1.Visible := False;
    CheckBox2.Visible := False;
    CheckBox3.Visible := False;
    CheckBox5.Visible := False;
  end;
  seLayerNumber.Value := j;
  seLayerNumber.MaxValue := NumLayer;
  if Mode = 2 then
  begin
    if (IPET_type <> 1) and (IPET_type <> 5) and (Independent <> 1) and (IPET_type < 7) then
    begin
      RF[1] := 2;
      cbRF.Enabled := False;
      Edit2.Enabled := False;
      Edit5.Enabled := False;
      Edit6.Enabled := False;
      CheckBox2.Enabled := False;
      if sp[1] <> 1 then
      begin
        sp[1] := 1;
        ShowMessage('Only van Genuchten - Mualem soil hydraulic relationship is allowed for simultaneous estimation of hydraulic properties.');
      end;
    end;
  end;
 // CheckBox4Click(Sender);
  Image3.Canvas.Brush.Color := Self.Color;
  Image3.Canvas.Brush.Style := bsSolid;
  Image3.Canvas.FillRect(Rect(0, 0, Image3.Width, Image3.Height));
  HyperText.TargetCanvas := Image3.Canvas;
  with HyperText do
  begin
    Write('[i]θ[/i]: Volumetric water content', 0, 0, 0);
    Write('[i]ψ[/i]: Pressure head(cm)', 0, 30, 0);
    Write('[i][r] m[/i] = 1 - 1 / [i]n[/i]', 0, 60, 0);
  end;
  ShowEquation;
  Language;
  AllowModified := True;
end;

//------------------------------------------------------
procedure TSoil.ExecuteClick(Sender: TObject);
begin
  DialogBox := 0;// Close;
end;
//------------------------------------------------------
procedure TSoil.BackClick(Sender: TObject);
begin
  if ShowMode > 0 then
    BitBtn1Click(Sender)
  else
  begin
    GoAhead := False;
    DialogBox := 2;
    Close;
  end;
end;
//------------------------------------------------------
procedure TSoil.NextClick(Sender: TObject);
begin
  case ShowMode of
  0:begin
      GoAhead := True;
      DialogBox := 4;
      Close;
    end;
  1:;
  2:Close;
  end;
end;
//------------------------------------------------------

procedure TSoil.cbRFClick(Sender: TObject);
begin
  Case cbRF.itemindex of
    0: RF[j] := 1;
    1: RF[j] := 2;
  end;
  ShowEquation;
end;
//------------------------------------------------------
procedure TSoil.ShowEquation;
var
  dumstr2: string;
  y: array[0..7] of Integer;
  i: Integer;
begin
  y[0] := 0;
  for i := 1 to 7 do
    y[i] := y[i-1] + 33;
  dumstr := '';
  cbRF.itemindex := RF[j] - 1;
  if j = 0 then
    j := 1;
  if Tref[j] = 0 then
    Tref[j] := 298.2;
  HyperText.TargetCanvas := Image2.Canvas;
  Image2.Canvas.Brush.Color := clForm;//Self.Color;
  Image2.Canvas.Brush.Style := bsSolid;
  Image2.Canvas.FillRect(Rect(0, 0, Image2.Width, Image2.Height));
 // Image2.Canvas.Rectangle(Rect(0, 0, Image2.Width, Image2.Height));
  case RF[j] of
  1:begin
      dumstr  := 'TrueRet.png';
      with HyperText do
      begin
        Write('[i]θ[r][_]sat[-][/i] = ', 0, y[0], 0);
        Write('[i]ζ[/i][r] = ', 0, y[1], 0);
        Write('[i]α[/i][r] = ', 0, y[2], 0);
        Write('[i][r]n[/i] =', 0, y[3], 0);
        Write('[i]Λ[/i][r] =', 0, y[4], 0);
        if not CheckBox4.Checked then
          Write('[i]m[/i] =', 0, y[5], 0);
      end;
      if np[j] = 0 then
        np[j] := 2;
      Edit2.Text := Copy(FloatToStr(zeta[j]), 1, 8);
      Edit3.Text := Copy(FormatFloat('0.######', alpha[j]), 1, 8);
      Edit4.Text := Copy(FloatToStr(np[j]), 1, 8);
      Edit5.Text := Copy(FloatToStr(Llambda[j]), 1, 8);
      Edit6.Text := Copy(FloatToStr(mp[j]), 1, 8);
      Edit12.Text :=  Copy(FloatToStr(alpha[-j]), 1, 8);
      Edit13.Text :=  Copy(FloatToStr(np[-j]), 1, 8);
      Edit15.Text := Copy(FloatToStr(mp[-j]), 1, 8);
      if Linked_m = 1 then
      begin
        CheckBox4.Checked := True;
        Edit6.Enabled     := False;
      end
      else
      begin
        CheckBox4.Checked := False;
      end;
      Edit7.Visible := False;
      Edit11.Visible := False;
      Edit14.Visible := False;
      CheckBox4.Visible := True;
      CheckBox5.Visible := True;
      if Mode = 2 then
      begin
        CheckBox2.Top := 248;
        CheckBox3.Visible := False;
      end;
    end;
  2:begin
      dumstr  := 'bimodal RF.png';
      with HyperText do
      begin
        Write('θ[i][r][_]sat[-][/i] = ', 0, y[0], 0);
        Write('[i]s[/i] = ', 0, y[1], 0);
        Write('α[_]1[-][r] = ', 0, y[2], 0);
        Write('[i][r]n[/i][_]1[-] =', 0, y[3], 0);
        Write('α[_]2[-][r] = ', 0, y[4], 0);
        Write('[i][r]n[/i][_]2[-] =', 0, y[5], 0);
        Write('θ[i][r][_]r[-][/i] = ', 0, y[6], 0);
      end;
      Edit6.Visible := True;
      Edit7.Visible := True;
      Edit11.Visible :=  True;
      Edit14.Visible :=  True;
      if np1[j] = 0 then np1[j] := 2;
      if np2[j] = 0 then np2[j] := 2;
      if sp[j]  = 0 then sp[j] := 1;
      Edit2.Text := Copy(FloatToStr(sp[j]), 1, 8);
      Edit3.Text := Copy(FloatToStr(alpha1[j]), 1, 8);
      Edit4.Text := Copy(FloatToStr(np1[j]), 1, 8);
      Edit5.Text := Copy(FormatFloat('0.#####', alpha2[j]), 1, 8);
      Edit6.Text := Copy(FloatToStr(np2[j]), 1, 8);
      Edit11.Text :=  Copy(FloatToStr(sp[-j]), 1, 8);
      Edit12.Text :=  Copy(FloatToStr(alpha1[-j]), 1, 8);
      Edit13.Text :=  Copy(FloatToStr(np1[-j]), 1, 8);
      Edit14.Text :=  Copy(FormatFloat('0.#####', alpha2[-j]), 1, 8);
      Edit15.Text :=  Copy(FloatToStr(np2[-j]), 1, 8);
      Edit7.Text :=  Copy(FloatToStr(Wr[j]), 1, 8);
      CheckBox4.Visible := False;
      CheckBox5.Visible := False;
      if Mode = 2 then
      begin
        CheckBox2.Top := 148;
        CheckBox3.Visible := True;
      end;
    end;
  end;
  dumstr  := ExeDir + Slash + 'Equations' + Slash + dumstr;
  //dumstr2 := ExeDir + Slash + 'Equations' + Slash + dumstr2;
  image1.picture.loadfromfile(dumstr);
  //image2.picture.loadfromfile(dumstr2);
  //dumstr := ExeDir + Slash + 'Equations' + Slash + 'RC params0.png';
  //image3.picture.loadfromfile(dumstr);
  Edit1.Text := Copy(FloatToStr(Wsat[j]), 1, 8);
  Edit8.Text := FormatFloat('##.0', Tref[j]-273.2);
  if SoilName[j] <> '' then
    Edit9.Text := SoilName[j]
  else
    Edit9.Text := 'A_Soil';
  Edit10.Text :=  SoilComment[j];
  if Edit7.Visible then
    Panel3.Visible := False
  else
    Panel3.Visible := True;
  if (Mode = 2) and (IPET_type <> 3) and (IPET_type <> 5) and (IPET_type < 7) then
  begin
    Label11.Visible := False;
    Edit11.Visible := False;
    Edit12.Visible := False;
    Edit13.Visible := False;
    Edit14.Visible := False;
    Edit15.Visible := False;
  end;
end;
//----------------------------------------------------------
procedure TSoil.BitBtn1Click(Sender: TObject);
begin
  GetParameters;
  dumint := j;
  if Mode = 2 then
    dumint := 1;
  Prepared[j] := False;
  SetProperty_of_RetCurveFig(dumint, CaseName + '_RetCurve');
end;
//----------------------------------------------------------
procedure TSoil.GetParameters;
begin
  Wsat[j]  := StrToFloat(Edit1.Text);
  Tref[j]  := StrToFloat(Edit8.Text) + 273.2;
  SoilName[j]    := Edit9.Text;
  SoilComment[j] := Edit10.Text;
  Case RF[j] of
    1:begin
        zeta[j]  := StrToFloat(Edit2.Text);
        alpha[j]  := StrToFloat(Edit3.Text);
        np[j]     := StrToFloat(Edit4.Text);
        Llambda[j]  := StrToFloat(Edit5.Text);
        LogOfLlambda[j] := ln(Llambda[j]);
        if Linked_m = 1 then
          mp[j] := 1 - 1 / np[j]
        else
          mp[j] := StrToFloat(Edit6.Text);
        if(Mode <> 2) or (IPET_type = 3) or (IPET_type = 5) then
        begin
          alpha[-j]  := StrToFloat(Edit12.Text);
          np[-j]     := StrToFloat(Edit13.Text);
          if Linked_m = 1 then
            mp[-j] :=  1 - 1 / np[-j]
          else
            mp[-j] :=  StrToFloat(Edit15.Text);
        end;
      end;
    2:begin
        sp[j]     := StrToFloat(Edit2.Text);
        alpha1[j] := StrToFloat(Edit3.Text);
        np1[j]    := StrToFloat(Edit4.Text);
        alpha2[j] := StrToFloat(Edit5.Text);
        np2[j]    := StrToFloat(Edit6.Text);
        Wr[j]     := StrToFloat(Edit7.Text);
        if MLB then
        begin
          mp1[j] := 1;
          mp2[j] := 1;
        end
        else
        begin
          mp1[j] := 1 - 1 / np1[j];
          mp2[j] := 1 - 1 / np2[j];
        end;
        if(Mode <> 2) or (IPET_type = 5) then
        begin
          alpha2[-j] :=  StrToFloat(Edit14.Text);
          alpha1[-j] :=  StrToFloat(Edit12.Text);
          sp[-j]    := StrToFloat(Edit11.Text);
          np1[-j]   := StrToFloat(Edit13.Text);
          np2[-j]   := StrToFloat(Edit15.Text);
          mp1[-j] :=  1 - 1 / np1[-j];
          mp2[-j] :=  1 - 1 / np2[-j];
        end;
      end;
  end;
end;
//------------------------------------------------------
procedure TSoil.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  if Mode = 2 then
  begin
    if CheckBox1.Checked = True then
      WsatFix := 1
    else
      WsatFix := 0;
    if CheckBox2.Checked = True then
      if RF[j] = 1 then
        LlambdaFix := 1
      else
        spFix := 1
    else
    begin
      LlambdaFix := 0;
      spFix   := 0;
    end;
    if CheckBox3.Checked = True then
      WrFix   := 1
    else
      WrFix   := 0;
    if CheckBox5.Checked = True then
      ZetaFix := 1
    else
      ZetaFix := 0;
  end;
  GetParameters;
end;
//----------------------------------------------------------
procedure TSoil.BitBtn2Click(Sender: TObject);
begin
  ShowMessage(Msg[9]);
  OpenDialog1.InitialDir := ExeDir + Slash + 'Soils';
  OpenDialog1.FileName   := 'Please double-click the file above.';
  if OpenDialog1.Execute = True then
    SoilName[j] := ExtractFileName(ChangeFileExt(OpenDialog1.FileName, ''));
  LoadProperties(j);
  FormShow(Sender);
end;
//----------------------------------------------------------
procedure TSoil.ed_NumLayerChange(Sender: TObject);
begin
end;
//----------------------------------------------------------
procedure TSoil.cbRFChange(Sender: TObject);
begin
  RF[j] := cbRF.ItemIndex + 1;
  ShowEquation;
end;
//----------------------------------------------------------
procedure TSoil.CheckBox2Click(Sender: TObject);
begin
end;
//----------------------------------------------------------
procedure TSoil.CheckBox4Click(Sender: TObject);
begin
  if (CheckBox4.Checked = True) then//and (RF[j] = 2) then
  begin
    Linked_m := 1;
    Edit6.Enabled   := False;
    Edit15.Enabled  := False;
  end
  else
  begin
    Linked_m := 0;
    Edit6.Enabled   := True;
    Edit15.Enabled  := True;
  end;
  if RF[j] = 2 then
    Edit15.Visible  := True;
  Edit6.Text  := Copy(FloatToStr(mp[j]), 1, 8);
  Edit15.Text := Copy(FloatToStr(mp[-j]), 1, 8);
end;
//===============================================================================================================================
procedure TSoil.Language;
var
{$ifdef MSWINDOWS}
  adjuster : Double;
{$endif}
  i : Shortint;
  IniFile: TIniFile;
  S, ResourceFileName: String;
begin
  S := Self.Name;
  ResourceFileName := ChangeFileExt(Application.ExeName, '.lng');
  if FileExists(ResourceFileName) then
  begin
    IniFile := TIniFile.Create(ResourceFileName);
    with IniFile do
      try
        Caption            := ReadString(S, 'Caption', '');
        Label1.Caption     := ReadString(S, 'Label1.Caption', '');
        Label2.Caption     := ReadString(S, 'Label2.Caption', '');
        Label3.Caption     := ReadString(S, 'Label3.Caption', '');
        Label6.Caption     := ReadString(S, 'Label6.Caption', '');
        Label7.Caption     := ReadString(S, 'Label7.Caption', '');
        Label8.Caption     := ReadString(S, 'Label8.Caption', '');
        BitBtn1.Hint       := ReadString(S, 'BitBtn1.Hint', '');
        CheckBox1.Hint     := ReadString(S, 'CheckBox1.Hint', '');
        CheckBox2.Hint     := ReadString(S, 'CheckBox2.Hint', '');
        CheckBox3.Hint     := ReadString(S, 'CheckBox3.Hint', '');
      //  Font.Name          := ReadString(S, 'Soil.Font.Name', '');
        i := cbRF.ItemIndex;
        cbRF.Items.Text    := StringReplace(ReadString(S, 'cbRF.Items.Text', ''), '<br>', #13#10, [rfReplaceAll, rfIgnoreCase]);
        cbRF.ItemIndex := i;
        for i := 1 to 17 do
          Msg[i] := StringReplace(ReadString(S, 'Msg' + IntToStr(i), ''), '<br>', #13#10, [rfReplaceAll, rfIgnoreCase]);
      finally
        Free;
      end;
  end;
  if LANG = 128 then
  begin
    {$ifdef MSWINDOWS}
    adjuster := Screen.Width / 1024.0;
    //if GetDeviceCaps(Canvas.Handle, VERTSIZE) < 130 then
    //  Font.Size := Trunc(Font.Size * adjuster);
    {$endif}
    Edit8.Top := 338;
  end;
  case ShowMode of
  0:begin
      Execute.Caption  := Msg[1];
      Next.Caption     := Msg[2];
      Back.Caption     := Msg[3];
    end;
  1:begin
      StatusBar1.SimpleText := Msg[4];
      Back.Caption     := Msg[5];
      Execute.Caption  := Msg[6];
      Next.Caption     := Msg[7];
    end;
  2:begin
      Back.Caption     := Msg[5];
      Execute.Caption  := Msg[6];
      Next.Caption     := Msg[8];
    end
  end;
  CheckBox2.Hint := CheckBox1.Hint;
  CheckBox3.Hint := CheckBox1.Hint;
end;

procedure TSoil.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var dum: Double;
begin
  if Key = 113 then //[F2]
  begin
    GetParameters;
    dum := StrToFloat(InputBox('Water content -> Pressure head', 'What is the water content?', '0.3'));
    dum := Pressure_Head(dum, Tref[j], Wsat[j], zeta[j], Wr[j], j, True);
    ShowMessage('Pressure head = ' + FloatToStr(dum));
  end;
end;
//----------------------------------------------------------
procedure TSoil.Panel2Click(Sender: TObject);
var dum: Double;
begin
  GetParameters;
  dum := StrToFloat(InputBox('Pressure head -> Water content', 'What is the pressure head?', '-1000'));
  dum := Water_Content(dum, Tref[j], Wsat[j], zeta[j], Wr[j], j, True);
  ShowMessage('Water content = ' + FloatToStr(dum));   
end;
//---------------------------------------------------------
procedure TSoil.LoadProperties(j: Shortint);
var
  i: Integer;
  f: Textfile;
  Version: Single;
  PropertiesFile: String;
  SL: TStringList;
  function NextValue: Double;
  var
    i: Integer;
  begin
    dumstr := TrimLeft(dumstr);
    i := Pos(' ', dumstr);
    if i = 0 then
      i := 100;
    Result := StrToFloat(Copy(dumstr, 1, i - 1));
    dumstr := Copy(dumstr, i, 100000);
  end;
begin
  Version := 0;
  PropertiesFile := BaseDir + Slash + SoilName[j] + '.dat';
  if Slash = '' then
    Initialize;
  if FileExists(PropertiesFile) = False then
    PropertiesFile := ExeDir + Slash  + 'Soils' + Slash + SoilName[j] + '.dat';
  if not FileExists(PropertiesFile) then
  begin
    ShowMessage(Msg[9]);
    with OpenDialog1 do
    begin
      FileName   := 'Please double-click the file above.';
      InitialDir := ExeDir + Slash + 'Soils';
      if Execute = True then
      begin
        SoilName[j] := ExtractFileName(ChangeFileExt(OpenDialog1.FileName, ''));
        PropertiesFile := ExeDir + Slash + 'Soils' + Slash + SoilName[j] + '.dat';
      end
      else
        Exit;
    end;
  end;
  SL := TStringList.Create;
  try
    SL.LoadFromFile(PropertiesFile);
    SL.Text := ConvertDecimalSeparator(SL.Text);
   // ShowMessage(SL.Text);
    SL.SaveToFile('temp.txt');
  finally
    SL.Free;
  end;
  AssignFile(f, 'temp.txt');
  Reset(f);
  Readln(f, dumstr);
  i := Pos('version', dumstr);
  if i > 0 then
    Version := StrToFloat(Copy(dumstr, i + 7, 5));
  Readln(f, SoilComment[j]);
  //ShowMessage(DecimalSeparator);
  for i := 1 to 3 do
    Readln(f, dumstr);
  RF[j] := Trunc(NextValue);
  Case RF[j] of
  1:begin
      Tref[j] := NextValue;
      Wsat[j] := NextValue;
      zeta[j] := NextValue;
      alpha[j]:= NextValue;
      np[j]   := NextValue;
      mp[j]   := NextValue;
      Llambda[j] := NextValue;
      if Version > 1.001 then
      begin
        Readln(f, dumstr); //Readln(f, dumstr);
        alpha[-j] := NextValue;
        np[-j]    := NextValue;
        mp[-j]    := NextValue;
      end
      else
      begin
        alpha[-j] := 2 * alpha[j];
        np[-j] := np[j];
        mp[-j] := mp[j];
      end;
     // mp[j] := mp[j]; // ? 1 - 1 / np[j];
      if Llambda[j] < 10000 then
        Llambda[j] := 10000000;
      LogOfLlambda[j] := ln(Llambda[j]);
    end;
  2:begin
      Tref[j]  := NextValue;
      Wsat[j]  := NextValue;
      sp[j]    := NextValue;
      alpha1[j]:= NextValue;
      np1[j]   := NextValue;
      alpha2[j]:= NextValue;
      np2[j]   := NextValue;
      Wr[j]    := NextValue;
      if Version > 1.001 then
      begin
        Readln(f, dumstr);
        sp[-j]     := NextValue;
        alpha1[-j] := NextValue;
        np1[-j]    := NextValue;
        alpha2[-j] := NextValue;
        np2[-j]    := NextValue;
      end
      else
      begin
        sp[-j] := sp[j];
        alpha1[-j] := 2 * alpha1[j];
        np1[-j] := np1[j];
        alpha2[-j] := 2 * alpha2[j];
        np2[-j] := np2[j];
      end;
      if MLB then
      begin
        mp1[j] := 1;
        mp2[j] := 1;
      end
      else
      begin
        mp1[j] := 1 - 1 / np1[j];
        mp2[j] := 1 - 1 / np2[j];
      end;
      if (Mode <> 2) or (IPET_type = 4) or (IPET_type > 6)then
      begin
        mp1[-j] :=  1 - 1 / np1[-j];
        mp2[-j] :=  1 - 1 / np2[-j];
      end;
    end;
  end;
  for i := 1 to 3 do
    Readln(f, dumstr);
  KF[j]   := Trunc(NextValue);
  Ksat[j] := NextValue;
  ak[j]   := NextValue;
  bk[j]   := NextValue;
  if KF[j] = 1 then
    ck[j] := NextValue;
  for i := 1 to 3 do
    Readln(f, dumstr);
  if version < 1.35 then
  begin
    a_tau[j] := NextValue;
    b_tau[j] := NextValue;
    dispersivity[j] := NextValue;
  end
  else
  begin
    a_tau[j] := NextValue;
    b_tau[j] := NextValue;
    c_tau[j] := NextValue;
    d_tau[j] := NextValue;
    dispersivity[j] := NextValue;
  end;
  for i := 1 to 3 do
    Readln(f, dumstr);
  Akh[j] := NextValue;
  Bkh[j] := NextValue;
  Ckh[j] := NextValue;
  Dkh[j] := NextValue;
  Ekh[j] := NextValue;
  for i := 1 to 3 do
    Readln(f, dumstr);
  AlbedoMax[j] := NextValue;
  AlbedoMin[j] := NextValue;
  Aal[j]       := NextValue;
  Bal[j]       := NextValue;
  ThickAlb[j]  := NextValue;
  for i := 1 to 3 do
    Readln(f, dumstr);
  Aen[j] := NextValue;
  Ben[j] := NextValue;
  Cen[j] := NextValue;
  Den[j] := NextValue;
  Een[j] := NextValue;
  CloseFile(f);
  dumstr := ExtractFileName(PropertiesFile);
  SoilName[j] := Copy(dumstr,1,Length(dumstr)-4);
end;

procedure TSoil.SaveProperties(j: Shortint; WriteOnFile: Boolean);
var SL: TStringList;
begin
  SL := TStringList.Create;
  try
    with SL do
    begin
      Add('>>>>>>>   Properties of Soil (version 1.40 format) <<<<<<<');
      Add(SoilComment[j]);
      Add('====  soil water retension curve  ====');
      Case RF[j] of
      1:begin
          Add('  RF  Tref   Wsat   zeta    alpha     n     m   Llambda');
          Add('  '+IntToStr(RF[j])+'   '+FloatToStr(Tref[j])+'  '+ FloatToStr(Wsat[j])+'   '+
              FloatToStr(zeta[j])+'  '+ FloatToStr(alpha[j])+'  '+FloatToStr(np[j])+'  '+FormatFloat('0.000', mp[j])+'  '+FloatToStr(Llambda[j]));
          Add('                         '+ FloatToStr(alpha[-j])+'  '+FloatToStr(np[-j])+'  '+FormatFloat('0.000', mp[-j]));
        end;
      2:begin
          Add('  RF   Tref   Wsat    s   alpha1   n1     alpha2   n2    Wr');
          Add('  '+IntToStr(RF[j])+'   '+FloatToStr(Tref[j])+'  '+ FloatToStr(Wsat[j])+'   '+
              FloatToStr(sp[j])+'   '+FloatToStr(alpha1[j])+'   '+FloatToStr(np1[j])+'    '+
              FloatToStr(alpha2[j])+'   '+FloatToStr(np2[j])+'   '+FloatToStr(Wr[j]));
          Add('                        '+FloatToStr(sp[-j])+'   '+FloatToStr(alpha1[-j])+'   '+FloatToStr(np1[-j])+'    '+
              FloatToStr(alpha2[-j])+'   '+FloatToStr(np2[-j]));
        end;
      end;
      Add('=====  Hydraulic Conductivity  =====');
      if (KF[j] = 2) or (KF[j] = 3) then
        begin
          Add('  KF   Ksat         ak       bk');
          Add('  '+IntToStr(KF[j])+'    '+FormatFloat('0.000000', Ksat[j])+'    '+FloatToStr(ak[j])+'    '+FloatToStr(bk[j]));
        end
      else
        begin
          Add('  KF   Ksat       ak       bk      ck');
          Add('  '+IntToStr(KF[j])+'    '+FloatToStr(Ksat[j])+'    '+FloatToStr(ak[j])+'    '+FloatToStr(bk[j])+'    '+FloatToStr(ck[j]));
        end;
      Add('=====  Solute Movement Property  =====');
      Add(' a_s    b_s    c_s    d_s    dispersivity');
      Add('  ' + FloatToStr(a_tau[j]) + '     ' + FloatToStr(b_tau[j]) + '     ' + FloatToStr(c_tau[j]) + '     ' + FloatToStr(d_tau[j]) + '     ' + FloatToStr(dispersivity[j]));
      Add('=====  Thermal Conductivity  =====');
      Add('  a_h      b_h      c_h      d_h      e_h');
      Add('  '+FloatToStr(Akh[j])+'    '+FloatToStr(Bkh[j])+'    '+FloatToStr(Ckh[j])+'    '+FloatToStr(Dkh[j])+'    '+FloatToStr(Ekh[j]));
      Add('=====  Albedo  =====');
      Add('  Max   Min    a_al   b_al  Walb');
      Add(' ' + FloatToStr(AlbedoMax[j]) + '   ' + FloatToStr(AlbedoMin[j]) + '   ' + FloatToStr(Aal[j]) + '   ' + FloatToStr(Bal[j]) + '   ' + FloatToStr(ThickAlb[j]));
      Add('=====  Enhancement Factor  =====');
      Add(' a_e   b_e   c_e   d_e   e_e');
      Add('  '+FloatToStr(Aen[j])+'   '+FloatToStr(Ben[j])+'   '+FloatToStr(Cen[j])+'   '+FloatToStr(Den[j])+'   '+FloatToStr(Een[j]));
      if WriteOnFile then
      begin
        try
          SaveToFile(ExeDir + Slash  + 'Soils' + Slash + SoilName[j] + '.dat');
        finally
          if not DirectoryExists(BaseDir) then
            CreateDir(BaseDir);
        end;
        SaveToFile(BaseDir + Slash + SoilName[j] + '.dat');
      end
      else
        VirtualFile := SL.Text;
    end
  finally
    SL.Free;
  end;
end;

//----------------------------------------------------------
function TSoil.Pressure_Head(W, T, Wsat, zeta, Wr: Double; j: Integer; Drying: Boolean): Double;
var
  a, b, c, Accuracy, fa, fc: Double;
  i: Integer;
begin
  a := ln(10000000);
  b := ln(0.1);
  c := 0;
  Accuracy := 0.00000001;
  fa := W - Water_Content(- exp(a), T, Wsat, zeta, Wr, j, Drying);
  if fa > 0 then
  begin
    for i := 1 to Trunc(ln(Abs(b - a) / Accuracy) / ln(2)) do
    begin
      c := (a + b) / 2;
      fc := W - Water_Content(- exp(c), T, Wsat, zeta, Wr, j, Drying);
      if fa * fc < 0 then
        b := c
      else
      begin
        a := c;
        fa := fc;
      end;
    end;
    Result := - exp(c);
  end
  else
    Result := - 1000000;
end;
//----------------------------------------------------------

function TSoil.Water_Content(P, T, Wsat_, zeta, Wr_: Double; j: Integer; Drying: Boolean): Double;    // j is the layer number
var
  GAMMA, dum: Double;
  i: Integer;
  procedure PrepareSegments(j: Integer);
  var
    i: Integer;
    h, ln10, dum: Double;
  begin
    Prepared[j] := True;
    Segments := False;
    ln10 := ln(10);
    for i := 0 to 140 do
    begin
      h := - Power(10, i * 0.05);
      dum := Water_Content(h, Tref[j], Wsat[j], zeta, Wr[j], j, True);
     // ShowMessage(FloatToStr(Wseg[j, i]));
      if RF[j] = 2 then
        dum := (dum - Wr[j]) / (Wsat[j] - Wr[j]);
      Wseg[j, i] := dum;
      dum := Water_Content(h, Tref[j], Wsat[j], zeta, Wr[j], j, False);
      if RF[j] = 2 then
        dum := (dum - Wr[j]) / (Wsat[j] - Wr[j]);
      Wseg[-j, i] := dum;
      lnPseg[i] := 0.05 * i * ln10;
    end;
    for i := 0 to 139 do
    begin
      Slope[j, i] := - (Wseg[j,i+1]  - Wseg[j,i])  / (lnPseg[i+1] - lnPseg[i]);
      Slope[-j,i] := - (Wseg[-j,i+1] - Wseg[-j,i]) / (lnPseg[i+1] - lnPseg[i]);
    end;
    Segments := True;
  end;

  function f(h: Double): Double;
  var
    j: Integer;
    ln_h, dum: Double;
  begin
    ln_h := ln(h);
    j := Trunc(ln_h / 0.1151292546497); //0.1151292546497 = 0.05 * ln(10)
    Result := (Wseg[i,j] * (lnPseg[j+1] - ln_h) + Wseg[i,j+1] * (ln_h - lnPseg[j])) / 0.1151292546497;    //0.1151292546497 = lnPseg[i+1] - lnPseg[i]
    j := i;
  end;

begin
  if Drying then i := j else i := -j;
  if (Mode <> 2) or (IPET_type > 6) then
    P := P * (1 + 0.0068 * (T - Tref[j]));//exp(0.0068 * (T - Tref[j]));
  if (Segments) and (Prepared[j] = False) then
    PrepareSegments(j);
  if P < -1 then
  begin
    if not Segments then
    begin
      Case RF[j] of
      1:begin
        dum := Power(1 + Power(alpha[i] * Llambda[j], np[i]), mp[i]);
        {
        GAMMA  := (dum * Wsat_ - zeta) / (dum - 1 );
        Result :=  (GAMMA - zeta) / Power(1 + Power(Abs(alpha[i]) * Teffect * -P, np[i]), mp[i])
                 - (GAMMA - zeta) / dum
                 + zeta * (1 - Sqr(ln(-P * Teffect + 1) / LogOfLlambda[j]));
        }
        GAMMA  := (dum * Wsat[j] - zeta) / (dum - 1 );
        Result :=  (GAMMA - zeta) / Power(1 + Power(alpha[i] * -P, np[i]), mp[i])
                 - (GAMMA - zeta) / dum
                 + zeta * (1 - Sqr(ln(-P + 1) / LogOfLlambda[j]));
        {
        if Drying then
          Result := Result * Wsat_ / Wsat[j]
        else
          Result := Wr_ + Result * (Wsat[j] - Wr_) / Wsat[j];
        }
        end;
      2:begin
          Result := sp[i] / Power(1 + Power(alpha1[i] * -P, np1[i]), mp1[i]);
          if sp[i] < 1 then
            Result := Result + (1 - sp[i]) / Power(1 + Power(alpha2[i] * -P, np2[i]), mp2[i]);
        end;
      else
        Result := Wsat_;
      end;
    end
    else
      Result := f(-P);
    if RF[j] = 1 then
      Result := Result / Wsat[j];
    if Drying then
      Result := Wr[j] + Result * (Wsat_   - Wr[j])
    else
      Result := Wr_   + Result * (Wsat[j] - Wr_);
    {
    if RF[j] = 1 then
      if Drying then
        Result := Result / Wsat[j]
      else
        Result := Wr_ + Result / Wsat[j];
     // Result := Result / Wsat[j];
    }
  end
  else
    Result := Wsat_ + Cwsat * P;   // fine tuning is required !
end;

{$WARNINGS OFF}
function TSoil.Water_Capacity(P, T, Wsat_, zeta, Wr_: Double; j: Integer; Drying: Boolean): Double;
var
  i: Integer;
  GAMMA, dum, dum_: Double;

  function f(h: Double): Double;
  var
    j: Integer;
  begin
    j := Trunc(ln(h) / 0.1151292546497); //0.1151292546497 = 0.05 * ln(10)
    Result := Slope[i,j] / h;
  end;

begin
  if Drying then i := j else i := -j;
  if (Mode <> 2) or (IPET_type > 6) then
  begin
    dum_ :=  1 + 0.0068 * (T - Tref[j]);  //exp(0.0068 * (T - Tref[j])); by this approx, 6% speed up
    P := P * dum_;
  end;
  if P < -1 then
  begin
    if not Segments then
    begin
      Case RF[j] of
      1:begin
        dum := Power(1 + Power(alpha[i] * Llambda[j], np[i]), mp[i]);
        {
        GAMMA := (dum * Wsat_ - zeta) / (dum - 1 );
        Result := mp[i] * (GAMMA - zeta) * Power(1 + Power(alpha[i] * Teffect * -P , np[i]), -mp[i] - 1) * np[i] * Power(alpha[i] * Teffect , np[i]) * Power( -P ,(np[i] - 1))
               + Teffect * zeta * 2 * ln(-Teffect * P + 1) / ((1 - Teffect * P) * Sqr(LogOfLlambda[j]));
        }
        GAMMA := (dum * Wsat[j] - zeta) / (dum - 1 );
        Result := mp[i] * (GAMMA - zeta) * Power(1 + Power(alpha[i] * -P , np[i]), -mp[i] - 1) * np[i] * Power(alpha[i] , np[i]) * Power( -P ,(np[i] - 1))
               + zeta * 2 * ln(- P + 1) / ((1 - P) * Sqr(LogOfLlambda[j]));
        {
        if Drying then
          Result := Result * Wsat_ / Wsat[j]
        else
          Result := Result * (Wsat[j] - Wr_) / Wsat[j];
        }
        end;
      2:begin
          Result := mp1[i] * sp[i] * Power(1 + Power(alpha1[i] * -P , np1[i]), -mp1[i] - 1) * np1[i] * Power(alpha1[i] , np1[i]) * Power(-P , (np1[i] - 1));
          if sp[i] < 1 then
            Result := Result + mp2[i] * (1 - sp[i]) * Power(1 + Power(alpha2[i] * -P , np2[i]), -mp2[i] - 1) * np2[i] * Power(alpha2[i] , np2[i]) * Power(-P , (np2[i] - 1));
        end;
      end;
    end
    else
      Result := f(-P);
    if Drying then
      Result := Result * (Wsat_   - Wr[j])
    else
      Result := Result * (Wsat[j] - Wr_);
    if RF[j] = 1 then
      Result := Result / Wsat[j];
  end
  else
    Result := Cwsat;  // fine tuning is required !
  if (Mode <> 2) or (IPET_type > 6) then
    Result := Result * dum_;
end;
{$WARNINGS ON}

function TSoil.HydraulicConductivity(W, T: Double; j: Integer): Double;
var
  Lgamma, Wrk: Double;
begin
  Result := 0;
  if MLB then
    Wrk := 0.07
  else
    Wrk := 0;
  W := Min(W, Wsat[j]);
  if W > 0.008 then
    Case KF[j] of
    1:  Result := t_unit * Ksat[j] * Power((W - Wrk) / (Wsat[j] - Wrk), ak[j] * Sqr(W) + bk[j] * W + ck[j]);
    2:  Result := t_unit * Ksat[j] * exp(ak[j] * (1 / (1 + bk[j]) - 1 / (W / Wsat[j] + bk[j])));
    3:  begin
          Lgamma := (W - Wr[j]) / (Wsat[j] - Wr[j]);
          if Lgamma <= 1 then
            Result := t_unit * Ksat[j] * Power(Lgamma, ak[j]) * Sqr(1 - Power(1 - Power(Lgamma, 1 / bk[j]), bk[j]))
          else
            Result := t_unit * Ksat[j];
        end;
    4:  Result := t_unit * Ksat[j] * exp( -ak[j] * Power(1 - Power(W / Wsat[j], bk[j]), ck[j]));
    end
  else
    Result := 0;
  if (Mode <> 2) or (IPET_type > 6) then
  begin
    if MyuRef[j] = 0 then
      MyuRef[j] := 0.022 * Tref[j] + 4.5;//exp(2050 / Tref[j] - 7);
    Result := Result * MyuRef[j] / (0.022 * T + 4.5); // 1.2% speed up
   // Result := Result * exp(2050 / Tref[j] - 7) / exp(2050 / T - 7);
  end;
end;

//----------------------------------------------------------
procedure TSoil.SetProperty_of_RetCurveFig(layer: Integer; FileName: String);
var
  i : Integer;
  f : TextFile;
  GX: array[0..100, 1..4] of Double;
  GY: array[0..100] of Double;
  IniFile: TIniFile;
  RetCurveFig : TXYGrapher;
  S: String;
begin
  IniFile := TIniFile.Create(MyDocDir + Slash + '.XYGrapher.ini');
  try
    IniFile.WriteInteger('Options', 'Height', 550);
    IniFile.WriteInteger('Options', 'Width' , 500);
  finally
    IniFile.Free;
  end;
  //--- Saving of Fitted Curve ---//
  GY[1] := -1;
  GX[1,1] := Wsat[layer];
  GX[1,2] := Wsat[layer];
  GX[1,3] := Ksat[layer];
  GX[1,4] := Ksat[layer];
  for i := 2 to 100 do
  begin
    GY[i] := - Power(10, 7 / 100 * i);
    GX[i,1] := Water_Content(GY[i], Tref[layer], Wsat[layer], zeta[layer], Wr[layer], layer, True);
    if i < 91 then
      GX[i,3] := HydraulicConductivity(GX[i,1], Tref[layer], layer) / t_unit
    else
      GX[i,3] := 0;
    if (Mode <> 2) or (IPET_type = 3) or (IPET_type = 5) then
    begin
      GX[i,2] := Water_Content(GY[i], Tref[layer], Wsat[layer], zeta[layer], Wr[layer], layer, False);
      if i < 91 then
        GX[i,4] := HydraulicConductivity(GX[i,2], Tref[layer], layer) / t_unit
      else
        GX[i,4] := 0;
    end;
  end;
  dumstr := BaseDir + Slash + SoilName[layer];
  if DirectoryExists(dumstr) = False then
    CreateDir(dumstr);
  AssignFile(f, dumstr + Slash + SoilName[layer] + '_swrc.txt');
  Rewrite(f);
  if LANG = 128 then
    Writeln(f, SoilName[j] + Msg[10])
  else
    Writeln(f, Msg[10] + SoilName[j]);
  j := layer;
  Writeln(f, Soil.cbRF.Items[Soil.cbRF.ItemIndex]);
  Case RF[j] of
  1:begin
    Writeln(f, ' Tref   Wsat   zeta    alpha      n         m         Llambda');
    Writeln(f, '  '+FormatFloat('##0.0000', Tref[j])+'  '+ FormatFloat('##0.0000', Wsat[j])+'   '+
            FormatFloat('##0.0000', zeta[j])+'  '+ FormatFloat('##0.#######', alpha[j])+
            '  '+FormatFloat('##0.0000', np[j])+'  '+FormatFloat('##0.0000', mp[j])+'  '+ FormatFloat('########0', Llambda[j]));
    if (Mode <> 2) or (IPET_type = 3) or (IPET_type = 5)  then
      Writeln(f, 'wetting:            '+ FormatFloat('##0.0000', alpha[-j])+'  '+FormatFloat('##0.0000', np[-j])+'  '+FormatFloat('##0.0000', mp[-j]));
    end;
  2:begin
    Writeln(f, ' Tref   Wsat   s   alpha1   n1     alpha2   n2    Wr');
    Writeln(f, '   '+FloatToStr(Tref[j])+'  '+ FloatToStr(Wsat[j])+'   '+
        FloatToStr(sp[j])+'   '+FloatToStr(alpha1[j])+'   '+FloatToStr(np1[j])+'    '+
        FloatToStr(alpha2[j])+'   '+FloatToStr(np2[j])+'   '+FloatToStr(Wr[j]));
    if (Mode <> 2) or (IPET_type = 3) or (IPET_type = 5) then
      Writeln(f, 'wetting:     '+ FloatToStr(sp[-j])+'   '+FloatToStr(alpha1[-j])+'   '+FloatToStr(np1[-j])+'    '+FloatToStr(alpha2[-j])+'   '+FloatToStr(np2[-j]));
    end;
  end;
  Writeln(f, Msg[11]);
  if (Mode <> 2) or (IPET_type = 3) or (IPET_type = 5)  then
  begin
    for i := 1 to 100 do
      Writeln(f, SetLengthRJ(FormatFloat('0.0000', GX[i,1]), 8) + ', ' + SetLengthRJ(FormatFloat('0.0000', GX[i,2]), 8) + ', ' + FormatFloat('########.0', GY[i]) + ', ' + SetLengthRJ(FormatFloat('0.000000000000', GX[i,3]), 13) + ', ' + SetLengthRJ(FormatFloat('0.000000000000', GX[i,4]), 13));
  end
  else
  begin
    for i := 1 to 100 do
      Writeln(f, SetLengthRJ(FormatFloat('0.0000', GX[i,1]), 8) + ', ' + FormatFloat('########.0', GY[i]) + ', ' + SetLengthRJ(FormatFloat('0.000000000000', GX[i,3]), 13));
  end;
  CloseFile(f);
  //---------------------------------
  RetCurveFig := TXYGrapher.Create;
  with RetCurveFig do
  begin
    if (Mode <> 2) or (IPET_type = 3) or (IPET_type = 5) then
      NumDataSeries := 2
    else
      NumDataSeries := 1;
    if num_ret_data > 0    then
      NumDataSeries := NumDataSeries + 1;
    if num_add_ret_data > 0 then
      NumDataSeries := NumDataSeries + 1;
    //----------
    DataSeries[1] := dumstr + Slash + SoilName[layer] + '_swrc.txt';
    if (Mode <> 2) or (IPET_type = 3) or (IPET_type = 5)then
      Yl[1] := 3
    else
      Yl[1] := 2;
    Xb[1] := 1; Xt[1] := 0; Yr[1] := 0; ReadStart[1] := 6; ReadEnd[1] := 0;
    LineStyle[1] := 1; LineWidth[1] := 0.25; LineColor[1] := 12615680; MarkStyle[1] := 0; MarkSize[1] := 0;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := -1; Yoffset[1] := 0;
    if (Mode <> 2) or (IPET_type = 3) or (IPET_type = 5) then
    begin
      DataSeries[2] := DataSeries[1];
      Xb[2] := 2; Xt[2] := 0; Yl[2] := 3; Yr[2] := 0; ReadStart[2] := 6; ReadEnd[2] := 0;
      LineStyle[2] := 1; LineWidth[2] := 0.25; LineColor[2] := 16711680; MarkStyle[2] := 0; MarkSize[2] := 0;
      Xmult[2] := 1; Xoffset[2] := 0; Ymult[2] := -1; Yoffset[2] := 0;
    end;
    //----------
    if num_ret_data > 0 then
    begin
      if num_add_ret_data > 0 then
        i := NumDataSeries - 1
      else
        i := NumDataSeries;
      DataSeries[i] := CaseName + '_RetData.csv';
      Xb[i] := 1; Xt[i] := 0; Yl[i] := 2; Yr[i] := 0; ReadStart[i] := 0; ReadEnd[i] := 0;
      LineStyle[i] := 0; LineWidth[i] := 0; LineColor[i] := 1; MarkStyle[i] := 1; MarkSize[i] := 1.5; MarkColor[i] := 16777215;
      Xmult[i] := 1; Xoffset[i] := 0; Ymult[i] := -1; Yoffset[i] := 0;
    end;
    //----------
    if num_add_ret_data > 0 then
    begin
      i := NumDataSeries;
      DataSeries[i] := fl_retention;
      Xb[i] := 1; Xt[i] := 0; Yl[i] := 2; Yr[i] := 0; ReadStart[i] := 0; ReadEnd[i] := 0;
      LineStyle[i] := 0; LineWidth[i] := 0; LineColor[i] := 1; MarkStyle[i] := 2; MarkSize[i] := 1.5; MarkColor[i] := 16777215;
      Xmult[i] := 1; Xoffset[i] := 0; Ymult[i] := -1; Yoffset[i] := 0;
    end;
    //----------
    Height := 65; Width := 65; Lmargin := 30; Tmargin := 20; FLineWidth := 0.4; NumFontSize := 9;
    NumFontName := 'Arial';
    dumint := Trunc(Wsat[j] * 10) + 1;
    Max[1] := 0.1 * dumint; Min[1] := 0; Division[1] := dumint; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 0.1 * dumint; Min[2] := 0; Division[2] := dumint; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] := 10000000; Min[3] := 1; Division[3] := 7; Minordivision[3] := 0; logarithm[3] := 1; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] := 10000000; Min[4] := 1; Division[4] := 7; Minordivision[4] := 0; logarithm[4] := 1; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 5;
    if num_ret_data > 0 then
      NumLegend := 6;
    if num_add_ret_data > 0 then
      NumLegend := 6;
    Legend[1] := Msg[12] + SoilName[j] + Msg[13];
    Legend[2] := Msg[14];
    Legend[4] := Msg[15];
    dumstr    := Msg[16];
    Legend[3] := '';
    Legend[5] := '';
    if IPET_type <> 3 then
      S := 'MSO'
    else
      S := 'MSI';
    if num_ret_data > 0 then
      if num_add_ret_data = 0 then
        Legend[6] := '%2 [1]' + S + '\n'
      else
        Legend[6] := '%' + IntToStr(NumDataSeries) + ' [1]' + dumstr + '%' + IntToStr(NumDataSeries - 1) + ' [1]' + S + '\n'
    else
      if num_add_ret_data > 0 then
        Legend[6] := '%2 [1]' + dumstr;
    LegendLspacing[1] := 4; LegendXshift[1] := 0; LegendYshift[1] := 0;
    LegendLspacing[2] := 0; LegendXshift[2] := 0; LegendYshift[2] := 0;
    LegendLspacing[3] := 0; LegendXshift[3] := 0; LegendYshift[3] := 0;
    LegendLspacing[4] := 0; LegendXshift[4] := 0; LegendYshift[4] := 0;
    LegendLspacing[5] := 0; LegendXshift[5] := 0; LegendYshift[5] := 0;
    LegendLspacing[6] := 5; LegendXshift[6] := 10; LegendYshift[6] := -25;
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  RetCurveFig.Free;
end;
//----------------------------------------------------------
function TSoil.SetLengthRJ(S: String; i: Shortint): String;
var j: Shortint;
begin
  for j := 1 to i - Length(S) do
    S := ' '+ S;
  Result := S;
end;

procedure TSoil.seLayerNumberChange(Sender: TObject);
begin
  if Edit1.Text <> '' then
    GetParameters;
  j := seLayerNumber.Value;
  if j < 1 then
    j := 1;
  if (j > 1) and (Wsat[j] < 0.1) then
    LoadProperties(j);
  ShowEquation;
end;

function TSoil.Water_content_at_given_K(K: Double; j: Integer): Double;
var
  a, b, c, Accuracy, fa, fc: Double;
  i: Integer;
begin
  a := 0.01;
  b := Wsat[j];
  c := 0;
  Accuracy := 0.001;
  fa := ln(K) - ln(HydraulicConductivity(a, Tref[j], j));
  for i := 1 to Trunc(ln(Abs(b - a) / Accuracy) / ln(2)) do
  begin
    c := (a + b) / 2;
    fc := ln(K) - ln(HydraulicConductivity(c, Tref[j], j));
    if fa * fc < 0 then
      b := c
    else
    begin
      a := c;
      fa := fc;
    end;
  end;
  Result := c;
end;

function TSoil.ConvertDecimalSeparator(S: String): String;
begin
  if DecimalSeparator = ',' then
  begin
    S := StringReplace(S, ',', ' ', [rfReplaceAll]);
    Result := StringReplace(S, '.', ',', [rfReplaceAll]);
  end
  else
  begin
    Result := StringReplace(S, ',', '.', [rfReplaceAll]);
  end
end;


initialization
  {$I RC_Params.lrs}

end.

