unit Simulation;

interface

{$mode delphi}{$H+}
{$define notsnow} //remove not
uses 
  LResources,Buttons,
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Math,
  ExtCtrls, ComCtrls;

const
  ETAs = 0.6; ETAh = 0.6; Margin = 0.000001; NumElem = 801; NumElmSnow = 50;

type
  TValues = array[0..NumElem] of Double;

  { TSimulator }

  TSimulator = class(TObject)
    function GetMilliTime:Double;
  private
    procedure Observate;
    procedure CalculateVaporMovement(ToEnd: Integer);
    procedure CalculateSoluteMovement;
    procedure CalculateHeatMovement;
    procedure CalculateHeatMovementInSnow;
    //procedure Report;
    procedure SolveTheCoefficientMatrix;
    procedure SetScanningCurve(j, i: Integer; Drying_: Boolean);
    procedure NumericalExperiment;
    function GetOneStepDisp(i: Integer): Double;
    function CalculateAtmos_BC: Boolean;
    function CalculateLiquidFlow: Boolean;
  public
    constructor Create; //override;
    function SetLengthRJ(dumstr: String; dumint: Shortint): String;
    function GetResidual(x: Double; n: Integer): Double;
    function GetScaledParameter(x: Double; i, j: Integer; Drying_: Boolean): Double;
    function GetAvgWaterContentAbove(Depth: Double): Double;
    function GetMassOfSaltAbove(Depth: Double): Double;
    function NextValue: Double;
    function NextString: String;
    procedure LoadClimaticData;
    procedure Simulate;
    procedure DrawFigure;
    procedure DetermineSpaceIncrements;
    procedure OutputToFile;
    procedure EvaluateDailyScaleParameters;
  end;

var
  Simulator: TSimulator;
  ETA, MaxEvap, t_conv, dt_min, time_in_a_day: Double;
  z_c : TValues;
  Ath, Bth, Cth, Dth, U, x: TValues;
  W, Wb, Wtent, Wice, Cw, Sw, Swp, Swr, Swi, P, Pw, Pb, Pold, K      : TValues;
  NRD, alpha                                             : TValues;
  ql, qv, Cv, Cvsat, RH, EN, Dv, rho                      : TValues;
  C, M, Crystl, Di, Dm, Dc, qs, Wold, Wbold, Wolds, Ss, pi, V     : TValues;
  Chnew, kh, qh, LH, Sh                                   : TValues;
  T, Tb, Ch, h                                            : TValues;
  Drying, Drying_                                         : array[0..NumElem] of Boolean;
  Wsat_A, zeta_A, Wr_A                                   : TValues; // Adapted

  TargetMaxDiff_logP, TargetESRErrorW, CriteriaForCeliasIteration, Gravity: Double;
  LowerTargetIteration, UpperTargetIteration, IterationScheme, InterHyCon : ShortInt;
  SL: TStringList;
  iteration, DayCounter, HistoryDrawStartingPoint, HistoryDrawEndingPoint: Integer;
  CanceledTime, MaxDiff_logP, q_rain: Double;
  Lhf, kfpd: Double;
  Canceled: Boolean;
  MLB: Boolean = False;//True;   // for fingering study
  NumMetData: Integer;
  //--- Snowpack Dynamics
  rho_snow, dz_snow, Hsnow, Tsnow, qh_snow, z_snow: array[1..NumElmSnow] of Double;
  iss: Integer;
  Wstorage_in_snow, Estorage_in_snow, time_last_snow: Double;
  //---------------------
  TherConPrepared, EnPrepared, DrPrepared: array[1..5] of Boolean;
  Khseg, Enseg, Drseg: array[1..5, 0..20] of Double;
  ThresholdPotential, ThresholdPotentialForSalinityStress : Double;
  Whys1, Whys2, Whys3: array[1..5] of Double;
  S: String;

implementation

uses
  WASH1dMain, RC_Params, K_Params, RootFinder;

constructor TSimulator.Create;
begin
  inherited Create;
  TargetMaxDiff_logP  := 1;
  TargetESRErrorW := 0.0001;
  CriteriaForCeliasIteration := 0.0001;
  LowerTargetIteration := 3;
  UpperTargetIteration := 7;
  IterationScheme := 2;
  InterHyCon := 1;
  SL := TStringList.Create;
end;

procedure TSimulator.LoadClimaticData;
var
  i,j: Integer;
  SL1: TStringList;
  S0: String;
begin
  SL1 := TStringList.Create;
  j := 0;
  try
    t_clm_con[0] := 0;
    SL1.LoadFromFile(fl_clmtc_con);
    for i := LoadingStartRow to SL1.Count do
    begin
      if j = MaximumMetData - 2 then
        Break;
      S := Main.ConvertDecimalSeparator(SL1[i-1]);
      try
        if LoadingStartRow = 1 then
          t_clm_con[j] := NextValue
        else
          t_clm_con[j] := NextValue - t_clm_con[0];
        if abc_type = 5 then
        begin
          Tp[j]     := NextValue;
          j := j + 1;
          Continue;
        end;
        WV[j]        := NextValue;
        Ta[j]        := NextValue + 273.2;
        S0 := NextString;
        if S0 <> '---' then
          RHa[j]     := StrToFloat(S0) / 100
        else
          RHa[j]     := 0.8;
        Prcp[j]      := NextValue / 10;   // mm         --> cm
        Rs[j]        := NextValue * 100 / 3600 * t_unit;  // MJ/m2/h  --> J/cm2/h
        {
        if (j = 0) and (SL2.Count > 6) then
        begin
          SL2[6] := StringReplace(SL2[6], '-Separator/', [rfReplaceAll]);
          //if Pos('/'[6]) = 5 then
          //else
          //  ShortDateFormat := 'd/m/yyyy';
         // ShowMessage(SL2[6]);
          //IniDay := StrToDate(SL2[6]);
        end;
        }
        if Length(StringReplace(S, ' ', '', [rfReplaceAll])) > 1 then
          S1[j]        := NextString;
        if Length(StringReplace(S, ' ', '', [rfReplaceAll])) > 1 then
          S2[j]        := NextString;
        j := j + 1;
      except
      end;
    end;
    t_clm_con[j] := Max(t_clm_con[j], t_end);
    t_clm_con[0] := 0;
  finally
    SL1.Free;
  end;
  NumMetData := i;
  if t_end >= t_clm_con[j] then
  begin
    t_clm_con[j] := t_end + dt_max;
    WV[j]        := WV[j-1];
    Ta[j]        := Ta[j-1];
    RHa[j]       := RHa[j-1];
    Prcp[j]      := Prcp[j-1];
    Rs[j]        := Rs[j-1];
  end;
end;

procedure TSimulator.Simulate;
var
  //f: TextFile;
  i, i2, j: Integer;
  starttime: Double;
  SL1: TStringList;
  S1: String;

  procedure LoadMeasuredProfile(FileName: String; var Y: TValues; ValType: Shortint = 1);
  var
    zm, Ym: TValues;
    i, i2, i3, i4, i5, i6: Integer;
  begin
    SL1 := TStringList.Create;
    i2 := 1;
    try
      SL1.LoadFromFile(FileName);
      if Pos('_profile.txt', FileName) > 0 then
      begin
        i4 := ValType;
        if solute then
          i5 := 9
        else
          i5 := 7
      end
      else
      begin
        i4 := 1;
        i5 := 1;
      end;
      for i := i5 to SL1.Count do
      begin
        S := Main.ConvertDecimalSeparator(SL1[i-1]);
        try
          zm[i2] := NextValue;
          for i6 := 1 to i4 - 1 do  // check if it is OK
            Ym[i2] := NextValue;
          Ym[i2] := NextValue;
          i2 := i2 + 1;
        except
        end;
      end;
    finally
      SL1.Free;
    end;
    i3 := i2;
    for i := 1 to n + 1 do
    begin
      i2 := 1;
      while (z[i] >= zm[i2]) and (i2 < i3) do
        i2 := i2 + 1;
      if z_c[i] < zm[1] then
        Y[i] := ((Ym[2] - Ym[1]) / (zm[2] - zm[1])) * (z_c[i] - zm[1]) + Ym[1]
      else
        if z_c[i] > zm[i3-1] then
          Y[i] := ((Ym[i3-1] - Ym[i3-2]) / (zm[i3-1] - zm[i3-2])) * (z_c[i] - zm[i3-2]) + Ym[i3-2]
        else
          Y[i] := (Ym[i2-1] * (zm[i2] - z_c[i]) + Ym[i2] * (z_c[i] - zm[i2-1])) / (zm[i2] - zm[i2-1]);
    end;
  end;
begin
  DetermineSpaceIncrements;
  //i := n+2;
  //SetLength(W,i)   ; SetLength(Wb,i)   ; SetLength(Wtent,i)  ; SetLength(Cw,i)  ; SetLength(Sw,i);
  //SetLength(P,i)   ; SetLength(Ptent,i); SetLength(Pbefore,i);
  //SetLength(K,i)   ; SetLength(Kmid,i) ; SetLength(ql,i)     ; SetLength(qv,i)  ;
  //SetLength(Cv,i)  ; SetLength(Cvsat,i); SetLength(RH,i)     ; SetLength(EN,i)  ; SetLength(Dv,i);
  //SetLength(C,i);
  //SetLength(T,i)   ; SetLength(Tb,i)   ; SetLength(Ch,i)     ; SetLength(H,i);
  //SetLength(THA,i) ; SetLength(THB,i)  ; SetLength(THC,i)    ; SetLength(THD,i) ; SetLength(U, i);
  //SetLength(THBD,i); SetLength(Dthd,i) ; SetLength(x,i);
  //i := Trunc(t_end / int_rec) + 1;
  //SetLength(time,i)  ; SetLength(Wtime,i); SetLength(Ptime,i)  ; SetLength(Ttime,i); SetLength(CumE,i); SetLength(CumD,i);
  //SetLength(ql_bot,i); SetLength(Evap,i) ;   //ўўўўўўўint_recўўўўўўўўўўўўўўўўўўўўAccess Violation Errorўўўўўўў
//------------------- Initialization -----------------//
  StopCalc := 0;
  time_CRWM := 0; dt_CRWM_next := 20 * dt_ini;
  time_ := 0; dt := dt_ini; time_in_a_day := 0; DayCounter := 0; HistoryDrawStartingPoint := 1;
  abc_done  := 0; Pbot_done := 1; OutFlow_done := 1; qltop_done := 1;
  CumEvap  := 0; CumInfilt := 0; CumDrain := 0; counter := 0; NumCalc := 0;
  Cum_qs_top := 0;  Cum_qs_bot := 0; cum_s_uptake := 0; Crystal := 0; V1before  := 0; Vnbefore := 0;
  CumSHT := 0; CumLHT :=  0; CumCHL :=  0; CumHLB :=  0; CumHLS :=  0;
  Wstorage_in_snow := 0; iss := NumElmSnow;
  Gravity := cos(3.1415926535897932385 * Angle / 180.0);
  Lhf := 335; // latent heat of fusion in J/g
  kfpd := 0.0000803; // coefficient of freezing point depression
  t_ql_top[3] := RefillInterval * 86400 / t_unit + t_ql_top[1];
  t_ql_top[4] :=  t_end + 1;
  CumIrrigation[4] := CumIrrigation[3];
  for i := 1 to NumElmSnow do
  begin
    dz_snow[i] := 0;
    Hsnow[i] := 0;
    Tsnow[i] := 273.2;
  end;
  CanceledTime := 0;
  t_conv := 86400 div t_unit;
  dt_min := 0.01 / t_unit;
  CumTr := 0;
  CumTrp := 0;
  if IterationScheme < 2 then
    ETA := 0.6
  else
    ETA := 1;
  if Tair = 0 then
    Tair := Soil.Tref[1];
  Cv[1] :=  0.00000002704 * Tair * Tair - 0.000014722 * Tair + 0.0020093;//exp(6 - 4976 / Tair);
  if Mode <> 2 then
    IPET_type := 0;
  if NumExp = 1 then
    t_evaporation[1] := 100000000000000;
  if IPET_type = 2 then
    ra := 999;
  for j := 1 to NumObs do
    for i := 1 to Ceil(t_end / int_rec) do
      Ptime[j,i] := 0;
  for i := 1 to 3000 do
    Front[i] := 0;
  //============ Constants =============//
  if abc_type  = 2 then
    Cva := RHair * (0.00000002704 * Tair * Tair - 0.000014722 * Tair + 0.0020093);//exp(6 - 4976 / Tair);
  case s_type of
  1:begin
    Diw0   := 0.000015 * t_unit;
    Cmax   := 310;
    NumIon := 2;
    Mweight :=  59;
    a_oc   := 0.0000028;//0.0026;
    b_oc   := 0.0003;//-0.025;
    c_oc   := 0.92;//0.53;
    if (Soil.RF[1] = 1) and (Soil.zeta[1] > 0.01) then
    begin
      ar     :=  0.685;  // imasa, 2004
      br     := -1.04;
    end
    else
    begin
      ar     :=  0.50;  // Toyoura
      br     := -0.54;
    end;
    end;
  2:begin // CaCl2
    Diw0 := 0.000013 * t_unit;
    Cmax := 310;  //??
    NumIon := 3;
    Mweight :=  40.1 + 35.5 * 2;
    a_oc   := 0.0000028;//0.0026;
    b_oc   := 0.0003;//-0.025;
    c_oc   := 0.92;//0.53;
    end;
  3:begin
    Diw0 := 0.000019 * t_unit;
    Cmax := 310;
    NumIon := 2;
    Mweight :=  74.5;
    a_oc   := 0.0000017;//0.00075;
    b_oc   := 0.00028;//-0.042;
    c_oc   := 0.92;//0.31;
    ar     :=  0.62;  // imasa-KCl, 2004
    br     := -0.62;
    end;
  end;
  for j := 1 to NumLayer do
  begin
    Whys1[j] := Soil.Wsat[j] * 0.8;
    Whys2[j] := Soil.zeta[j] * 0.8;
    Whys3[j] := (1 - Soil.sp[j]) * Whys1[j];
  end;
//===================  Initial Condition  ========================
  //----- water -----//
  for i := 1 to n+1 do
  begin
    qv[i] := 0;
    ql[i] := 0;
  end;
    if (water = True) or (IPET_type = 11) then
    with Soil do
    begin
      {
      for j := 1 to NumLayer do
        if RF[j] = 1 then
          LogOfLlambda[j] := ln(Llambda[j]);
      }
      for i := 1 to n + 1 do
      begin
        if IniDrying then
          Drying[i] := True
        else
          Drying[i] := False;
        Drying_[i] := Drying[i];
        j := L[i];
        Wsat_A[i] := Wsat[j];
        zeta_A[i] := zeta[j];
        Wr_A[i]   := Wr[j];
       // Just_a_moment[i] := 0;
      end;
      if not heat then
        Tini := Tair;
      case ic_type of
      1:for i := 1 to n do
        begin
          j := L[i];
          p[i] := Pressure_Head(w_ini, Tini, Wsat_A[i], zeta_A[i], Wr_A[i], j, Drying[i]);
          W[i] := W_ini;
        end;
      2:for i := 1 to n + 1 do
        begin
          j := L[i];
          p[i] := p_ini;
          W[i] := Water_Content(p[i], Tini, Wsat_A[i], zeta_A[i], Wr_A[i], j, Drying[i]);
        end;
      3:for i := 1 to n + 1 do
        begin
          j := L[i];
          p[i] := p_top_ini + (z[i] + z[i-1]) / 2;
          W[i] := Water_Content(p[i], Tini, Wsat_A[i], zeta_A[i], Wr_A[i], j, Drying[i]);
        end;
      4:begin
          LoadMeasuredProfile(fl_profile, W);
          if Pos('_profile.txt', fl_profile) > 0 then
          begin
            LoadMeasuredProfile(fl_profile, Wsat_A, 5);
            LoadMeasuredProfile(fl_profile, Wr_A, 7);
            for i := 1 to n + 1 do
              if Wr_A[i] < -0.9 then
                Drying[i] := True
              else
                Drying[i] := False;
            for i := 1 to n + 1 do
              Drying_[i] := Drying[i];
            LoadMeasuredProfile(fl_profile, Wr_A, 6);
          end;
          for i := 1 to n + 1 do
          begin
            j := L[i];
            W[i] := Max(W[i], 0.005);
            P[i] := Pressure_Head(W[i], Tini, Wsat_A[i], zeta_A[i], Wr_A[i], j, Drying[i]);
            if P[i] > -0.2 then
              W[i] := Wsat_A[j];
          end;
        end;
      end;
    end
    else
      for i := 1 to n + 1 do
      begin
        p[i]   := 0.0;
        W[i]   := 0.3;
      end;
  if Hpond > 0 then
  begin
    P[1] := Hpond;
    W[1] := Soil.Wsat[1] + Soil.Cwsat * Hpond;
  end;
  if Tini < 0 then
    for i := 1 to n + 1 do
    begin
      Wice[i]  := W[i];
      W[i] := 0;
    end
  else
    for i := 1 to n + 1 do
    begin
      Wice[i] := 0;
    end;
  W_storage_ini  := 0;
  for i := 1 to n do
    W_storage_ini := W_storage_ini + (W[i] + Wice[i]) * dz[i];
  W_storage_old := W_storage_ini;
  W[n+1] := W[n]; W[0] := W[1];  // imaginary values.
  for i := 1 to n + 1 do
  begin
    Pold[i] := P[i];
   // Pb[i] := exp((ln(Abs(P[i])) * dz[i+1] / 2 + ln(Abs(P[i+1])) * dz[i] / 2) / ((dz[i] + dz[i+1]) / 2));
    Wtent[i] := W[i];
    Wold[i]  := W[i];
    Wolds[i] := W[i];
    Wb[i] := (W[i] * dz[i+1] / 2 + W[i+1] * dz[i] / 2) / ((dz[i] + dz[i+1]) / 2);
    Wbold[i] := Wb[i];
  end;
               // _b : at the compartment boundaries --
  Wb[n+1] := Wb[n];
  //----- heat -----//
  if not heat then
    for i := 1 to n do
      T[i] := Tair
  else
    if FileExists(fl_profile) then
    begin
      LoadMeasuredProfile(fl_profile, T, 2);
      for i := 1 to n do
        T[i] := T[i] + 273.2;
    end
    else
      for i := 1 to n do
        T[i] := Tini;
  T[n+1] := T[n];
  E_Storage_ini := 0;
  for i := 1 to n do
  begin
    j := L[i];
    Ch[i] := 2.13 * (1 - Soil.Wsat[j]) + 4.18 * W[i] + 2 * Wice[i];
    h[i] := Ch[i] * T[i];
    E_Storage_ini := E_Storage_ini + (h[i] + W[i] * Lhf) * dz[i];
  end;
  if h_l_bc_type = 1 then
    T[n+1] := Tbot
  else
    T[n+1] := Tini;
  for i := 1 to n do
    Tb[i] := (T[i] * dz[i+1] / 2 + T[i+1] * dz[i] / 2) / ((dz[i] + dz[i+1]) / 2);
  E_Storage_old := E_Storage_ini;
  //------- solute -------//
  if not solute then
    C_ini := 0;
  S_Storage_ini := 0;
  case s_ic_type of
  1:for i := 1 to n do
    begin
      C[i] := C_ini;
      M[i] := C[i] * W[i];
      Crystl[i] := 0;
    end;
  2:begin
      LoadMeasuredProfile(fl_iniCprofile, C, s_ic_type + 1);
      for i := 1 to n do
      begin
        M[i] := C[i] * W[i];
        Crystl[i] := 0;
      end;
    end;
  3:begin
      LoadMeasuredProfile(fl_iniCprofile, M, s_ic_type + 1);
      for i := 1 to n do
        if M[i] / W[i] < Cmax then
          C[i] := M[i] / W[i]
        else
        begin
          C[i] := Cmax;
          Crystl[i] := M[i] - W[i] * Cmax;
        end;
    end;
  end;
  for i := 1 to n do
    S_Storage_ini := S_Storage_ini + M[i] * dz[i];
  C[n+1] := C[n];
  if CRWM = 1 then
  begin
    for i := 1 to Trunc(Crystal / MPP) do
      zp[i] := -1;
    NumP := Trunc(Crystal / MPP);
    for j := 1 to n do
    begin
      for i := NumP + 1 to NumP + Trunc(M[j]*dz[j]/ MPP) do
      begin
        zp[i]  := z[j-1] + dz[j] * Random;
        ncp[i] := j;
      end;
      NumP := Trunc(M[j]*dz[j] / MPP) + NumP;
    end;
  end;
  if not water then
  begin
    //if Mode = 1 then
    for i := 0 to n + 1 do
      ql[i] := q_irg;
    //else
      //for i := 0 to n + 1 do
      //  ql[i] := ql_top;
    if ql[0] < 0 then
      ql[0] := 0;
  end;
  //----------------------------------------------------------
  Observate;
  if Graph = 1 then
  begin
    Main.Profile;
    Main.DrawProfile(W,z,n,0,0.6,clBlue);
    Main.DrawProfile(C,z,n,0, Max(15 * Ceil(Max(c_ini, C_irg) / 6), 6),$009090);
    Main.DrawProfile(T,z,n,263.2,323.2,clRed);
  end;
  //================= Load of Boundary Condition Data ================//
  //--- Upper Boundary Flux ---//
  if (((u_bc_type = 3) or (u_bc_type = 4)) and (Mode < 3)) and (FileExists(fl_ql_top)) then
  begin
    SL1 := TStringList.Create;
    t_ql_top[1] := 0;
    CumIrrigation[1]:= 0;
    if ObsPtIrg = 0 then
      i2 := 2
    else
      i2 := 3;
    try
      SL1.LoadFromFile(fl_ql_top);
      for i := 1 to SL1.Count do
        begin
        S := Main.ConvertDecimalSeparator(SL1[i-1]);
        S1 := NextString;
        try
          if (t_unit = 3600) and (Pos(':', S1) > 0) then
            t_ql_top[i2] := 24 * StrToTime(S1)
          else
            t_ql_top[i2] := StrToFloat(S1);
          CumIrrigation[i2] := NextValue;
          //qltop[i2]       := NextValue;  until 2.3
          i2 := i2 + 1;
        except
        end;
        end;
    finally
      SL1.Free;
    end;
    t_ql_top[i2]  := 1000000000000;
    CumIrrigation[i2] := CumIrrigation[i2-1];
    //Num_qltop_Data := i2 - 1;
  end;
  //---- Atmospheric Boundary Condition ----//
  case abc_type of
  3:begin
      SL1 := TStringList.Create;
      i2 := 1;
      try
        SL1.LoadFromFile(fl_clmtc_con);
        for i := 1 to SL1.Count do
        begin
          S := Main.ConvertDecimalSeparator(SL1[i-1]);
          try
            flx[i2] := NextValue;
            fly[i2] := NextValue;
            i2 := i2 + 1;
          except
          end;
        end;
      finally
        SL1.Free;
      end;
      dumint := i2;
      for i := 1 to dumint do
      begin
        t_clm_con[i]      := flx[i];
        CumEvaporation[i] := fly[i];
      end;
      t_clm_con[dumint + 1] :=  100000000000000000.0;
      CumEvaporation[dumint + 1]  := CumEvaporation[dumint-1];
   end;
  4..5: begin
          LoadClimaticData;
        end;
  end;
  if s_u_bc_type = 2 then
  begin
    SL1 := TStringList.Create;
    i2 := 0;
    try
      SL1.LoadFromFile(fl_C_irg);
      for i := 1 to SL1.Count do
      begin
        S := Main.ConvertDecimalSeparator(SL1[i-1]);
        try
          t_c_irg[i2] := NextValue;
          Cirg[i2]    := NextValue;
          i2 := i2 + 1;
        except
        end;
      end;
    finally
      SL1.Free;
    end;
    NumCirgData := i2 - 1;
  end;
  //----- Lower Boundary Data ----//
  if Mode = 1 then
    case l_bc_type of
    4:begin
      SL1 := TStringList.Create;
      i2 := 0;
      try
        SL1.LoadFromFile(fl_p_bot);
        for i := 1 to SL1.Count do
        begin
          S := Main.ConvertDecimalSeparator(SL1[i-1]);
          try
            t_p_bot[i2] := NextValue;
            Pbot[i2]    := NextValue;
            i2 := i2 + 1;
          except
          end;
        end;
      finally
        SL1.Free;
      end;
      NumPbotData := i2 - 1;
      end;
    5:begin
      SL1 := TStringList.Create;
      i2 := 1;
      try
        SL1.LoadFromFile(fl_outflow);
        for i := 1 to SL1.Count do
        begin
          S := Main.ConvertDecimalSeparator(SL1[i-1]);
          try
            dum := NextValue;
            if dum < t_outflow[i2-1] then
              Break;
            t_outflow[i2] := dum;
            CumOutFlow[i2] :=  NextValue;
            i2 := i2 + 1;
          except
          end;
        end;
      finally
        SL1.Free;
      end;
      NumOutflowData_for_DP := i2 - 1;
      end;
    end;
  if s_l_bc_type = 2 then
  begin
    SL1 := TStringList.Create;
    i2 := 0;
    try
      SL1.LoadFromFile(fl_C_bot);
      for i := 1 to SL1.Count do
      begin
        S := Main.ConvertDecimalSeparator(SL1[i-1]);
        try
          t_c_bot[i2] := NextValue;
          Cbot[i2]    := NextValue;
          i2 := i2 + 1;
        except
        end;
      end;
    finally
      SL1.Free;
    end;
    NumCbotData := i2 - 1;
  end;
  if (CRWM = 1) and (not water) then
    Soil.dispersivity[1] := Ds / (q_irg / 0.3);
  //-- Other Preparation --//
  Main.ProgressBar1.Max := Trunc(t_end / int_rec);
  Main.ProgressBar1.Position := 0;
  //Main.ProgressBar1.Top := 1 + Main.Panel4.Top + Main.Panel4.Height;
  for i := 1 to 5 do
  begin
    Soil.Prepared[i]   := False;
    TherConPrepared[i] := False;
    EnPrepared[i]      := False;
    DrPrepared[i]      := False;
  end;
  if PlantName <> '' then
  begin
    ThresholdPotential                  :=  Pm50 * Power(1 / 0.99 - 1, 1 / pm_);
    ThresholdPotentialForSalinityStress :=  Po50 * Power(1 / 0.99 - 1, 1 / pm_);
  end;

  SL := TStringList.Create;
  with SL do
  begin
    Clear;
    Add('Time evolution of pressure head at the bottom');
    Add('   time     pressure head(cm)');
    Add(SetLengthRJ('0', 12) + '  ' + FormatFloat('###0', P_bot));
  end;
  {$I-}
  if Graph <> 1 then
    Main.ProgressBar1.Visible := True;
  time_in_a_day := tin_a_day_ini;
  EvaluateDailyScaleParameters;
  starttime := GetMilliTime;
///////////////////////////////  Main Loop  //////////////////////////////////
  repeat
    //--------  Numerical Analysys  ------------//
    if water then
    begin
      if CalculateAtmos_BC = False then
      begin
        DrawFigure;
        Break;
      end;
      CalculateVaporMovement(n);
      if CalculateLiquidFlow = False then
      begin
        DrawFigure;
        Break;
      end;
    end;
    if Canceled = False then
    begin
      if solute then
        CalculateSoluteMovement;
      if heat then
      begin
        CalculateHeatMovement;
        CalculateHeatMovementInSnow;
      end;
      time_ := time_ + dt;
      time_in_a_day := time_in_a_day + dt;
      if time_in_a_day >= 86400 / t_unit then
      begin
        EvaluateDailyScaleParameters;
        time_in_a_day := time_in_a_day - 86400 / t_unit;
        Inc(DayCounter);
      end;
    end;
    //--------  Record and Graphical Output ---------//
    if time_ >= int_rec * (counter + 1) then
    begin
      counter := counter + 1;
      if CRWM = 1 then
      begin
        if not water then
          for i := 0 to n + 1 do
            ql[i] := q_irg;
        dumint := 12345;
        CalculateSoluteMovement;
      end;
      Observate;
      if NumExp = 1 then
        NumericalExperiment;
      if Graph = 1 then
        DrawFigure
      else
        Main.ProgressBar1.StepIt;
    end;
    if StopCalc = 1 then Exit;
    //--------- Time Step Control ------------//
    avdt := avdt + dt; navdt := navdt + 1;
    //-- water --//
    if water then
    begin
      if IterationScheme = 2 then
      begin
        if iteration <= LowerTargetIteration then
          dt := dt * 1.3
        else
          if iteration >= UpperTargetIteration then
            dt := dt * 0.7;
      end
      else
      begin
        if ESRErrorW > TargetESRErrorW then
          dt := dt * 0.7
        else
          dt := dt * 1.3;
      end;
      if ln(MaxDiff_logP) > TargetMaxDiff_logP then
        dt := dt * 0.5;
      dt := Min(dt_max, dt);
      dt := Max(dt_min, dt);
    end
    else
      dt := dt * 1.2;
    //-- solute --//
    if (solute) and (CRWM <> 1) and (time_ > 0) then
    begin
      for i := 1 to n - 1 do
      begin
        if Dc[i] > 0 then
        begin
          dum := 2 * dz[i] * dz[i] / Dc[i] / 2;
          if dum < dt then
            dt := dum;
        end;
      end;
      dt := dt;
    end;
    if time_ + dt > int_rec * (counter + 1) then
      dt := int_rec * (counter + 1) - time_ + 0.000001;
  until time_ >= t_end;
//////////////////////////////////////////////////////////////////////////////
  DrawFigure;
  Observate;
  if NumExp = 1 then
  begin
    SL.Add(SetLengthRJ(FormatFloat('#######0.0000', time_), 12) +'  '+ FormatFloat('###0', P_bot));
    SL.SaveToFile(caseName+'_Pbot.dat');
  end;
  SL.Free;
  Main.ProgressBar1.Visible := False;
  if Mode = 1 then
  begin
    Beep;
    ShowMessage('Run time = '+ FormatFloat('####0.00',GetMilliTime - starttime) + ' sec');
  end
  else
    ;//ShowMessage('ak ='+ FormatFloat('#0.00',ak)+' bk = '+ FormatFloat('#0.00',bk));
  if AllowRecord = 1 then
    OutputToFile;
end;
//----------------------------------------------------------

{ for numerical experiment of Two-Stage Method or Multi-Step Outflow Method}
procedure TSimulator.NumericalExperiment;
  procedure WriteOnFile;
  begin
    SL.Add(SetLengthRJ(FormatFloat('#######0.0000', time_), 12) + '  '+ FormatFloat('###0', P_bot));
  end;
begin
  if (Abs(ql[n]*10*86400 / t_unit) < ChgCriteria)
    and (qv[0] > -0.00000000000000001) then
    if MultiStep < 2 then
    begin
      if P[4] < P_bot / 2 then
      begin
        if Abs(P_bot - LowestPbot) > 0.1 then
          WriteOnFile;
        if Soil.Ksat[1] < 0.01 then
          if P_bot > LowestPbot / 2 - 0.1 then
          begin
            P_bot := 4 * P_bot;
            WriteOnFile;
          end
          else
            if MultiStep = 0 then
            begin
              t_evaporation[1] := time_;
              WriteOnFile;
            end
        else
          if P_bot > -35.1 then
          begin
            P_bot := P_bot - 5;
            WriteOnFile;
          end
          else
          begin
            t_evaporation[1] := time_;
            WriteOnFile;
          end;
      end
    end
    else
      if P[4] < P_bot / 2 then
      begin
        if Abs(P_bot - LowestPbot) < 0.1 then
        begin
          WriteOnFile;
          t_end := time_ * 0.99;
          Application.ProcessMessages;
        end;
        if P_bot < LowestPbot / 2 - 0.1 then
        begin
          P_bot := 0.5 * P_bot;
          WriteOnFile;
        end;

      end;
end;

procedure TSimulator.DrawFigure;
var
  i: Integer;
  GYmax, GYmin: Double;
begin
  Main.Profile;
  Main.DrawProfile(W, z_c, n, 0, 0.6, clBlue);
  Main.DrawProfile(Swp , z_c, n, 0, 10 / t_unit, clTeal);
  Main.DrawProfile(Swr , z_c, n, 0, 10 / t_unit, clGreen);
  if Wstorage_in_snow > 0 then
  begin
    Main.DrawProfile(rho_snow, z_snow, NumElmSnow - 1, 0, 0.6, clBlue);
    Main.DrawProfile(Tsnow, z_snow, NumElmSnow - 1, 263.2, 323.2, $000080FF)
  end;
  if (Wice[1] > 0) or (Wice[n div 2] > 0) or (Wice[n] > 0) then
    Main.DrawProfile(Wice, z_c, n, 0, 0.6, clNavy);
  if solute then
  begin
    Main.DrawProfile(C, z_c, n, 0, Max(15 * Ceil(Max(c_ini, C_irg) / 6), 6), $009090);
    if Mode = 2 then
      Main.DrawProfile(Cm, zms, NumSampling, 0, 6, clRed);
  end;
  if heat then
    Main.DrawProfile(T, z_c, n, 263.2, 323.2, $000080FF)
  else
  begin
    if P[n] < -150 then
      dum := -900
    else
      dum := -150;
    Main.DrawProfile(P,z_c, n, dum, 0,clGray);
  end;
  if (Mode = 2) and ((IPET_type = 1) or (IPET_type = 4) or (IPET_type = 5)) then
    Main.DrawProfile(Wm, zm, NumSampling, 0, 0.6, clOlive);
  for i := 0 to n do
  begin
    ql[i] := -ql[i]*10*86400/ t_unit;
    qv[i] := -qv[i]*10*86400/ t_unit;
  end;
  Main.History;
  if Mode <> 2 then
  begin
    Main.FluxProfile;
    Main.DrawFluxProfile(ql,z_c, clNavy);
    Main.DrawFluxProfile(qv,z_c, clAqua);
  end
  else
    if water then
      Main.DrawProperty;
  for i := 1 to n do
  begin
    ql[i] := -ql[i] * t_unit / (10 * 86400);
    qv[i] := -qv[i] * t_unit / (10 * 86400);
  end;
  //---- History Drawing ----//
  if (DayCounter >= 9) and (IPET_type < 7) then
    HistoryDrawStartingPoint := Trunc(86400 * (DayCounter - 9) / (int_rec * t_unit) ) + 1;
  for i := 1 to counter do
    dumarray[i] := Wtime[1, i];
  Main.HistoryDraw(time, dumarray, HistoryDrawStartingPoint, counter, -0.6, 0.4, clBlue, psSolid);
  if solute then
  begin
    for i := 1 to counter do
      dumarray[i] := Ctime[1, i];
    Main.HistoryDraw(time, dumarray, HistoryDrawStartingPoint, counter, -12 ,   8, $009090, psSolid);
  end;
  if heat then
  begin
    for i := 1 to counter do
      dumarray[i] := Ttime[1, i];
    Main.HistoryDraw(time, dumarray, HistoryDrawStartingPoint, counter, -60 ,  40, $000080FF, psSolid);
  end;
  if Mode = 2 then
  begin
    if (IPET_type = 2) and (UseOutFlowForK = 0) and (l_bc_type = 4) and (Tensio > 0) and (z_lb0 > 0) then
      dum := z_lb / z_lb0
    else
      dum := 1;
    if fl_outflow <> '' then
      for i := 1 to NumOutflowData do
        dumarray[i] := dum * CumOutflow[i];
    case IPET_type of
    5:begin
        GYmin := -12;
        GYmax :=   8;
      end;
    6:begin
        GYmin := -30;
        GYmax :=  20;
      end;
    7..8: begin
           GYmin := -0.6;
           GYmax :=  0.4;
          end;
    else
      begin
        GYmin := -3;
        GYmax :=  2;
      end;
    end;
    if fl_outflow <> '' then
      Main.HistoryDraw(t_outflow, dumarray, 1, NumOutflowData, GYmin, GYmax, clOlive, psSolid);
    Main.HistoryDraw(time, CumD, 1, counter, GYmin, GYmax, clNavy, psSolid);
    if (IPET_type = 1) or (IPET_type = 4) then
      Main.HistoryDraw(t_evaporation, CumEvaporation, 1, NumEvapData, -3 , 2, clOlive, psSolid);
    if (IPET_type = 2) or (IPET_type = 4) or (IPET_type = 6) then
      Main.HistoryDraw(t_tensio, Ptens1, 1, NumTensioData, -600, 400, clOlive, psSolid);
    if (IPET_type <> 1) or (UseOutFlowForK = 1) then
      Main.HistoryDraw(t_P_bot, Pbot, 0, NumPbotData, -600, 400, clTeal, psDot);
    Main.HistoryDraw(time, CumE, 1, counter, -3 , 2, clAqua, psSolid);
  end
  else
  begin
    Main.HistoryDraw(time, ql_bot, HistoryDrawStartingPoint, counter, -60, 40, clNavy, psSolid);
    Main.HistoryDraw(time, Evap  , HistoryDrawStartingPoint, counter, -60, 40, clAqua, psSolid);
  end;
  for i := 1 to counter do
    dumarray[i] := Ptime[1, i];
  Main.HistoryDraw(time, dumarray, HistoryDrawStartingPoint, counter, -600, 400, clTeal, psSolid);
  //counter := counter + 1;
  Main.Report;
  qv[0]  := -qv[0]/(10*86400) * t_unit;
  ql[0]  := -ql[0]/(10*86400) * t_unit;
  ql[n]  := -ql[n]/(10*86400) * t_unit;
  avdt := 0; navdt := 0;
  Application.ProcessMessages;
end;

procedure TSimulator.OutputToFile;
var
  i, j: Integer;
  SL1: TStringList;
begin
  AssignFile(f, CaseName + '_profile.txt');
  Rewrite(f);
  Writeln(f, '>>>>>>>>>>>>>, Result of the Simulation ,<<<<<<<<<<<<<');
  Writeln(f, 'time, counter, number of calculation');
  Writeln(f, FormatFloat('#######0.000', time_) + Separator + IntToStr(counter) + Separator + IntToStr(NumCalc));
  Writeln(f, 'W_storage_ini, CumDrain, CumInfilt, CumEvap, CumTr, W_storage');
  Writeln(f, FormatFloat( '#0.0000', W_storage_ini) + Separator + FormatFloat('##0.0000', CumDrain) + Separator +
             FormatFloat('##0.0000', CumInfilt) +  Separator+ FormatFloat('##0.0000', CumEvap) +  Separator+ FormatFloat('##0.0000', CumTr)  + Separator + FormatFloat('#0.0000', W_storage_old));
  if solute then
  begin
    Writeln(f, 'S_storage_ini, Cum_qs_bot, Cum_qs_top, Crystal, S_storage');
    Writeln(f, FormatFloat( '#0.0000', S_storage_ini) + Separator+ FormatFloat('##0.000',Cum_qs_bot) + Separator +
               FormatFloat('##0.000', Cum_qs_top) + Separator + FormatFloat('##0.000', Crystal) + Separator + FormatFloat('#0.000', S_storage));
  end;
  Writeln(f, 'z(cm), VWC, T(C), c(mg/cm3), VSC(mg/cm3), Wsat, Wr, Drying, ql(cm/d), qv(cm/d)');
  for i := 1 to n do
  begin
    if (z[i] + z[i-1]) / 2 < 10 then
      dumstr := ' '
    else
      dumstr := '';
    Writeln(f, dumstr + FormatFloat('#0.000', (z[i] + z[i-1]) / 2) + Separator +  FormatFloat('0.0000', W[i] + Wice[i]) + Separator +
            FormatFloat('#0.0', T[i]-273.2) + Separator + SetLengthRJ(FormatFloat('##0.000', C[i]),5) + Separator+ SetLengthRJ(FormatFloat('##0.00', M[i]), 6)
            + Separator+ FormatFloat('0.000', Wsat_A[i])+ Separator+ FormatFloat('0.000', Wr_A[i])+ Separator+ BoolToStr(Drying[i]) + Separator+ FormatFloat('##0.00', ql[i] / t_unit * 86400) + Separator+ FormatFloat('0.00', qv[i] / t_unit * 86400));
  end;
  CloseFile(f);
//-------------------
  AssignFile(f, caseName + '_flux.txt');
  Rewrite(f);
  Writeln(f, '>>>>>>>>>>>>, Cumulative Evaporation and Outflow ,<<<<<<<<<<<<<');
  Writeln(f, 'time, CumE, CumTr, CumD, CumI, Front, Cum_qs_bot, Vreservoir');
  for i := 1 to counter do
    Writeln(f, SetLengthRJ(FormatFloat('######0.000', i * int_rec), 11) + Separator +
        SetLengthRJ(FormatFloat('#0.0000', CumE[i]),8) + Separator +
        SetLengthRJ(FormatFloat('#0.0000', CumT[i]),8) + Separator +
        SetLengthRJ(FormatFloat('#0.0000', CumD[i]),8) + Separator +
        SetLengthRJ(FormatFloat('#0.0000', CumI[i]),8) + Separator +
        SetLengthRJ(FormatFloat('#0.0000', Front[i]),8) + Separator +
        SetLengthRJ(FormatFloat('#0.0000', Cumqs[i]),8) + Separator +
        SetLengthRJ(FormatFloat('#0.0000', Vr[i]),8));
  CloseFile(f);
  for j := 1 to NumObs do
  begin
    AssignFile(f, caseName + '_observation' + IntToStr(j) + '.txt');
    Rewrite(f);
    Writeln(f, '>>>>>>>>>>>>, Values at the Observation Point ,<<<<<<<<<<<<<');
    Writeln(f, 'depth = ' + FormatFloat('##0.0', z_obs[j]) + 'cm');
    Writeln(f, 'time, VWC, pressure(cm), c(mg/cm3), T(C), avgVWC_above_the_depth, mass_of_salt_above(mg/cm2), ql(cm/d), qv(cm/d)');
    for i := 1 to counter do
      Writeln(f, FormatFloat('######0.000', i * int_rec) + Separator +
                 FormatFloat('0.0000', Wtime[j,i])+ Separator +
                 FormatFloat('#######0.0', Ptime[j,i]) + Separator +
                 FormatFloat('##0.0000', Ctime[j,i]) + Separator +
                 FormatFloat('#0.0', Ttime[j,i]) + Separator  +
                 FormatFloat('0.000', AvgWaterContentAbove[j,i]) + Separator  +
                 FormatFloat('0.000', MassOfSaltAbove[j,i]) + Separator  +
                 FormatFloat('##0.000', qltime[j,i]) + Separator  +
                 FormatFloat('##0.000', qvtime[j,i]));
    CloseFile(f);
  end;
  SL1 := TStringList.Create;
  try
    if (NumExp = 1) and (Mode = 1) then
    begin
      with SL1 do
      begin
        Clear;
        Add('Hypothetical outflow data');
        Add('time, CumOutflow');
        i := 0;
        repeat
          i := i + 1;
          Add(SetLengthRJ(FormatFloat('######0.000', i * int_rec), 11) + Separator+ SetLengthRJ(FormatFloat('#0.0000', RandG(CumD[i], ArtSTEOutflow)),8));
        until (i * int_rec > t_evaporation[1]) or (i * int_rec > t_end);
        SaveToFile(caseName + '_outflow.txt');
        dumint := i;
        if MultiStep = 1 then
        begin
          Clear;
          Add('Hypothetical tensiometer readings');
          Add('time, Pressure head');
          for i := 1 to counter do
            Add(SetLengthRJ(FormatFloat('######0.000', i * int_rec), 11) + Separator +
                SetLengthRJ(FormatFloat('#######0.0', Ptime[1,i]), 9));
          SaveToFile(caseName + '_pressure.txt');
        end
        else
        begin
        //----------------------------------------------------------
          { Hypothetical sampling for numerical experiment on Two-Stage Method assuming dz = 0.05cm }
          Clear;
          Add('Hypothetical VWC profile');
          Add('z(cm), VWC');
          for i := 1 to 4 do
          begin
            dum := 0;
            for j := 1 to 10 do
              dum := dum + W[j+(i-1)*10] / 10;
            Add(FormatFloat('#0.00', i * 0.5) + Separator + FormatFloat('0.0000', RandG(dum, ArtCEVWC * dum)));
          end;
          for i := 1 to 2 do
          begin
            dum := 0;
            for j := 1 to 20 do
              dum := dum + W[j+(i-1)*20+40] / 20;
            Add(FormatFloat('#0.00', 2 + i) + '   ' + FormatFloat('0.0000', RandG(dum, ArtCEVWC * dum)));
          end;
          dum := 0;
          for j := 1 to 24 do
            dum := dum + W[j+80] / 24;
          Add(FormatFloat('#0.00', 5.2) + '   ' + FormatFloat('0.0000', RandG(dum, ArtCEVWC * dum)));
          SaveToFile(caseName+'_profile.txt');
          //----------------------------------------------------------
          Clear;
          Add('Hypothetical water retention data');
          Add('VWC, Pressure(cm)');
          for i := 1 to 4 do
          begin
            dum := 0;
            for j := 1 to 10 do
              dum := dum + w[j+(i-1)*10] / 10;
            with Soil do
              dum2 := Pressure_Head(dum, 298.2, Wsat_A[1], zeta_A[1], Wr_A[1], 1, Drying[1]);
            if dum2 < -1000 then
              Add(FormatFloat('0.0000',dum) + Separator + FormatFloat('##########', RandG(dum2, ArtCEPsychro * dum2)));
          end;
          SaveToFile(caseName+'_psychro.txt');
          //----------------------------------------------------------
          Clear;
          Add('Hypothetical evaporation data');
          Add('time, CumEvaporation');
          i := dumint;
          repeat
            Add(SetLengthRJ(FormatFloat('######0.000', i * int_rec), 11) + Separator + SetLengthRJ(FormatFloat('#0.0000', RandG(CumE[i], ArtSTEEvap)),8));
            i := i + 6;
          until i > counter;
          Add(SetLengthRJ(FormatFloat('######0.000', counter * int_rec), 11) + Separator + SetLengthRJ(FormatFloat('#0.0000', RandG(CumE[counter], ArtSTEEvap)),8));
          SaveToFile(caseName + '_evaporation.txt');
          Clear;
        end;
      end;
    end;
  finally
    SL1.Free;
  end;
  if FileExists(Main.GetAbsoluteDirectory(BaseDir, 'BorderDepths.txt')) then
    ShowResult.SliceRings;
end;

procedure TSimulator.Observate;
var
  i, j: Integer;
  zu, zl: Double;
  SL: TStringList;
begin
  time[counter] := time_;
   // W2cmtime(counter) := Walbedo
   // Tair[counter] := Tair;
   // Crystal[counter] := Crystal
  Wb[0] := Wb[1];
  for j := 1 to NumObs do
  begin
    Wtime[j, counter] := (W[NCtime[j]] * (z_obs[j] - z_c[NCtime[j]-1]) + W[NCtime[j]-1] * (z_c[NCtime[j]] - z_obs[j])) / ((dz[NCtime[j]] + dz[NCtime[j]-1]) / 2);
    if (P[NCtime[j]] < 0) and (P[NCtime[j]-1] < 0) then
      Ptime[j, counter] := - exp((ln(- P[NCtime[j]]) * (z_obs[j] - z_c[NCtime[j]-1]) + ln(- P[NCtime[j]-1]) * (z_c[NCtime[j]] - z_obs[j])) / ((dz[NCtime[j]] + dz[NCtime[j]-1]) / 2))
    else
      Ptime[j, counter] := (P[NCtime[j]] * (z_obs[j] - z_c[NCtime[j]-1]) + P[NCtime[j]-1] * (z_c[NCtime[j]] - z_obs[j])) / ((dz[NCtime[j]] + dz[NCtime[j]-1]) / 2);
    Ctime[j, counter] := C[NCtime[j]];
    Ttime[j, counter] := - 273.2 + (Tb[NCtime[j]] * (z_obs[j] - z[NCtime[j]-1]) + Tb[NCtime[j]-1] * (z[NCtime[j]] - z_obs[j])) / dz[nctime[j]];
    AvgWaterContentAbove[j, counter] := GetAvgWaterContentAbove(z_obs[j]);
    MassOfSaltAbove[j, counter] := GetMassOfSaltAbove(z_obs[j]);
    qltime[j, counter] := 86400/ t_unit * (ql[NCtime[j]] * (z_obs[j] - z[NCtime[j]-1]) + ql[NCtime[j]-1] * (z[NCtime[j]] - z_obs[j])) / ((dz[NCtime[j]] + dz[NCtime[j]-1]) / 2);
    qvtime[j, counter] := 86400/ t_unit * (qv[NCtime[j]] * (z_obs[j] - z[NCtime[j]-1]) + qv[NCtime[j]-1] * (z[NCtime[j]] - z_obs[j])) / ((dz[NCtime[j]] + dz[NCtime[j]-1]) / 2);
  end;
  Evap[counter]  := - qv[0]*10*86400/ t_unit;
  if NumExp = 1 then
    if Evap[counter] > MaxEvap then
      MaxEvap := Evap[counter]
    else
      if Evap[counter] < MaxEvap * 0.1 then
        t_end := time_ * 0.99;
  ql_bot[counter] :=  - ql[n]*10*86400/ t_unit;
  CumE[counter] := CumEvap; //qv[0] * 3600 * 24
  CumT[counter] := CumTr;
  CumD[counter] := CumDrain;//ql[n] * 10 * 86400
  CumI[counter] := CumInfilt;//ql[n] * 10 * 86400
  Cumqs[counter] := Cum_qs_bot;//ql[n] * 10 * 86400
  Vr[counter] := Vreservoir;

  for i := 1 to n do
    if ((W[i] < Wfront) and (W[i+1] > Wfront)) or ((W[i] > Wfront) and (W[i+1] < Wfront)) then
    begin
      zu := 0.5 * (z[i-1] + z[i]);
      zl := 0.5 * (z[i]   + z[i+1]);
      Front[counter] := (zu * (W[i+1] - Wfront) + zl * (Wfront - W[i])) / (W[i+1] - W[i]);
      break;
    end;
  SL := TStringList.Create;
  try
    if rec = 1 then
    begin
      with SL do
      begin
        Clear;
        CumTrp := Max(0.0001, CumTrp);
        Add(FormatFloat('##000000.0000', time_) + Separator + FormatFloat('##0.000', avdt) + Separator + IntToStr(navdt) + Separator +
            FormatFloat('0.000000', WCumRError) + Separator+ FormatFloat('0.000000', SCumRError)  + Separator+ FormatFloat('0.000000', ECumRError) + Separator + FormatFloat('#0.000', Hpond) + Separator +
            FormatFloat('#0.000', CumEvap) + Separator+ FormatFloat('#0.000', CumTr)  + Separator+ FormatFloat('#0.000', CumInfilt)  + Separator+ FormatFloat('0.00', CumTr / CumTrp)  + Separator+ FormatFloat('#0.00', Vreservoir));
        Add('z(cm), VWC, psi(cm), T(C), c(mg/cm3), VWC*c(mg/cm3), ql(mm/d), qv(mm/d), -Swp, -Swr');
      //  ql[0] := ql[1];
        dum := qv[0];
        qv[0] := qv[1];
        for i := 1 to n do
        begin
          if (z[i] + z[i-1]) / 2 < 10 then dumstr := ' ' else dumstr := '';
          Add(dumstr + FormatFloat('#0.00',(z[i] + z[i-1]) / 2) + Separator + FormatFloat('0.0000',W[i]) + Separator +
              SetLengthRJ(FormatFloat('#######0.0', P[i]),8) + Separator +
              FormatFloat('#0.0', T[i]-273.2) + Separator + SetLengthRJ(FormatFloat('##0.00',C[i]), 5) + Separator +
              SetLengthRJ(FormatFloat('##0.00', W[i]*C[i]),6) + Separator + SetLengthRJ(FormatFloat('##0.00', ql[i]/t_unit * 864000),6) + Separator+ SetLengthRJ(FormatFloat('##0.000', 0.5*(qv[i] + qv[i-1])/t_unit*864000),4) + Separator+
              FloatToStr(-Swp[i]/t_unit*864000) + Separator + FloatToStr(-Swr[i]/t_unit*864000));
        end;
        dumstr := BaseDir + Slash + 'History' + Slash;
        if DirectoryExists(dumstr) = False then
          Createdir(dumstr);
        SaveToFile(dumstr + ExtractFileName(caseName) + '_' + FormatFloat('000', counter) + '.txt');
        qv[0] := dum;
      end;
    end;
  finally
    SL.Free;
  end;
end;

function TSimulator.GetAvgWaterContentAbove(Depth: Double): Double;
var
  i, j: Integer;
begin
  j := 0;
  while z[j] <= Min(Depth, z_lb) do
    j := j + 1;
  if z[1] <> Min(Depth, z_lb) then
  begin
    dum := 0;
    for i := 1 to j - 1 do
    //  dum := W[i] + dum;
      dum := W[i] * dz[i] + dum;
    //dum := dum / (j - 1);
    dum := dum / z[j];
  end
  else
    dum := w[1];
  Result := dum;
end;

function TSimulator.GetMassOfSaltAbove(Depth: Double): Double;
var i, j: Integer;
begin
  j := 0;
  while z[j] <= Depth do
    j := j + 1;
  if z[1] <> Depth then
  begin
    dum := 0;
    for i := 1 to j - 1 do
      dum := M[i] * dz[i] + dum;
  end
  else
    dum := M[1] * z[1];
  Result := dum;
end;
(*>>>>>>>>>>>>>>>>>>>>>>>   Atmospheric Boundary Condition  <<<<<<<<<<<<<<<<<<<<<<<<<*)
function TSimulator.CalculateAtmos_BC: Boolean;
var
  i : Integer;
  t_prev, t_next, ETpenman, Delta_over_PsycConst, Cvs, LH, Tsurface, hour_angle, cosz, RSD, Delta, ETref: Double;
begin
  i := 0; t_prev := 0; t_next := 0;
  if (abc_type >= 3) then
  begin
    i := abc_done;
    while t_clm_con[i] <= time_ do
      i := i + 1;
    if i > abc_done then
    begin
      PR[i-1] := ql[0];
      Rg[i-1] := Rshort;
      abc_done := i;
    end;
    t_prev := t_clm_con[i-1];
    t_next := t_clm_con[i];
  end;
  case abc_type of
  1:qv[0] := 0;
  2:begin
      if (time_ < t_evaporation[1]) or (MultiStep = 1) then
        Cv[1] := Cva;
      qv[0] := -(Cv[1] - Cva) / ((ra + rsc + rm)/ t_unit);
    end;
  3:begin
      qv[0] := - (CumEvaporation[i] - CumEvaporation[i-1]) / ( t_next - t_prev );
      if P[1] < -10000 then
      begin
        if P[1] < -1000000 then
        begin
          Result := False;
          Exit;
        end
        else
        begin
          dum := log10(-P[1]);
          qv[0] := (ql[1] * (dum - 4) + qv[0] * (5 - dum));// dum * 10000 / (- P[n]);  //0.5 * ql[5];
        end;
      end;
    end;
  4:begin
      Rshort := (Rs[i-1] * ( t_next - time_ ) + Rs[i] * (time_ - t_prev )) / ( t_next - t_prev );
      q_rain:= (Prcp[i-1] * ( t_next - time_ ) + Prcp[i] * (time_ - t_prev )) / ( t_next - t_prev );
      Vwind := (wv[i-1] * ( t_next - time_ ) + wv[i] * (time_ - t_prev )) / ( t_next - t_prev );
      Tair  := (Ta[i-1] * ( t_next - time_ ) + Ta[i] * (time_ - t_prev )) / ( t_next - t_prev );
      RHair := (RHa[i-1]* ( t_next - time_ ) + RHa[i]* (time_ - t_prev )) / ( t_next - t_prev );
      Tair := Max(250, Tair);
      Cva   := RHair * (0.00000002704 * Tair * Tair - 0.000014722 * Tair + 0.0020093);//exp(6 - 4976 / Tair);
      Vwind := Max(Vwind, 0.4);
      //-- aerodynamic resistance --//
      ra := (1 + 0.2 * LAI) * 0.01 * ln(2000) * ln(2000) / (0.16 * Vwind); //surface roughness=0.1cm
      if Wstorage_in_snow = 0 then
        Tsurface := T[1]
      else
        Tsurface := Tsnow[iss];
      if Tair < Tsurface then
      begin
        dum := 9.81 * (2 - 0.01) * (Tair - Tsurface) / (Tair * Vwind * Vwind);
        ra := ra / (1 - 10 * dum);
      end;
      if T[1] > 270 then
        qv[0] := -(Cv[1] - Cva) / ((ra + rsc + rm)/ t_unit)
      else
        qv[0] := 0;
      //--- downward radiation ---//
      Rldown := t_unit * (0.605 + 0.048 * Sqrt(1370000 * Cva) + KRl) * StephBolz * IntPower(Tair,4) ;
      //------- albedo -------//
      if Wstorage_in_snow = 0 then
        with Soil do
        begin
          if z[1] <> Soil.ThickAlb[1] then
          begin
            Walb := 0;
            for i := 1 to NCalb - 1 do
              Walb := W[i] * dz[i] + Walb;
            Walb_bot := (Wb[NCalb] * (ThickAlb[1] - z[NCalb-1]) + Wb[NCalb-1] * (z[NCalb] - ThickAlb[1])) / dz[NCalb];
            Walb := (Wb[NCalb-1] + Walb_bot) * dz[NCalb] / 2 + Walb;
            Walb := Walb / ThickAlb[1];
          end
          else
            Walb := w[1];
          albedo := AlbedoMin[1] + (AlbedoMax[1] - AlbedoMin[1]) / (1 + Power(Aal[1] * Max(0, Walb), Bal[1]));
        end;
     //----------------------//
      Rlup := t_unit * (0.9 + 0.18 * w[1]) * StephBolz * IntPower(Tsurface,4);
      Rl := Rldown - Rlup;
     // Rg := Rg0 * exp(-.82 * LAI)
     // Rl := Rldown * (.5 + .5 * exp(-LAI))
      Rn := (1 - albedo) * KRshort * Rshort + Rl;
     //----------------------------------------------------------------------------------------------------------------------------------//
      if (Wstorage_in_snow = 0) and ((Tair > 273.2) or (q_rain = 0)) then
        qh[0] := Rn + (Tair - T[1]) * 0.0012 / ( 0.5 * ra / t_unit) + ql[0] * 4.18 * (Tair - 1) + qv[0] * 4.18 * T[1]
      else
      begin
        qh[0] := 0.2 * kh[1] * (Tsnow[NumElmSnow] - T[1]) / 1;// + q_rain * 4.18 * 273.2;
        qh_snow[NumElmSnow] := qh[0];
      end;
      //----------------------------------------------------------------------------------------------------------------------------------//
      // Please check!! Is ra for heat the same as that of vapor ?
      //--- Estimatio of potential transpiration rate using the Penman Monteith Equation ---//
      hour_angle := 3.142 * (12 - time_in_a_day * t_unit / 3600) / 12;
      cosz := (sin(latitude) * sin(dec_sun) + cos(latitude) * cos(dec_sun) * cos(hour_angle));
      if (cosz > 0.02) and (Rshort > 0) then
        RSD := Min(1, Rshort / (0.1367 * t_unit * exp(-0.288 / cosz) * cosz))
      else
        RSD := 1; // 0.288 = ln(0.75)
      Cvs := 0.61 * exp(17.3 * (Tair - 273.2) / (Tair - 273.2 + 237.3));  //kPa
      Cva   := RHair * Cvs;
      Rl := t_unit * (- 0.34 + 0.14 * Sqrt(Cva)) * (1.35 * RSD - 0.35) * StephBolz * IntPower(Tair,4) ;//51315 *
      Rn := (1 - 0.23) * Rshort + Rl; // J / (t_unit cm2)
      Delta := 4098 * CVa / Sqr(Tair - 273.2 + 237.3);  // 51315 * CVa = vapor pressure in kPa
      LH := (597 - 0.6 * (Tair - 273.2)) * 4.18; // J/cm3
      //ETref := t_unit * 0.45 * exp(6 - 4976 / Tair) * (1 - RHair) * (0.5 + 0.537 * Vwind);
      //ETref := Max(0, (Delta_over_PsycConst * Rn / LH + ETpenman) / (Delta_over_PsycConst + 1)); // cm/t_unit
      ETref := 0.1 * (0.408 * Delta * 0.01 * Rn + 0.0674 * (0.0103 * t_unit / Tair) * Cvs * (1 - RHair) * Vwind)
            / (Delta + 0.0674 * (1 + 0.34 * Vwind));    //cm/t_unit. Coefficient 0.01 is used to convert from  J / (t_unit cm2) to MJ / (t_unit m2).
      if Tair > 278.2 then
        Trp := ETref * Kc
      else
        Trp := 0;
     //--- Estimatio of potential transpiration rate using the Penman Equation ---//
      (*
      Rldown := t_unit * (0.605 + 0.048 * Sqrt(1370000 * Cva)) * StephBolz * IntPower(Tair,4) ;
      Rlup := t_unit * 0.95 * StephBolz * IntPower(Tair,4);
      Rl := Rldown - Rlup;
      Rn := (1 - 0.25) * Rshort + Rl; // J / (t_unit cm2)
      Delta_over_PsycConst := exp(0.0521 * (Tair - 273.2) - 0.294);
      //0.00000045 = 0.1 * 0.35 / 864000 (mm/d --> cm/s) from Hillel, p604
      // 1 mmHg = 1 g/m3 of water vapor = 0.0000001 g/cm3
      //53.7 = 86400 / 1609 (m/s --> miles/d)
      LH := (597 - 0.6 * (Tair - 273.2)) * 4.18; // J/cm3
      ETpenman := t_unit * 0.45 * exp(6 - 4976 / Tair) * (1 - RHair) * (0.5 + 0.537 * Vwind);
      ETpenman := Max(0, (Delta_over_PsycConst * Rn / LH + ETpenman) / (Delta_over_PsycConst + 1)); // cm/t_unit
      if Tair > 278.2 then
        Trp := ETpenman * Kc
      else
        Trp := 0;
      *)
    end;
  5:begin
      Trp := (Tp[i-1] * ( t_next - time_ ) + Tp[i] * (time_ - t_prev )) / ( t_next - t_prev );
      qv[0] := 0;
    end;
  end;
  if (abc_type = 2) or (abc_type = 4) then
  begin
    //-- salty crust resistance --//
    if (S_crust = 1) and (Crystal > 0.001) then
    begin
      //if Crystal < 5 then
      dum := Crystal + exp(- br / ar);
      {
      else
      begin
        dum := 0; i := 1;
        repeat
          dum := dum + M[i] * dz[i];
          i := i + 1;
        until z[i] > 0.25;
        dum := dum + M[i] * (0.25 - z[i-1]);
      end;
      }
     // dum := dum + Crystal;
     // rsc := 0.39 * ln(dum) - 0.67; // S1Masa 1999
      rsc := ar * ln(dum) + br;
      if rsc < 0.0 then
        rsc := 0.0;
    end
    else
      rsc := 0;
  end;
  Result := True;
end;
(*>>>>>>>>>>>>>>>>>>>>>>>   Water Vapor Movement  <<<<<<<<<<<<<<<<<<<<<<<<<*)
procedure TSimulator.CalculateVaporMovement(ToEnd: Integer);
var
  i, j: Integer;
  chi, Po, Dvb, Cvb, RHb, ENb: Double;

  function EnhancementFactor(x: Double; j: Integer): Double;
  var
    i: Integer;
  begin
    if (Segments = True) and (EnPrepared[j] = False) then
    begin
      Segments := False;
      EnPrepared[j] := True;
      for i := 0 to 20 do
        Enseg[j,i] := EnhancementFactor(i /20, j);
      Segments := True;
    end;
    if (Segments = False) or ((Segments = True) and (EnPrepared[j] = False)) then
    begin
      with Soil do
        Result := Aen[j] + Ben[j] * x - (Aen[j] - 1) * exp(-Cen[j] * Power(x, Een[j]));
    end
    else
    begin
      i := Trunc(x * 20);
      //i := Min(Max(i, 0), 20);
      Result := (Enseg[j,i] * ((i + 1) * 0.05 - x) + Enseg[j+1,i] * (x - i * 0.05)) / 0.05;
    end;
  end;

begin
  //-------------------------------------//
  for i := 1 to ToEnd do
  begin
    chi  := a_oc * Sqr(C[i]) + b_oc * C[i] + c_oc;  //a_oc * C[i,j] + b_oc * Power(C[i,j], c_oc) + 1
    Po   := -10.2 * (NumIon * C[i] / Mweight) * chi * 8.314 * T[i];
    Pw[i]:= P[i] + Po ;
    if Pw[i] < -10000 then
      RH[i] := 1 + Pw[i] / 4697 / T[i]
    else
      RH[i] :=  exp(Pw[i] / 4697 / T[i]);
    if (heat) or (time_ = 0) then
      Cvsat[i] := 0.00000002704 * T[i] * T[i] - 0.000014722 * T[i] + 0.0020093;//exp(6 - 4976 / T[i]);  6% difference in speed
    Cv[i]   := Cvsat[i] * RH[i];
  end;
  with Soil do
    for i := 1 to ToEnd - 1 do
    begin
      j := L[i];
      if heat then
        En[i] := EnhancementFactor(Max(0, W[i] / Wsat[j]), j)
      else
        En[i] := 1;
      Dv[i] := t_unit * 0.229 * Power(T[i] / 273.2, 1.75) * 0.66* (Wsat[j] - W[i+1]); //^ (5 / 3)
      if (Wsat[j] > 0.85) and (z[i] < 0.2) then  // if the porous media is gause,
        Dv[i] := Dv[i] * 2;
      if (Mode = 2) and (IPET_type = 3) and (time_ > t_slicing) and (Abs(z[i] - z_slice) < 0.05) then
        Dv[i] := 0;
    end;
  for i := 1 to ToEnd - 1 do
  begin
    Dvb := (Dv[i] * dz[i+1] + Dv[i+1] * dz[i]) / (dz[i] + dz[i+1]);
    Cvb := (Cv[i] * dz[i+1] + Cv[i+1] * dz[i]) / (dz[i] + dz[i+1]);
    RHb := (RH[i] * dz[i+1] + RH[i+1] * dz[i]) / (dz[i] + dz[i+1]);
    ENb := (En[i] * dz[i+1] + En[i+1] * dz[i]) / (dz[i] + dz[i+1]);
    qv[i] := - Dvb * (ENb * RHb * (Cvsat[i+1] - Cvsat[i])
              + Cvb / (4712 * Tb[i]) * (Pw[i+1] - Pw[i]))
              / (z_c[i+1] - z_c[i]);
  end;
 //---------------------------------------------//
  for i := 1 to ToEnd do
    Sw[i] := -(qv[i] - qv[i-1]) / dz[i];
end;
(*>>>>>>>>>>>>>>>>>>>>>>>   Liquid Water Flow   <<<<<<<<<<<<<<<<<<<<<<<<<*)
function TSimulator.CalculateLiquidFlow: Boolean;
var
  i, j, j2: Integer;
  Flag, SinkType: Shortint;
  Kmid: TValues;
  t_prev, t_next, Athd, Bthd, Cthd, P_, Po: Double;
  //Teffect: Double;

  function ReductionFunction(P, P50, pow: Double): Double;
  begin
    Result := 1 / (1 + Power(P / P50, pow));
  end;

begin
  Result   := True;
  Canceled := False;
try
 //---- root water uptake ----//
  if (Trp > 0) and (PlantName <> '') then
  begin
    Tr := 0;
    for i := 1 to n do
    begin
      if  NRD[i] > 0 then
      begin
        Po := Pw[i] - P[i];
        if (P[i] < ThresholdPotential) or (Po < ThresholdPotentialForSalinityStress) then      // this technique enhance speed by 10%
          alpha[i] := 1 / (1 + Power(Max(0, P[i] / Pm50) + Max(0, Po / Po50), pm_))//ReductionFunction(P[i]        , Pm50, pm_)
        else
          alpha[i] := 1;
        Swp[i] := NRD[i] * Trp;
        Swr[i] := Swp[i] * alpha[i];// * alpha_s[i];
        Tr := Tr + Swr[i] * dz[i];
      end
      else
      begin
        Swp[i] := 0;
        Swr[i] := 0;
      end;
      Sw[i] := Sw[i] - Swr[i];
    end;
    CumTr  := CumTr  + Tr  * dt;
    CumTrp := CumTrp + Trp * dt;
  end;
 //---- Freezing and Thawing ---//
  for i := 1 to n do
    Sw[i] := Sw[i] - Swi[i];
 //---- upper boundary ----//
  SinkType := 1; // initialization
  case u_bc_type of
  1:q_irg := ql_top;
  2:if P[1] > Hpond then
      q_irg := ql[1] * 0.95
    else
      q_irg := ql[1] * 1.05;
  3..4:begin
      if (FileExists(fl_ql_top)) or (Mode = 3) or (RefillInterval > 0) then
      begin
        i := qltop_done;
        while t_ql_top[i] < time_ do
          i := i + 1;
        qltop_done := i;
        t_prev := t_ql_top[i-1];
        t_next := t_ql_top[i];
        if t_next > t_prev then
          //q_irg := (qltop[i-1] * ( t_next - time_ ) + qltop[i] * ( time_ - t_prev )) / ( t_next - t_prev );//until 2.3
          q_irg := (CumIrrigation[i] - CumIrrigation[i-1]) / (t_next - t_prev);
      end;
    end;
  end;
  if ObsPtIrg > 0 then
  begin
    if u_bc_type = 4 then
      q_irg := 0;
    if (time_ < t_ql_top[2]) and (Vreservoir > 0) then
      q_irg := ql_top
    else
    begin
      if (counter > 1) and (Wtime[ObsPtIrg, counter-1]  < CriticalValue) and (time_ > t_ql_top[1]) then
      begin
        t_ql_top[2] := time_ + AutoIrgDepth / ql_top;
        t_ql_top[1] := t_ql_top[2] + 43200 / t_unit;
        q_irg := 0;
      end
      else
      begin
        if u_bc_type = 3 then
          qltop_done := 3;
        t_ql_top[2] := -1000000;
      end;
    end;
  end;
  if RefillInterval > 0 then
  begin
    if (time_ > t_ql_top[3]) and (CumIrrigation[4] = CumIrrigation[3])  then
    begin
      IrgCapacity := 0;
      for j := 1 to n do
         IrgCapacity := IrgCapacity + (Wfc - W[i]) * dz[i];
      CumIrrigation[4] := CumIrrigation[3] + Max(IrgCapacity, 0);
      t_ql_top[4] := t_ql_top[3] + Max(IrgCapacity, 0) / ql_top;
    end;
    if (time_ > t_ql_top[4]) then
    begin
      t_ql_top[3] := t_ql_top[3] + RefillInterval * 86400 / t_unit;
      t_ql_top[4] := t_ql_top[3] + 1000000000;
      CumIrrigation[4] := CumIrrigation[3];
    end;
    qltop_done := 0;
  end;

  if (Tair > 273.2) and (Wstorage_in_snow = 0) then
    ql[0] := q_irg + q_rain
  else
    ql[0] := 0;
 if P[1] > 0 then
  begin
   // ql[0] := ql[1] * 0.999;  // fine tuning is required !
    if (u_bc_type <> 2) and ((Hpond > 0) or (ql[0] > 0)) then
      Hpond := (ql[0] - ql[1]) * dt + Hpond;
  end;
  {
  if Hpond > 0 then
  begin
    Pold[1] := Hpond;
    W[1] := Wsat[1] + Soil.Cwsat * Hpond;
  end;
  }
  Sw[1] := Sw[1] + ql[0] / dz[1];
  //Sw[1] := Sw[1] + 0.5 * ql[0] / dz[1];
  //Sw[2] := Sw[2] + 0.5 * ql[0] / dz[2];
 //---- lower boundary ----//
  case l_bc_type of
  1:  P[n+1] := P_bot + dz[n] / 2;
  2:begin
      P[n+1] := P[n];
      SinkType := 1;
    end;
  4:begin
      i := Pbot_done;
      while t_p_bot[i] < time_ do
        i := i + 1;
      Pbot_done := i;
      t_prev := t_p_bot[i-1];
      t_next := t_p_bot[i];
      if t_next > t_prev then
        P[n+1] := (Pbot[i-1] * ( t_next - time_ ) + Pbot[i]* ( time_ - t_prev )) / ( t_next - t_prev );
    end;
  5:begin
      if (t_outflow[1] = 0) and (OutFlow_done = 1) then
        OutFlow_done := 2;
      i := OutFlow_done;
      while (t_outflow[i] < time_) and (i < NumOutflowData_for_DP) do
        i := i + 1;
      OutFlow_done := i;
      if SinkType = 1 then
      begin
        //dum := (CumOutFlow[i-1] * ( t_next - (time_ + dt)) + CumOutFlow[i]* ((time_ + dt) - t_prev )) / ( t_next -  t_prev );
        dum := (CumOutFlow[i] - CumOutFlow[i-1]) / ( t_outflow[i] - t_outflow[i-1] );
        if P[n] > - 10000 then
          ql[n] := dum
        //  ql[n] := (dum - CumDrain) / dt
        else
        begin
          {
          if P[n] > - 10000 then
            ql[n] := dum * 5000 / - P[n]
          else
          }
          begin
            Result := False;
            Application.ProcessMessages;
            Exit;
          end;
        end;
        Sw[n] := Sw[n] - ql[n] / dz[n];
      end;
      P[n+1] := P[n];
      //--------------
      {
      i := Pbot_done;
      while t_p_bot[i] < time_ do
        i := i + 1;
      Pbot_done := i;
      t_prev := t_p_bot[i-1];
      t_next := t_p_bot[i];
      if t_next > t_prev then
        dum := (Pbot[i-1] * ( t_next - time_ ) + Pbot[i]* ( time_ - t_prev )) / ( t_next - t_prev );
      if SinkType = 0 then
        P[n+1] := dum;
      }
    end;
  end;
  with Soil do
    W[n+1] := Water_Content(P[n+1], T[n+1], Wsat_A[n+1], zeta_A[n+1], Wr_A[n+1], NumLayer, Drying[n+1]);
  for i := 1 to n do
  begin
    P[i]     := Pold[i];
    Wtent[i] := W[i];
  end;
  Wtent[n+1] := W[n+1];
  case IterationScheme of
  0: iteration := 20;
  1: iteration := 19;
  2: iteration :=  0;
  end;
  //=================================================
  repeat
    iteration := iteration + 1;
    NumCalc := NumCalc + 1;
    //---- Unsaturated Hydraulic Conductivity ----//
    if InterHyCon < 2 then
      for i := 1 to n + 1 do
      begin
        if C[i] > 1 then
          dum := 1 / (1 + 0.0029 * Power(0.1 * C[i], 1.77))
        else
          dum := 1;
        Kmid[i] := dum * Soil.HydraulicConductivity(0.5 * (W[i] + Wtent[i]), T[i], L[i]);
      end;
    for i := 1 to n do
    begin
      //---- Fingering ----//
      {
      if ((W[i] < 0.01) and (P[i+1] < -5)) or ((W[i+1] < 0.01) and (P[i] < -5)) then
        K[i] := 0
      else
      }
      begin
      //  if (iteration = 1) or ((W[i] > 0.01) and (W[i+1] > 0.01)) then
        begin
      //    if (time_ > 0) and (K[i] = 0) and (dt > 0.0001) then
      //      dt := 0.00000001 * dt;
          case InterHyCon of
          0:begin
              if Kmid[i] < Kmid[i+1] then
                K[i] := 0.75 * Kmid[i] + 0.25 * Kmid[i+1]
              else
                K[i] := 0.25 * Kmid[i] + 0.75 * Kmid[i+1];
              //K[i] := 0.5 * (Kmid[i] + Kmid[i+1]);
            end;
          1: K[i] := Sqrt(Kmid[i] * Kmid[i+1]);
          2: K[i] := Soil.HydraulicConductivity(0.25 * (W[i] + W[i+1] + Wtent[i] + Wtent[i+1]), T[i], L[i]);
          end;
          with Soil do
            if K[i] > t_unit * Ksat[L[i]] then
              K[i] := t_unit * Ksat[L[i]];  // for fingering
        end;
      end;
    end;
    if i_mb > 0 then
    begin
      i := i_mb;
      r_mb := 1 / (Abs(HC_mb) * Power(AEVmembrane / Min(0.5 * (P[i] + P[i+1]), AEVmembrane), ExponentMembrane));
      //dum := K[i] / (dz[i] / (dz[i] / K[i] + r_mb));
      K[i] :=  dz[i] / (dz[i] / K[i] + r_mb);
    end;
    for i := 1 to n + 1 do
    begin
      if C[i] > 20 then
      begin
        rho[i] := 0.001 * (1 - 0.4) * C[i] + 1;  // density of water
        if s_type = 1 then
          K[i] := K[i] / (15.2 * Sqr(0.001 * 0.5 * (C[i] + C[i+1])) + 0.000318 * 0.5 * (C[i] + C[i+1]) + 1); // viscosity effect
      end
      else
        rho[i] := 1;
      if (Mode = 2) and (IPET_type = 3) and (time_ > t_slicing) and (Abs(z[i] - z_slice) < 0.05) then
        K[i] := 0;
    end;
    //--- lower boundary ---//
    if l_bc_type = 3 then
      K[n] := 0;
    if (l_bc_type = 5) and (SinkType = 1) then
      K[n] := 0;
    if (l_bc_type = 1) or (l_bc_type = 4) or (SinkType = 0) then
      if (HC_filter > 0) and ((IPET_type <> 2) or (UseOutFlowForK = 1) or (l_bc_type <> 4)) then // and (Tensio <> 2)
        K[n] := HC_filter * t_unit * dz[n];
      //  K[n] := 1 / (1 / HC_filter + Abs(parlb_a) * Power(CumDrain + 0.00001, Parlb_b)) * t_unit * dz[n];
    if (NumExp = 1) or (IPET_type = 1) then
      if (time_ >= t_evaporation[1]) and (MultiStep <> 1) then
        K[n] := 0;
    //----- Water Capacity -----//
    for i := 1 to n do
    begin
      j := L[i];
      if IterationScheme = 1 then
        P_ := 0.5 * (P[i] + Pold[i])
      else
        P_ := P[i];
      with Soil do
        Cw[i] := Water_Capacity(P_, T[i], Wsat_A[i], zeta_A[i], Wr_A[i], j, Drying[i]);
    end;
    //------- Seepage face -------//
    if l_bc_type = 6 then
    begin
      if P[n] > p_bot - dz[n] / 2 then
      begin
        K[n]   := 2 * K[n];
        P[n+1] := p_bot - dz[n] / 2   // free drainage
      end
      else
        P[n+1] := P[n] + dz[n+1]; // zero flux
    end;
    with Soil do
      W[n+1] := Water_Content(P[n+1], T[n+1], Wsat_A[n+1], zeta_A[n+1], Wr_A[n+1], NumLayer, Drying[n+1]);
    //------- Building of Coefficient Matrix -------//
    if IterationScheme = 1 then
    begin
      for i := 1 to n do
        P[i] := Pold[i];
    end;
  //  CalculateVaporMovement(8);
    if (iteration = 1) or (IterationScheme < 2) then
      Flag := 0
    else
      Flag := 1;
    for i := 1 to n do
    begin
      U[i]    := Gravity * (rho[i] * K[i] - rho[i-1] * K[i-1]) / dz[i] - Sw[i] + Flag * (Wtent[i] - W[i]) / dt;
      Ath[i+1]:=   2 * ETA *  K[i] / (dz[i+1] * (z[i+1] - z[i-1]));
      Bth[i]  := -2 * ETA * (K[i] / (dz[i]   * (z[i+1] - z[i-1])) + K[i-1] / (dz[i] * (z[i] - z[i-2]))) - Cw[i] / dt;
      Cth[i]  :=  2 * ETA *  K[i] / (dz[i]   * (z[i+1] - z[i-1]));
      Athd    := -2 * (1 - ETA) *  K[i-1] / (dz[i] * (z[i]   - z[i-2])) * P[i-1];
      Bthd    := (2 * (1 - ETA) * (K[i]   / (dz[i] * (z[i+1] - z[i-1]))
                    + K[i-1] / (dz[i] * (z[i] - z[i-2]))) - Cw[i] / dt) * P[i];
      Cthd    := -2 * (1 - ETA) *  K[i]   / (dz[i] * (z[i+1] - z[i-1])) * P[i+1];
      Dth[i]  := Athd + Bthd + Cthd + U[i];
    end;
    Dth[n] := Dth[n] - Cth[n] * P[n+1]; // 01/06/01 changed
    SolveTheCoefficientMatrix;
    MaxDiffW := 0;
    for i := 1 to n do
    begin
     // if x[i] < 100 then
      if (x[i] < 100) and (x[i] > -10000000000) then
        P[i] := x[i]
      else
      begin
        dt := dt;
        {
        dt := dt / 10;
        CanceledTime := CanceledTime + dt;
        Canceled := True;
        Exit;
        }
      end;
      //if P[i] > 200 then
      //  P[i] := 0;  // For fingering
      j := L[i];
      with Soil do
        dum := Water_Content(P[i], T[i], Wsat_A[i], zeta_A[i], Wr_A[i], j, Drying[i]);
      if MaxDiffW < ABS(dum - Wtent[i]) then
        MaxDiffW := ABS(dum - Wtent[i]);
      Wtent[i] := dum;
    end;
  until ((MaxDiffW < CriteriaForCeliasIteration) or (iteration > 20));
  //---- Water Budget Check -----//
  W_storage_new := 0;
  for i := 1 to n do
    W_storage_new := (Wtent[i] + Wice[i]) * dz[i] + W_storage_new;
  ql[n] := -K[n] * ((Pold[n+1] - (ETA * P[n] + (1 - ETA) * Pold[n])) / ((z[n+1] - z[n-1])/2) - 1);
  if (l_bc_type = 5) and (SinkType = 1) then
    ql[n] := - Sw[n] * dz[n];
  // Each Step Relative Error for Water
  ESRErrorW := ABS((W_storage_new - W_storage_old) - (ql[0] + qv[0] - ql[n] - Tr) * dt) / W_storage_new;
  if (ql[0] + qv[0] - ql[n] - Tr) <> 0 then
    ESErrorW := ((W_storage_new - W_storage_old) - (ql[0] + qv[0] - ql[n] - Tr) * dt) / ABS((ql[0] + qv[0] - ql[n]) * dt);
  MaxDiff_logP := 0.01;
  MaxDiffW := 0;
  for i := 1 to n do
  begin
    if (Pold[i] <> 0) then
      MaxDiff_logP := Max(MaxDiff_logP, Max(-Pold[i], -P[i]) / Min(-Pold[i], -P[i]));
     // MaxDiff_logP := Max(MaxDiff_logP, ln(-Pold[i]/-P[i]));
    Pold[i] :=  P[i];
    W[i]   := Wtent[i];
    j := L[i];
    //----- Handling of Hysteresis ----//
    if (ConsiderHysteresis = 1) and ((Mode <> 2) or (IPET_type > 6)) then
      if Drying_[i] then
      begin
        if ((W[i] - Wold[i]) / dt * t_conv > Margin) then
        begin
          SetScanningCurve(j, i, False);
          Drying_[i] := False;
          Drying[i]  := False;
        end
        else
          with Soil do
            if (Wsat_A[i] < Whys1[j]) and ((RF[j] = 1) and (W[i] < Whys2[j])) or ((RF[j] = 2) and (W[i] < Whys3[j])) then
            begin
              dum := Water_Content(P[i], T[i], Wsat[j], zeta[j], Wr[j], j, False);
              if W[i] < dum then
              begin
                Drying[i]  := False;
                Wsat_A[i] := Soil.Wsat[j];
                Wr_A[i]   := Soil.Wr[j];
              end;
            end;
      end
      else
      begin
        if ((W[i] - Wold[i]) / dt * t_conv < -Margin) then
        begin
          SetScanningCurve(j, i, True);
          Drying_[i] := True;
          Drying[i]  := True;
        end;
        {
        else
          with Soil do
            if (Wr_A[i] > Wsat[j] * 0.6) and (W[i] > Wsat[j] * 0.6) then
            begin
              dum := Water_Content(P[i], T[i], Wsat[j], zeta[j], Wr[j], j, True);
              if W[i] > dum then
              begin
                Drying[i]  := True;
                Wsat_A[i] := Soil.Wsat[j];
                Wr_A[i]   := Soil.Wr[j];
              end;
            end;
        }
      end;
    //if W[i] > Wsat[j] then
    //  W[i] := Wsat[j] - 0.0001;
    Wold[i]   := W[i];
  end;
  Pold[n+1] :=  P[n+1];
  //--- Flux for Graphical Output and Solute Transfer---//
  for i := 1 to n do
    ql[i] := -K[i] * ((P[i+1] - P[i]) / ((z[i+1] - z[i-1]) / 2) - Gravity * rho[i]);
  if (l_bc_type = 5) and (SinkType = 1) then
    ql[n] := - Sw[n] * dz[n];
  //---- Cumulative Values for Water Budget ----//
  CumEvap   := -qv[0] * dt + CumEvap;
  CumInfilt :=  ql[0] * dt + CumInfilt;
  CumDrain  :=  ql[n] * dt + CumDrain;
  if Abs(CumInfilt - CumDrain - CumEvap - CumTr) >  W_storage_ini then
    WCumRError :=  ABS((W_storage_new - W_storage_ini) - (CumInfilt - CumDrain - CumEvap - CumTr)) / Abs(CumInfilt - CumDrain - CumEvap - CumTr)
  else
    WCumRError :=  ABS((W_storage_new - W_storage_ini) - (CumInfilt - CumDrain - CumEvap - CumTr)) / W_storage_ini;
  W_storage_old := W_storage_new;
  if Vreservoir_max > 0 then
    Vreservoir := Min(Vreservoir_max, Max(0, Vreservoir + (0.01 * (q_rain * StorageEfficiency * Acatchment - q_irg * Acropping) - LossRate) * dt))
  else
    Vreservoir := 1;
  //-- moving pressure lower boundary for fingering study --//
  if MLB then
  begin
    ql[n] := 0;
    if CumDrain > Soil.Wsat[1] * dz[n] then
    begin
      j := Trunc(CumDrain / (Soil.Wsat[1] * dz[n]));
      CumDrain := CumDrain - j * (Soil.Wsat[1] * dz[n]);
      for i := 1 to j do
      begin
        P[n+i+1]    := P[n+1];
        Pold[n+i+1] := P[n+1];
        W[n+i+1]    := W[n];
        T[n+i+1]    := T[n];
        Tb[n+i]     := T[n];
        C[n+i+1]    := C[n+1];
        L[n+i+1]    := L[n+1];
        j2 := L[n+i+1];
        Wsat_A[n+i+1] := Soil.Wsat[j2];
        zeta_A[n+i+1] := Soil.zeta[j2];
        Wr_A[n+i+1]   := Soil.Wr[j2];
        Drying[n+i+1] := Drying[n+i];
        dz[n+i+1]   := dz[n+1];
        z[n+i+1]    := z[n+i] + dz[n+1];
        z_lb := z[n];
      end;
      n := n + j;
      z_lb := z[n];
    end;
  end;
  //----  at the compartment boundaries ----//
  W[n+1] := W[n];  W[0] := W[1]; // imaginary values.
  for i := 1 to n do
    Wb[i] := (W[i] * dz[i+1] / 2 + W[i+1] * dz[i] / 2) / ((dz[i] + dz[i+1]) / 2);
  Wb[n+1] := Wb[n];
  //---- calculation of mean pore water velocity for CRWM ---//
  if CRWM = 1 then
  begin
    Vmax := 0;
    for i := 1 to n do
    begin
      V[i] := ql[i] / Wb[i];
      if Abs(V[i]) > Vmax then
        Vmax := Abs(V[i]);
    end;
    dt_CRWM_next := 2 * Soil.dispersivity[1] / Abs(Vmax);
    if Vmax * (86400 / t_end) > 1000 then
      dt_CRWM_next := dt_CRWM_next / 20;
    if time_CRWM + dt_CRWM_next > int_rec * (counter + 1) then
      dt_CRWM_next := int_rec * (counter + 1) - time_ - 0.001;
  end;
except
  Application.ProcessMessages;
  Application.Terminate;
end;
end;


procedure TSimulator.SetScanningCurve(j, i: Integer; Drying_: Boolean);
var
  Wmd, Wmw, f : Double;
begin
  if Drying_ then
  begin
    {
    case Soil.RF[j] of
    1:if P[i] < 0 then
        Wsat_A[i] := FindRoot(Soil.zeta[j], Soil.Wsat[j], 0.000001, i, j, 2, True)
      else
        Wsat_A[i] := Soil.Wsat[j];
    2:if P[i] < 0 then
      begin
        Wmd := Soil.Water_Content(P[i], T[i], Soil.Wsat[j], Soil.zeta[j], Soil.Wr[j], Soil.RF[j], True);
        Wsat_A[i] := Soil.Wr[j] + (W[i] - Soil.Wr[j]) / (Wmd - Soil.Wr[j]) * (Soil.Wsat[j] - Soil.Wr[j])
      end
      else
        Wsat_A[i] := Soil.Wsat[j];
    end;
    }
    if P[i] < 0 then
    begin
      Wmd := Soil.Water_Content(P[i], T[i], Soil.Wsat[j], Soil.zeta[j], Soil.Wr[j], j, True);
      Wsat_A[i] := Soil.Wr[j] + (W[i] - Soil.Wr[j]) / (Wmd - Soil.Wr[j]) * (Soil.Wsat[j] - Soil.Wr[j])
    end
    else
      Wsat_A[i] := Soil.Wsat[j];
    zeta_A[i] := Soil.zeta[j];
    Wr_A[i]   := Soil.Wr[j];
  end
  else
  begin
    {
    case Soil.RF[j] of
    1:begin
        if P[i] < 0 then
          //zeta_A[i] := FindRoot(0.0001, Soil.Wsat[j] - 0.0001, 0.000001, i, j, 2, False)
          Wr_A[i] := FindRoot(-0.1, Soil.Wsat[j] - 0.0001, 0.000001, i, j, 2, False)
        else
          //zeta_A[i] := Soil.zeta[j];
          Wr_A[i] := 0;
        zeta_A[i] := Soil.zeta[j];
      end;
    2:if P[i] < 0 then
      begin
        Wmw := Soil.Water_Content(P[i], T[i], Soil.Wsat[j], Soil.zeta[j], Soil.Wr[j], Soil.RF[j], False);
        Wr_A[i] := Soil.Wsat[j] - (Soil.Wsat[j] - W[i]) / (Soil.Wsat[j] - Wmw) * (Soil.Wsat[j] - Soil.Wr[j])
      end
      else
        Wr_A[i] := Soil.Wr[j];
    end;
    }
    zeta_A[i] := Soil.zeta[j];
    if P[i] < 0 then
    begin
      Wmw := Soil.Water_Content(P[i], T[i], Soil.Wsat[j], Soil.zeta[j], Soil.Wr[j], j, False);
     // Wr_A[i] := Soil.Wsat[j] - (Soil.Wsat[j] - W[i]) / (Soil.Wsat[j] - Wmw) * (Soil.Wsat[j] - Soil.Wr[j])
      f := (Wmw - Soil.Wr[j]) / (Soil.Wsat[j] - Soil.Wr[j]);
      Wr_A[i] := (W[i] - f * Soil.Wsat[j]) / (1 - f);
      f := W[i] - Soil.Water_Content(P[i], T[i], Soil.Wsat[j], Soil.zeta[j], Wr_A[i], j, False);
    end
    else
      Wr_A[i] := Soil.Wr[j];
    Wsat_A[i] := Soil.Wsat[j];
  end;
end;

function TSimulator.GetScaledParameter(x: Double; i, j: Integer; Drying_: Boolean): Double;
begin
  if Drying_ then
    Result := W[i] - Soil.Water_Content(P[i], T[i], x, Soil.zeta[j], Soil.Wr[j], j, True)
  else
   // Result := W[i] - Soil.Water_Content(P[i], T[i], Soil.Wsat[j], x, Soil.Wr[j], j, False);
    Result := W[i] - Soil.Water_Content(P[i], T[i], Soil.Wsat[j], Soil.zeta[j], x, j, False);
end;
(*>>>>>>>>>>>>>>>>>>>>>>>   Solute  Movement  <<<<<<<<<<<<<<<<<<<<<<<<<*)

procedure TSimulator.CalculateSoluteMovement;
var
  i, i2, i3, j: Integer;
  Athd, Bthd, Cthd, t_prev, t_next, dispersivity_: Double;
  A_, B_, C_: Double;

  function TortuosityFactor(x: Double; j: Integer): Double;
  var
    i: Integer;// this techique enhances speed by 3%
  begin
    if (Segments = True) and (DrPrepared[j] = False) then
    begin
      Segments := False;
      DrPrepared[j] := True;
      for i := 0 to 20 do
        Drseg[j,i] := TortuosityFactor(i/20, j);
      Segments := True;
    end;
    if (Segments = False) or ((Segments = True) and (DrPrepared[j] = False)) then
    begin
      with Soil do
        Result :=  a_tau[j] * (1 - exp( - Power(b_tau[j] * (x + 0.001) * Wsat[j], c_tau[j]))) + d_tau[j] * x * Wsat[j];
    end
    else
    begin
      i := Trunc(x * 20);
      //i := Min(Max(i, 0), 20);
      Result := (Drseg[j,i] * ((i + 1) * 0.05 - x) + Drseg[j+1,i] * (x - i * 0.05)) / 0.05;
    end;
  end;

begin
  if CRWM = 1 then
  begin
    dz_ := (z[n+1] - z[n-1]) / 2;
    if not water then
    begin
      ql[0] := q_irg;
      ql[n] := q_irg;
    end;
    if ql[n] > 0 then
      qs[n] := -Wb[n] * Dm[n] * (C[n+1] - C[n]) / dz_ + ql[n] * C[n]
    else
      qs[n] := ql[n] * C[n+1]; // -Wb[n] * Dm[n] * (C[n+1] - C[n]) / dz_ ;
    if ql[0] > 0 then
      qs[0] := q_irg * C_irg
    else
      qs[0] := 0;
    if dumint <> 12345 then
    begin
      Cum_qs_bot := Cum_qs_bot + qs[n]* dt;
      Cum_qs_top := Cum_qs_top + qs[0]* dt;
      dumint := 0;
    end
    else
      dumint := 0;
    dt_CRWM := time_ - time_CRWM;
    if dt_CRWM < dt_CRWM_next then
      Exit
    else
      time_CRWM := time_;
  end;
 //---- upper boundary ----//
  if s_u_bc_type = 2 then
  begin
    i := Cirg_done;
    while t_c_irg[i] < time_ do
      i := i + 1;
    Cirg_done := i;
    t_prev := t_c_irg[i-1];
    t_next := t_c_irg[i];
    if t_next > t_prev then
      C_irg := (Cirg[i-1] * ( t_next - time_ ) + Cirg[i]* ( time_ - t_prev )) / ( t_next - t_prev );
  end;
  qs[0] := q_irg * C_irg;
 //---- lower boundary ----//
  if s_l_bc_type = 2 then
  begin
    i := Cbot_done;
    while t_c_bot[i] < time_ do
      i := i + 1;
    Cbot_done := i;
    t_prev := t_c_bot[i-1];
    t_next := t_c_bot[i];
    if t_next > t_prev then
      C_bot := (Cbot[i-1] * ( t_next - time_ ) + Cbot[i] * ( time_ - t_prev )) / ( t_next - t_prev );
  end;
  case s_l_bc_type of
  1: C[n+1] :=  C_bot;
  2: C[n+1] :=  C_bot;
  3: C[n+1] :=  C[n];
  end;
 //------------------------
  Wb[0] := w[1];
  Wb[n+1] := W[n];
  Tb[0] := Tb[1] ;
  ql[n+1] :=  ql[n];
  NegativeDCwarning := 0;
  Vmax := 0;
  if (water) or ((Mode = 2) and (time_ = 0)) then
    for i := 1 to n do
    begin
      j := L[i];
      if (water) or (Mode = 2) then
      begin
        V[i] := ql[i] / Wb[i];
        with Soil do
        begin
          //-- molecular Diffusion coefficient --//
          Di[i] := Diw0 * TortuosityFactor(Max(0, Wb[i]) / Wsat[j], j);
          //-- mechanical Dispersion coefficient--//
          if (CDE_surface_modification) and (z[i] < 2) then
            dispersivity_ := dispersivity[j] * 0.5//0.333 //(0.333 * z[i] + 0.333) //(0.375 * z[i] + 0.25)
          else
            dispersivity_ := dispersivity[j];
          Dm[i] := dispersivity_ * ABS(V[i]);
          //--------------------------------------//
        end;
      end
      else
      begin
        Dm[i] := Ds;
        V[i]  := q_irg / 0.3;
      end;
      if Abs(V[i]) > Vmax then
        Vmax := Abs(V[i]);
      //-- collected Dispersion coefficient --//
      dz_ := (z[i+1] - z[i-1]) / 2;
      Dc[i] := Dm[i] + Di[i] - (ABS(V[i]) * dz_ / 2) - V[i] * V[i] * dt / 8 / Wb[i] * (Wb[i] - Wbold[i]);
      if (Dc[i] < 0) and (CRWM <> 1) then
      begin
        Dc[i]  := 0.000001;
       // NegativeDCwarning := 1;
       // time_ := t_end + 1;
      end;
      Wbold[i] := Wb[i];
      //-- solute flux --//
     // if ql[i] > 0 then
     //   qs[i] := -Wb[i] * Dc[i] * (C[i+1] - C[i]) / dz_ + ql[i] * C[i]
     // else
     //   qs[i] := -Wb[i] * Dc[i] * (C[i+1] - C[i]) / dz_ + ql[i] * C[i+1];
    end;
  if CRWM <> 1 then
  //=====  Convection Dispersion Equation =====//
  begin

    //-- Implicit FDM --//
    for i := 1 to n do
    begin
      //-- non-equilibrium crystallization --//
      if (alpha_crys < 0.01) and (Crystl[i] > 0) and (water) then  // and (ql[i] > 0)
      begin
        Ss[i] := alpha_crys * t_unit * W[i] * (Cmax - C[i]);
        Crystl[i] := Crystl[i] - Ss[i] * dt;
      end
      else
      begin
        //--- Solute uptake by plant roots ---//
        Ss[i] := - kpu * C[i] * Swr[i];
        cum_s_uptake := cum_s_uptake - Ss[i] * dz[i] * dt;
        // Ss[i] := 0;
        //-------------------------------------//
      end;
    end;
    Ss[1] := Ss[1] + qs[0] / dz[1];
    for i := 0 to n do
      if ql[i] > 0 then
        pi[i] := 1
      else
        pi[i] := 0;
    for i := 1 to n do
    begin
      A_ := (2 * Wb[i-1] * Dc[i-1] / (z[i]   - z[i-2]) + ql[i-1] * pi[i-1]) / dz[i];
      B_ := (2 * Wb[i]   * Dc[i]   / (z[i+1] - z[i-1]) + 2 * Wb[i-1] * Dc[i-1] / (z[i] - z[i-2]) + ql[i] * pi[i] - ql[i-1] *  (1 - pi[i-1])) / dz[i];
      C_ := (2 * Wb[i]   * Dc[i]   / (z[i+1] - z[i-1]) - ql[i] * (1 - pi[i])) / dz[i];
      Ath[i]:=  ETAs * A_  ;
      Bth[i]:= -ETAs * B_ - W[i] / dt ;
      Cth[i]:=  ETAs * C_ ;
      Athd := -(1 - ETAs) * A_ * C[i-1];
      Bthd := ((1 - ETAs) * B_ - Wolds[i] / dt) * C[i];
      Cthd := -(1 - ETAs) * C_ * C[i+1];
      Dth[i] := Athd + Bthd + Cthd - Ss[i] ;
    end;
    Dth[n] := Dth[n] - Cth[n] * C[n+1];  // 01/06/01 changed

    SolveTheCoefficientMatrix;
    Crystal := 0;
    for i := 1 to n do
    begin
      C[i] := x[i];
      Wolds[i] := W[i];
    //  M[i] := W[i] * C[i];
      //-- equilibrium crystallization --//
      if (alpha_crys > 0.01) and ((C[i] > Cmax) or (Crystl[i] > 0)) and (water) then
      begin
        Crystl[i] := Crystl[i] - W[i] * (Cmax - C[i]);
        if Crystl[i] > 0 then
          C[i] := Cmax
        else
        begin
          C[i] := Cmax + Crystl[i] / W[i];
          Crystl[i] := 0;
        end;
      end;
      Crystal := Crystal + Crystl[i] * dz[i];//(M[i] - W[i] * Cmax) * dz[i];
      M[i] := W[i] * C[i] + Crystl[i];
    end;
    //-- mass conservation --//
    {
    for i := 1 to n do
      begin
      M[i] := -(qs[i] - qs[i-1]) / dz_ * dt + M[i];
      C[i] := M[i] / W[i];
      end;
    }
  end
  else
  begin
    //=======  Convective Random Walk Model  =======//
    //-- determination of a,Ym and Y0  --//
    Dm[n+1] := Dm[n];
    for i := 1 to n do
      begin
      //-- v modification (Uffink,1985) --
      // v[i] := v[i] + (Dm[i+1] - Dm[i]) / dz[i];
      //----------------------------------
      if v[i] < 0 then SIGN[i] := -1 else SIGN[i] := 1;
      mean_[i] := ABS(v[i]) * dt_CRWM;
      //STD := Sqrt(2 * Dm[i] * dt_CRWM);
      Xmax[i] := 6 * Soil.dispersivity[L[i]] + 1.5 * mean_[i];
      Ym[i]   := 4 * mean_[i] / Sqr(Xmax[i]);
      Y0[i]   := (2 * Xmax[i] - 4 * mean_[i]) / (mean_[i] * Xmax[i])
      end;
    //-- moving of particle --//
    for i := 1 to NumP do
      if (zp[i] > 0) and (zp[i] < z_lb) then
        begin
        OneStepDisp := GetOneStepDisp(ncp[i]);
        if water then
          for i2 := 1 to 50 do
            begin
            RN := -2; //=  6 / 3
            for i3 := 1 to 4 do RN := RN + Random; // 4 = 12 / 3
            //'  RN0 := RND[1]
            //'  RN := (RN0 - .5) * 12 ^ .5
            OneStepDiff := RN * Sqrt(6 * Di[ncp[i]] * dt_CRWM / 50); // 6 = 3 * 2
            zp[i] := zp[i] + OneStepDisp / 50 + OneStepDiff;
            if zp[i] <= 0 then zp[i] := 0.01;// else if zp[i] >= z_lb then zp[i] := z_lb - 0.01;
            if (Abs(V[n]* (86400 / t_unit)) < 0.5) and (zp[i] > z_lb) then zp[i] := z_lb - (zp[i] - z_lb);  // reflective lower boundary
            if (ql[n] < 0) and (zp[i] > z_lb) then zp[i] := z_lb - (zp[i] - z_lb);  // reflective lower boundary
            end
        else
          zp[i] := zp[i] + OneStepDisp;
        end;
    //--- making of particle ---//
    // lower boundary
    Integler_lb := - C_bot * ql[n] * dt_CRWM + Integler_lb;
    //if Integler_lb < 0 then Integler_lb := 0;
    if Integler_lb > MPP then
    begin
      mean_[n] := ABS((V[n] + Vnbefore) / 2) * dt_CRWM;
      //STD := Sqrt(2 * Dm[i] * dt_CRWM);
      Xmax[n] := 6 * Soil.dispersivity[NumLayer] + 1.5 * mean_[n];
      Ym[n]   := 4 * mean_[n] / Sqr(Xmax[n]);
      Y0[n]   := (2 * Xmax[n] - 4 * mean_[n]) / (mean_[n] * Xmax[n]);
      Vnbefore := V[n];
      dumint := NumP;
      NumP := NumP + Trunc(-C_bot * ql[n] * dt_CRWM / MPP);
      for i := dumint + 1 to NumP do
      begin
        ncp[i] := n;
        repeat
          zp[i] := z_lb + Xmax[n] * Random;
          OneStepDisp := GetOneStepDisp(n);
        until zp[i] + OneStepDisp < z_lb - 0.000001;
        zp[i] := zp[i] + OneStepDisp;
      end;
      Integler_lb := Integler_lb - MPP * INT(-C_bot * ql[n] * dt_CRWM / MPP);
    end;
    // surface boundary
    Integler_ub := C_irg * ql[0] * dt_CRWM + Integler_ub;
    if Integler_ub > MPP then
      mean_[1] := ABS((V[1] + V1before) / 2) * dt_CRWM;
      //STD := Sqrt(2 * Dm[i] * dt_CRWM);
      Xmax[1] := 6 * Soil.dispersivity[1] + 1.5 * mean_[1];
      Ym[1]   := 4 * mean_[1] / Sqr(Xmax[1]);
      Y0[1]   := (2 * Xmax[1] - 4 * mean_[1]) / (mean_[1] * Xmax[1]);
      V1before := V[1];
    begin
      dumint := NumP;
      NumP := NumP + Trunc(C_irg * ql[0] * dt_CRWM / MPP);
      for i := dumint + 1 to NumP do
      begin
        ncp[i] := 1;
        repeat
          zp[i] := - Xmax[1] * Random;
          OneStepDisp := GetOneStepDisp(1);
        until zp[i] + OneStepDisp > 0.000001;
        zp[i] := zp[i] + OneStepDisp;
      end;
      Integler_ub := Integler_ub - MPP * INT(C_irg * ql[0] * dt_CRWM / MPP);
    end;
    //---------------------------
    for i := 1 to n do
      M[i] := 0;
    for i := 1 to NumP do
      if (zp[i] >= 0) and (zp[i] <= z_lb) then
      begin
        if com_ratio = 1 then
          ncp[i] := Trunc(zp[i]/dz[1]) + 1
        else
          ncp[i] := Ceil(LogN(com_ratio, 1 + zp[i] * (com_ratio - 1) / dz[1]));
        M[ncp[i]] := M[ncp[i]] + MPP / dz[ncp[i]];
      end;
    for i := 1 to n do
      C[i] := M[i] / W[i];
    //--- crystallization ---//
    i := 0;
    if C[1] > 310 then
      while M[1] / W[1] > 310 do
      begin
        i := i + 1;
        if (ncp[i] = 1) and (zp[i] > 0) then
        begin
          M[1] := M[1] - MPP / dz[1];
          zp[i] :=  -1;
          Crystal := Crystal + MPP;
        end;
      end
    else if Crystal > 0 then
    //--- dissolution ---//
      while M[1] / W[1] < 310 do
      begin
        i := i + 1;
        if i > NumP then break;
        if zp[i] = -1 then
        begin
          M[1] := M[1] + MPP / dz[1];
          zp[i] :=  0.01;
          Crystal := Crystal - MPP;
        end;
      end;
    //---------------------//
    C[1] := M[1] / W[1];
    if not water then
    begin
      dt_CRWM_next := 2 * Soil.dispersivity[1] / Abs(Vmax);
      if time_ + dt_CRWM_next > int_rec * (counter + 1) then
        dt_CRWM_next := int_rec * (counter + 1) - time_ - 0.001;
    end;
  end;
//===================================================================
  dz_ := (z[n+1] - z[n-1]) / 2;
  if CRWM <> 1 then
    if ql[n] > 0 then
      qs[n] := -Wb[n] * Dc[n] * (C[n+1] - C[n]) / dz_ + ql[n] * C[n]
    else
      qs[n] := -Wb[n] * Dc[n] * (C[n+1] - C[n]) / dz_ + ql[n] * C[n+1];
 //---- Mass Balance ---//
  S_storage := 0;
  for i := 1 to n do
    S_storage := S_storage + M[i] * dz[i];
  if CRWM <> 1 then
  begin
    Cum_qs_bot := Cum_qs_bot + qs[n]* dt;
    Cum_qs_top := Cum_qs_top + qs[0]* dt;
  end;
  dum := Max(S_storage_ini, S_storage);
  dum := Max(dum, Abs(Cum_qs_bot));
  if dum > 0 then
    SCumRError := ABS((S_storage - S_storage_ini) - (Cum_qs_top - Cum_qs_bot - cum_s_uptake)) / dum
  else
    SCumRError := 0;
end;

(*>>>>>>>>>>>>>>>>>>>>>>>   Heat Movement  <<<<<<<<<<<<<<<<<<<<<<<<<*)

procedure TSimulator.CalculateHeatMovement;
var
  i, j: Integer;
  Athd, Bthd, Cthd, Tfp: Double;

  function ThermalCondictivity(x: Double; j: Integer): Double;
  var
    i: Integer;
  begin
    if (Segments = True) and (TherConPrepared[j] = False) then
    begin
      Segments := False;
      TherConPrepared[j] := True;
      for i := 0 to 20 do
        Khseg[j,i] := ThermalCondictivity(i /20, j);
      Segments := True;
    end;
    if (Segments = False) or ((Segments = True) and (TherConPrepared[j] = False)) then
      Result := t_unit * (Soil.Akh[j]  + Soil.Bkh[j] * x) - (Soil.Akh[j] - Soil.Dkh[j]) * exp(-Soil.Ckh[j] * Power(x, Soil.Ekh[j]))
    else
    begin
      i := Trunc(x * 20);
      //i := Min(Max(i, 0), 20);
      Result := (khseg[j,i] * ((i + 1) * 0.05 - x) + khseg[j+1,i] * (x - i * 0.05)) / 0.05;
    end;
  end;
begin
  with Soil do
    for i := 1 to n + 1 do
    begin
      j := L[i];
      Chnew[i] := 2.13 * (1 - Wsat[j]) + 4.18 * W[i] + 2 * Wice[i];
      kh[i] := ThermalCondictivity(Max(0, (Wb[i] + (Wice[i] + Wice[i+1])) / Wsat[j]), j);
    end;

  if h_l_bc_type = 2 then
    kh[n] := 0;
 //-- Lower Boundary Condition --//
  qh[n] := - kh[n] * (T[n+1] - T[n]) / ((z[n+1] - z[n-1]) / 2) + ql[n] * 4.18 * (pi[n] * T[n] + (1 - pi[n]) * T[n+1]) + ql[n] * Lhf;
 //-- Latent Heats --//
  for i := 1 to n do
  begin
    Swi[i] := 0;
    if T[i] < 278 then
    begin
      Tfp := 273.2 + kfpd * Pw[i];
      //if time_ > 2200 then
      //  ShowMessage(FloatToStr(Tfp));
      if ((T[i] < Tfp) and (W[i] > 0.01)) or ((T[i] > Tfp) and (Wice[i] > 0)) then
      begin
        Swi[i] :=  0.001 * (Tfp - T[i]);
        if Wice[i] + W[i] > Soil.Wsat[L[i]] - 0.02 then
          Swi[i] := 0;  // 09/04/07
        if Wice[i] + Swi[i] * dt < 0 then
        begin
          Swi[i] := - Wice[i] / dt;
          Wice[i] := 0;
        end
        else
        begin
          if W[i] - Swi[i] * dt < 0.01 then
            Swi[i] := (W[i] - 0.01) / dt;
          Wice[i] := Wice[i] + Swi[i] * dt;
        end;
      end;
    end;
    LH[i] := (597 - 0.6 * (T[i] - 273.2)) * 4.18; // J/cm3
    Sh[i] := - LH[i] * (qv[i] - qv[i-1]) / dz[i] - LOSS * (T[i] - Tini) + Swi[i] * Lhf - Swr[i] * 4.18 * T[i];  // J/cm3/s
  end;
 //------- Building of Coefficient Matrix -------//
  Sh[1] := Sh[1] + qh[0] / dz[1];
  for i := 0 to n do
    if ql[i] > 0 then pi[i] := 1 else pi[i] := 0;
  Ch[0] := Ch[1]; Ch[n+1] := Chnew[n+1];//tentative
  //if h_l_bc_type = 1 then
    h[n+1] :=  Tbot * Chnew[n+1];
  for i := 1 to n do
  begin
    Ath[i+1] :=  ETAh / (Chnew[i]  * dz[i+1])* (2 * kh[i]  / (z[i+1] - z[i-1]) + ql[i] * 4.18 * pi[i]);
    Bth[i]   := -ETAh / (Chnew[i]  * dz[i])  * (2 * kh[i]  / (z[i+1] - z[i-1]) + 2 * kh[i-1] / (z[i] - z[i-2]) + ql[i] * 4.18 * pi[i] - ql[i-1] * 4.18 * (1 - pi[i-1])) - 1 / dt;
    Cth[i]   :=  ETAh / (Chnew[i+1]* dz[i])  * (2 * kh[i]  / (z[i+1] - z[i-1]) - ql[i] * 4.18 * (1 - pi[i]));
    Athd := -(1 - ETAh) / (Ch[i-1]* dz[i])   * (2 * kh[i-1]/ (z[i]   - z[i-2]) + ql[i-1] * 4.18 * pi[i-1]) * h[i-1];
    Bthd := ((1 - ETAh) / (Ch[i]  * dz[i])   * (2 * kh[i]  / (z[i+1] - z[i-1]) + 2 * kh[i-1] / (z[i] - z[i-2]) + ql[i] * 4.18 * pi[i] - ql[i-1] * 4.18 * (1 - pi[i-1])) - 1 / dt) * h[i];
    Cthd := -(1 - ETAh) / (Ch[i+1]* dz[i])   * (2 * kh[i]  / (z[i+1] - z[i-1]) - ql[i] * 4.18 * (1 - pi[i])) * h[i+1];
    Dth[i] := Athd + Bthd + Cthd - Sh[i];
  end;
  if h_l_bc_type = 1 then
    Dth[n] := Dth[n] - Cth[n] * h[n+1]; // 01/06/01 changed
  if abc_type = 1 then
    qh[0] := 200 * (Tini - T[1]) * Ch[0] * dz[1] ;
  heatcalc := 1;
 //-----//
  SolveTheCoefficientMatrix;
 //-----//
  heatcalc := 0;
  for i := 1 to n do
  begin
    h[i] := x[i];
    Tnew := h[i] / Chnew[i];
    Ch[i] :=  Chnew[i];
    if ABS(Tnew - T[i]) > maxchangeT then
      maxchangeT := ABS(Tnew - T[i]);
    P[i] := P[i] * (1 + 0.0068 * (T[i] - Tnew)); //exp(0.0068 * (T[i] - Tnew));  2% up
    Pold[i] := P[i];
    T[i] := Tnew;
  end;
 //-- Boundary Value --//
  for i := 1 to n do
    Tb[i] := (T[i] * dz[i+1] / 2 + T[i+1] * dz[i] / 2) / ((dz[i] + dz[i+1]) / 2);
 //--- Energy Balance ---//
  E_storage := E_storage_old;
  E_storage := 0;
  for i := 1 to n do
    E_storage := E_storage + (h[i] + W[i] * Lhf) * dz[i];
  TotalLoss := 0;
  for i := 1 to n do
  begin
    TotalLoss := TotalLoss + LOSS * (T[i] - Tair) * dz[i];
    CumSHT := - Swr[i] * 4.18 * T[i] * dz[i] * dt + CumSHT;
  end;
  //rerrore := ABS((EStoRAGENEW - EStoRAGEOLD) - (qh[0] - qh[n] - toTALHLOSS + qv[0] * LH[1)) * dt) / EStoRAGENEW 'Each Step Relative Error
  //eserrore := ABS((EStoRAGENEW - EStoRAGEOLD) - (qh[0] - qh[n] - toTALHLOSS + qv[0] * LH[1)) * dt) / ((qh[0] - qh[n] - toTALHLOSS + qv[0] * LH[1)) * dt) 'Each Step Error
  CumSHT := (qh[0] - qv[0] * 4.18 * T[1] - ql[0] * 4.18 * (Tair - 1)) * dt   + CumSHT; //Cumulative sensible heat transfer in J /cm2
  CumLHT := qv[0] * LH[1] * dt + CumLHT;   // Cumulative latent heat transfer
  CumHLB := qh[n] * dt + CumHLB;           // Cumulative heat loss from the bottom
  CumHLS := TotalLoss * dt + CumHLS;       // Cumulative heat loss from the side
  // investigate why Tr is not required to include
  CumCHL := ((qv[0] - Tr * 0) * 4.18 * T[1] + ql[0] * 4.18 * (Tair - 1) + (ql[0] + qv[0] - Tr) * Lhf) * dt + CumCHL; // Cumulative convective heat loss from the surface
  ECumRError := ABS((E_Storage - E_Storage_ini) - (CumSHT + CumLHT + CumCHL - CumHLB - CumHLS)) / E_Storage_ini;
  //if ABS(ECumRError) > .1 then PRINT "Cumulative relative error for heat transfer is too large!,"; eCumrerror
end;

procedure TSimulator.CalculateHeatMovementInSnow;
var
  khsnow, Ch, Sw, dt_snow: Double;
  i, j, k: Integer;
  procedure GetHeatFluxAtSnowSurface;
  begin
    if Tair < 273.2 then
      dum := 1.93
    else
      dum := 4.18;
    if Wstorage_in_snow = 0then
      Tsnow[iss] := Tair - 1;
    qh_snow[iss-1] := Rn + (Tair - Tsnow[iss]) * 0.0012 / ( 0.5 * ra / t_unit) + q_rain * dum * (Tair - 1);
    qh[0] := 0.2 * kh[1] * (Tsnow[NumElmSnow] - T[1]) / 1;// + q_rain * 4.18 * 273.2;
    qh_snow[NumElmSnow] := qh[0];
  end;
  
begin
  {$ifdef snow}
  if ((Tair < 273.2) and (q_rain > 0)) or (Wstorage_in_snow > 0) then
  begin
    dt_snow := 0.01 * dt;
    dum := dz_snow[iss];
    if Tair < 273.2 then
    begin
      dz_snow[iss] := dz_snow[iss] + q_rain * dt / 0.03;
      if dz_snow[iss] > 2 then
      begin
        Dec(iss);
        dz_snow[iss] := dz_snow[iss+1] - 1;
        dz_snow[iss+1] := 1;
        rho_snow[iss] := 0.03;
        rho_snow[iss+1] := (rho_snow[iss+1] * dum - rho_snow[iss] * dz_snow[iss] + q_rain * dt) / dz_snow[iss+1];
        Hsnow[iss]   := Tsnow[iss]   * 1.93 * rho_snow[iss];
        Hsnow[iss+1] := Tsnow[iss+1] * 1.93 * rho_snow[iss+1];
        Tsnow[iss] := Tsnow[iss+1];
        qh_snow[iss-1] := qh_snow[iss];
      end
      else
        rho_snow[iss] := (rho_snow[iss] * dum + q_rain * dt) / dz_snow[iss];
      if q_rain > 0 then
        time_last_snow := time_;
    end
    else
      rho_snow[iss] := (rho_snow[iss] * dum + q_rain * dt) / dz_snow[iss];
    GetHeatFluxAtSnowSurface;
    if (iss = NumElmSnow) and (dz_snow[NumElmSnow] < 0.5) and (rho_snow[NumElmSnow] < 0.5) then
    begin
      Tsnow[NumElmSnow] := 0.5 * (Tair + T[1]);
      Hsnow[NumElmSnow] := Tsnow[NumElmSnow] * 1.93 * rho_snow[NumElmSnow];
    end
    else
      for k := 1 to 100 do
      begin
        for i := iss to NumElmSnow - 1 do
        begin
          khsnow := t_unit * (0.0005 + 0.0095 * (0.5 * (rho_snow[i+1] + rho_snow[i]) - 0.03) / 0.047);
          qh_snow[i] := - 2 * khsnow * (Tsnow[i+1] - Tsnow[i]) / (dz_snow[i+1] + dz_snow[i]);
        end;
        for i := iss to NumElmSnow do
        begin
          Hsnow[i] := Hsnow[i] + dt_snow * (qh_snow[i-1] - qh_snow[i]) / dz_snow[i];
          Ch := 1.93 * rho_snow[i]; //J/cm3/K
          Tsnow[i] := Hsnow[i] / Ch;
          if Tsnow[i] > 273.2 then
          begin
            Sw := 0.001 * (Tsnow[i] - 273.2);
            Hsnow[i] := Hsnow[i] - dt_snow * Sw * Lhf;
            dum := dz_snow[i];
            dz_snow[i] :=  dz_snow[i] - dt_snow * Sw * dz_snow[i] / rho_snow[i];
            if  rho_snow[i] < 0.8 then
            begin
              rho_snow[i] := rho_snow[i] * dum / dz_snow[i];
              Ch := 1.93 * rho_snow[i];
              Tsnow[i] := Hsnow[i] / Ch;
            end
            else
            begin
              if dz_snow[i] < 0 then
                Inc(iss);
              Ch := 1.93 * rho_snow[i];
              Tsnow[i] := Hsnow[i] / Ch;
              j := i;
              repeat
                j := j + 1;
              until (rho_snow[j] < 0.8) or (j = NumElmSnow + 1);
              if j <= NumElmSnow then
                rho_snow[j] := rho_snow[j] + dt_snow * Sw * dz_snow[i]
              else
                ql[0] :=  Sw * dz_snow[i];
            end
          end;
        end;
        GetHeatFluxAtSnowSurface;
      end;
    Wstorage_in_snow := 0;
    for i := iss to NumElmSnow do
      Wstorage_in_snow := Wstorage_in_snow + rho_snow[i] * dz_snow[i];
    z_snow[NumElmSnow] := - dz_snow[NumElmSnow] / 2;
    for i := NumElmSnow - 1 downto iss do
      z_snow[i] := z_snow[i+1] - 0.5 * (dz_snow[i+1] + dz_snow[i]);
  end;
 {$endif}
end;

//==============================================================================================
procedure TSimulator.DetermineSpaceIncrements;
var
  c: Double;
  i, j: Integer;
begin
  z_border[NumLayer] := z_lb;
  com_ratio :=  (z_lb - z[1]) / (z_lb - dz_bot);
  z[0] := 0;
  if com_ratio = 1 then
  begin
    n := Trunc(z_lb / z[1] + 0.00001);
    for i := 1 to n do
    begin
      dz[i] := z[1];
      z[i] := z[i-1] + dz[i];
      z_c[i] := (z[i] + z[i-1]) / 2;
    end;
  end
  else
  begin
    n := Trunc(ln(z_lb * (com_ratio - 1) / z[1] + 1) / ln(com_ratio) + 0.00001);
    c := FindRoot(-0.01, 0.01, 0.00000001, n, 1, 1, True);
    for i := 1 to n do
    begin
      dz[i] := z[1] * Power(com_ratio + c * i, i - 1);
      z[i] := z[i-1] + dz[i];
      z_c[i] := (z[i] + z[i-1]) / 2;
    //  z[n] := z_lb;
    //  dz[n] := z[n] - z[n-1];
    end;
  end;
  {
  dum := 0;
  for i := 1 to n do
    dum := dum + dz[i];
  ShowMessage(FloatToStr(dum));
  }
  z_c[n] := (z[n] + z[n-1]) / 2;
  z[n+1]  := z[n] + dz[n];
  dz[n+1] := dz[n];
  //-- layer identification --//
  for i := 1 to n - 1 do
  begin
    j := 1;
    while z[i] > z_border[j] + 0.000001 do
      j := j + 1;
    L[i] := j;
  end;
  //--------------------------//
  L[0] := L[1];
  L[n] := L[n-1];
  L[n+1] := L[n-1];
  dz[0] := dz[1];
  z[-1] := 0; //imaginary value
  //--- observation point ---//
  for j := 1 to NumObs do
  begin
    i := 0;
    while z_c[i] <= z_obs[j] do
      i := i + 1;
    NCtime[j] := i;  //NCtime :compartment number containing the point of continuous measurement
  end;
  //---- albedo ----//
  i := 0;
  while (z[i] < Soil.ThickAlb[1]) and (i < n) do
    i := i + 1;
  NCalb := i;
  //--- low peameable membrane ---//
  if Z_membrane > 0 then
  begin
    i := 1;
    while z_c[i] < Z_membrane do
      i := i + 1;
    i_mb := i;
  end
  else
    i_mb := 0;
end;

function TSimulator.GetResidual(x: Double; n: Integer): Double;
var
  Sn: Double;
  i: Integer;
begin
  com_ratio  := Power(dz_bot / z[1], 1 / (n - 1)) - x * n;
  Sn := 0;
  for i := 1 to n do
    Sn := Sn + z[1] * Power(com_ratio + x * i, i - 1);
  Result := z_lb - Sn;
end;

function TSimulator.NextValue: Double;
var
  i: Integer;
begin
  S := TrimLeft(S);
  i := Min(Pos(' ', S), Pos(#9, S));
  if i = 0 then
    i := Pos(' ', S);
  if i = 0 then
    i := Pos(#9, S);
  //if (i = 0) and (DecimalSeparator = '.') then
  //  i := Pos(',', S);
  if i = 0 then
    i := 100;
  Result := StrToFloat(Copy(S, 1, i - 1));
  S := Copy(S, i, 100000);
end;

function TSimulator.NextString: String;
var
  i: Integer;
begin
  S := TrimLeft(S);
  i := Min(Pos(' ', S), Pos(#9, S));
  if i = 0 then
    i := Pos(' ', S);
  if i = 0 then
    i := Pos(#9, S);
  if i = 0 then
    i := 100;
  Result := Copy(S, 1, i - 1);
  S := Copy(S, i, 100000);
end;

//----------------------------------------------------------
procedure TSimulator.SolveTheCoefficientMatrix;
var i: Integer;
begin
  // solving symmetric and tridiagonal matrix using the Thomas algorithm
  for i := 1 to n - 1 do
  begin
    dum := Ath[i+1] / Bth[i];
    Bth[i+1] := Bth[i+1] - dum * Cth[i];
    Dth[i+1] := Dth[i+1] - dum * Dth[i];
  end;
  x[n] := Dth[n] / Bth[n];
  for i := n - 1 downto 1 do
    x[i] := (Dth[i] - Cth[i] * x[i+1]) / Bth[i];
end;
//----------------------------------------------------------
function TSimulator.GetOneStepDisp(i: Integer): Double;
var RN: Double;
begin
  RN := Random;
  if RN < Ym[i] / 2 * (Xmax[i] - mean_[i]) then
    Result := SIGN[i] * (Xmax[i] - Sqrt(2 * RN * (Xmax[i] - mean_[i]) / Ym[i]))
  else
    begin
    dum := Sqr(Y0[i]) - 2 * (Ym[i] - Y0[i]) / mean_[i] * (RN - 1);
    if dum < 0.0001 then
      ShowMessage('!');
    Result := SIGN[i] * mean_[i] * (-Y0[i] + Sqrt(dum)) / (Ym[i] - Y0[i]);
    end;
end;
//----------------------------------------------------------
function TSimulator.GetMilliTime:Double;
var
  Present: TDateTime;
  hour,min,Sec, MSec: Word;
begin
  Present :=  Now;
  DecodeTime(Present,hour,min,sec,msec);
  GetMilliTime := hour*3600+min*60+sec+msec/1000
end;
//----------------------------------------------------------
function TSimulator.SetLengthRJ(dumstr: String; dumint: Shortint): String;
var i, n: Shortint;
begin
  n := Length(dumstr);
  for i := 1 to dumint - n do
    dumstr := ' '+ dumstr;
  Result := dumstr;
end;
//----------------------------------------------------------

procedure TSimulator.EvaluateDailyScaleParameters;
var
  i: Integer;
  a, z_, Stress, Checker: Double;

  function f(a, b, c: Double): Double;
  begin
    Result := a * (1 - exp(b * (CumTr + CumTr_ini))) + c;
  end;

begin
  drt  := f(adrt, bdrt, cdrt);
  a := (brt + 1) * Power(drt, - brt - 1);
  Checker := 0;
  for i := 1 to n do
  begin
    z_ := 0.5 * (z[i] + z[i-1]) - zr0;
    if (z_ < drt) and (z_ > 0) then
      NRD[i] := a * Power(drt - z_, brt)
    else
      NRD[i] := 0;
    Checker := Checker + NRD[i] * dz[i];
  end;
  for i := 1 to n do
    NRD[i] := NRD[i] / Checker;
  LAI := f(aLAI, bLAI, 0);
  Kc  := f(aKc, bKc, cKc) - dKc * Power (CumTr_ini + CumTr, eKc);  // CumTr_ini was added on 2020/3/15
  KRshort := exp(-0.82 * LAI);
  KRl     := 0.2 * (1 - exp(-0.5 * LAI));
  DIY := StartDate + DayCounter - Jan1;
  dec_sun := 23.45 * 3.142 / 180 * sin(2 * 3.142 * (284 + DIY) / 365);  //declination of the sun
  if Wstorage_in_snow > 0 then
  begin
    albedo := 0.9 - 0.5 / (1 + 499 * exp(-0.1 * (time_last_snow - time_) * t_unit / 86400)) ;  //499 = (0.5 - 0.001) / 0.001
    Stress := 0; // in Pa
    for  i:= iss + 1 to NumElmSnow do
    begin
      Stress :=  Stress + 98.1 * rho_snow[i-1] * dz_snow[i-1]; // 1 gf/cm2 = 10 kgf/m2 = 98.1 Pa
      dum := dz_snow[i] * rho_snow[i];
      dz_snow[i] := dz_snow[i] + 86400 * Stress * 1000 * rho_snow[i] / exp(3440000 * 1000 * rho_snow[i] - 0.096 * (Tsnow[i] - 273.2));
      rho_snow[i] := dum / dz_snow[i];
    end;
    
  end;
end;

end.
